#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "data.h"
#include <errno.h>
#include <glib.h>
#include <math.h>
#include <stdlib.h>
#include <assert.h>
#include <time.h>

#ifdef WIN32
#define OPEN_FILE_FLAG O_RDONLY | O_BINARY
#else 
#define OPEN_FILE_FLAG O_RDONLY
#endif

G_LOCK_DEFINE_STATIC (processing_flag);
static volatile int processing_flag = 1;

void extract_line(char *, int, float *);

extern int errno;
extern int WINDOW_SIZE;
static int file_list_loaded=-1,com_list_loaded=-1;
static char text[256];

extern GtkWidget *w2DView;
extern GtkWidget *wControl;
extern GtkWidget *dlgDetGeo;
extern GtkWidget *daColorTable;
extern GtkWidget *da2DView;
extern GtkWidget *etGamma,*etZMax,*etZMin;
extern GtkWidget *ckbtnDetRec;
extern GtkWidget *etSamNormal,*etIncidentAng;
extern GtkWidget *et1DGrid;
extern DataSet *dataset;
extern stPreference *globalPref;
extern GtkWidget *tvInfo,*tvFileList;
extern GtkTextBuffer *tbuf_info;
extern GtkWidget *lbStatusMsg;
extern GtkSpinButton *sbParaSet, *sbCurSet, *sbSet1, *sbSet2;
extern GtkWidget *etScaleFactor, *etConst, *etSetComments;
extern GtkWidget *etLineProfileCx,*etLineProfileCy,*etLineProfileLength;
extern GtkWidget *etLineProfileOrient,*etLineProfileRadius;
extern GtkWidget *etWavelength, *etDd, *etDetSwing, *etCenX, *etCenY;
extern GtkWidget *etDetOrient, *etDetTilt, *etDetPhi, *etBeamTiltX, *etBeamTiltY;
extern GtkWidget *cbxRecent;
extern GtkWidget *etPeakThresh, *etZingerRadius, *etZingerTolerance;

extern char datafilename[];

static char filename[256];
static gint xx0,yy0;
static float dQ,Qr0,Qz0;
static int num_recent_images=0;

int q_available=0;

char *batch_com[]={"JPEG","EXTR","EXTB","CVQ1","SUMB","CVQ2","SUMM","PEAK","ANGD","SAVE"};

extern DataSet sto[8]; 

int get_a_file(int flags, const char* title) {
  GtkWidget* FileSelection;
  gint result;
  int fd=-1;
  char buf[256];
  char *p;

  memset( (void *)(&filename[0]), 0, 256 );
  FileSelection = 
    gtk_file_chooser_dialog_new(title,NULL,GTK_FILE_CHOOSER_ACTION_OPEN,
				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
				NULL);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (FileSelection), 
				       globalPref->cwd);
  if (gtk_dialog_run (GTK_DIALOG (FileSelection)) == GTK_RESPONSE_ACCEPT) {
      
    p = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (FileSelection));
    strncpy( &filename[0], p, 255 );
    g_free(p); 
    
    strcpy( globalPref->cwd, filename );
    // change globalPref->cwd to current directory
    strcpy( globalPref->cwd, g_path_get_dirname(globalPref->cwd) );
    strcat( globalPref->cwd, "/" );

    sprintf(buf,"Loading file: %s ...\n",filename);
    send_info_message (buf);
    strncpy(datafilename,filename,255);
    fd = open ( filename, flags );
    /* printf("%s\n", filename); */
    if (fd == -1) 
      send_info_message ("Cannot open file.");
  }

  gtk_widget_destroy(GTK_WIDGET (FileSelection));
  return(fd);
  
}

void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int fd;
  fd = get_a_file( O_RDONLY, "Open data file" );
  if (fd == -1) return;

  //read_data(fd, &sto[6], 1); /**/
  close(fd);
  //send_info_message ("Done.\n");
  
  // filename is set by get_a_file
  // add file name to "recent images" combobox
  // this change will trigger loading file
  
  gtk_combo_box_prepend_text((gpointer)cbxRecent,filename);
  gtk_combo_box_set_active((gpointer)cbxRecent,0);
  while (num_recent_images>=10) {
    gtk_combo_box_remove_text((gpointer)cbxRecent,num_recent_images); 
    num_recent_images--;    
  }  
  num_recent_images++;

  //gtk_widget_queue_draw(da2DView);
}


void change_widget_font(GtkWidget *widget, PangoFontDescription *font_desc) {
  GList *clist,*item;
  
  gtk_widget_modify_font(widget,font_desc);
  if (!GTK_IS_CONTAINER(widget)) return;
  clist=gtk_container_get_children((GtkContainer *)widget);
  if (!clist) return;

  item=clist;
  do {
    widget=(GtkWidget *)item->data;
    change_widget_font(widget,font_desc);
    item=item->next;
  } while (item);

  g_list_free(clist);

}



void
on_change_font_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget * dlgFont;
  char * fontName;
  char buf[256];
  int result;
  static PangoFontDescription *font_desc=NULL;
  GtkStyle* style;          

  style=gtk_widget_get_style(wControl);

  dlgFont = gtk_font_selection_dialog_new ("select font");
  gtk_font_selection_dialog_set_font_name((GtkFontSelectionDialog *)dlgFont,
					  pango_font_description_to_string(style->font_desc));
  g_signal_connect_swapped (dlgFont,
			    "response",
			    G_CALLBACK (gtk_widget_hide),
			    dlgFont);

  result = gtk_dialog_run (GTK_DIALOG (dlgFont));  
  fontName=gtk_font_selection_dialog_get_font_name((GtkFontSelectionDialog *)dlgFont);
  gtk_widget_destroy(dlgFont);

  if (fontName || result==-5) {   // -5 is OK, -6 is Cancel
    sprintf(buf,"changing font to %s.\n",fontName);
    send_info_message(buf);
    if (font_desc) pango_font_description_free(font_desc);
    font_desc=pango_font_description_from_string (fontName);
    change_widget_font(wControl,font_desc);
    g_free(fontName);
  }  


}


void
on_save_as_jpg1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  char buf[512];
  GError *ge=NULL;
  
  if (!dataset) {
    // data not available
    return;
  }

  //while (dataset->status<DATA_STATUS_INDEXED && gtk_events_pending ()) 
  //  gtk_main_iteration ();
  // this saves the whole frame
  //gdk_pixbuf_save(dataset->pixbuf,"/tmp/data.jpg","jpeg",NULL);
  // this saves only the visible part
  sprintf(buf,"%s/view-data.jpg",getenv(HOME_ENV));
  //gdk_pixbuf_save(globalPref->zoombuf,buf,"jpeg",&ge,NULL);
  gdk_pixbuf_save(dataset->pixbuf,buf,"jpeg",&ge,NULL);

}

void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint response;
  GtkWidget *dialog;
  FILE *outfile;
  GtkStyle* style;   
  char buf[256];

  /* really quit */
  dialog = gtk_message_dialog_new (GTK_WINDOW(wControl),
				   GTK_DIALOG_MODAL,
				   GTK_MESSAGE_QUESTION,
				   GTK_BUTTONS_OK_CANCEL,
				   "Really quit?");
  response = gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);  
  if (response == GTK_RESPONSE_OK) {
    // save font
    sprintf(buf,"%s/.view.gtk/conf.font",getenv(HOME_ENV));
    //printf("%s\n",buf);
    outfile = fopen(buf,"w+");
    if (outfile) {
      style=gtk_widget_get_style(wControl);
      fprintf(outfile,"%s\n",pango_font_description_to_string(style->font_desc));
      fclose(outfile);
    } 

    sprintf(buf,"%s/.view.gtk/conf.general",getenv(HOME_ENV));
    //printf("%s\n",buf);
    outfile = fopen(buf,"w+");
    if (outfile) {
      fprintf(outfile,"%d   # WINDOW_SIZE\n",WINDOW_SIZE);
      fclose(outfile);
    } 

    gtk_main_quit();
  }
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dialog;
  
  /* really quit */
  dialog = gtk_message_dialog_new (GTK_WINDOW(wControl),
				   GTK_DIALOG_MODAL,
				   GTK_MESSAGE_INFO,
				   GTK_BUTTONS_OK,
				   "       view.gtk\n\n written by Lin Yang\nmost recently updated on Mar. 31, 2011");
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);  

}

void
slideshow_event       (GtkWidget       *widget,
                                        gpointer         user_data)
{
  static GtkTextBuffer *buffer=NULL;
  GtkTextIter iter1,iter2;  
  int fd,i,ti,sr,action = GPOINTER_TO_INT( user_data );
  static int line_number,line_count=-1;
  char *data_file;
  
  if (file_list_loaded<0) return;

  if (buffer==NULL) buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tvFileList));
  i = gtk_text_buffer_get_line_count (buffer);
  if (i!=line_count) { // maybe the file list is changed
    line_count=i;
    line_number=0;
  }

  //printf("lc=%d, ln=%d  ===>",line_count,line_number);
  switch (action) {
  case 1: // goto first
    line_number = 0;
    break;
  case 2: // previous
    if (line_number>0) line_number--;
    break;
  case 3: // next
    if (line_number<line_count-2) line_number++;
    break;
  case 4: // last
     line_number = line_count-2;
  }
  //printf(" %d\n",line_number);
  
  gtk_text_buffer_get_iter_at_line (buffer, &iter1, line_number);
  gtk_text_buffer_get_iter_at_line (buffer, &iter2, line_number+1);
  data_file = gtk_text_buffer_get_text ( buffer, &iter1, &iter2, FALSE);

  sr = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (ckbtnDetRec));

  ti=strlen(data_file);
  if (ti<2) return;
  if (data_file[ti-1]=='\n') data_file[ti-1]=0;
  strcpy(filename, data_file);
  sprintf(text,"loading %s ... ", g_path_get_basename(filename));
  send_info_message(text);
  fd = open( data_file, OPEN_FILE_FLAG);
  if (fd<0) {
    send_info_message("failed to open file.\n");
    return;
  }
  send_info_message("done.\n");
  read_data(fd, &sto[6], 1);
  close(fd);
 
  if (sr) {
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (ckbtnDetRec), sr); 
    toggle_DetRec(GTK_TOGGLE_BUTTON (ckbtnDetRec),0);
  }

  gtk_widget_queue_draw(da2DView);
    
}


int fill_text(GtkWidget *widget, FILE *input, int format) {
  int i,ok,count=0;
  GtkTextIter start,end;
  GtkTextBuffer *buffer;
  char buf[512];
  struct stat file_stat;

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(widget));
  // clear old text first
  gtk_text_buffer_get_start_iter(buffer, &start);
  gtk_text_buffer_get_end_iter(buffer, &end);
  gtk_text_buffer_delete(buffer, &start, &end);

  while (!feof(input)) {
    text[0]=0;
    fgets(&text[0],255,input);
    if (text[0]==0 || text[0]=='#' || text[0]=='\n') continue;
    ok=0;
    switch (format) {
    case 0:  // file list, need to check whether the file exists
      if (text[0]!='/') {
	sprintf(buf,"%s/%s",globalPref->cwd,text);
	strncpy(text,buf,255);
      } 
      // if (stat(buf,&file_stat)==0) ok=1;
      //printf("%s\n:  %d\n",text,errno);
      ok=1;
      break;
    case 1:  // command list, valid command only
      for (i=0;i<sizeof(batch_com)/sizeof(char *);i++) {
	if (strncmp(text,batch_com[i],4)==0) {
	  globalPref->com_list[count]=i;
	  ok=1;
	  break;
	}
      }
      break;
    }

    if (ok) {
      if (format==0) file_list_loaded=1;
      else if (format==1) com_list_loaded=1;
      gtk_text_buffer_insert ( buffer, &end, text, -1);
      gtk_text_view_scroll_to_iter ( GTK_TEXT_VIEW(widget), &end, 0,
				     FALSE, 0, 0);
      count++;
    }
  }

  return(count);
}

gboolean
load_file_list                         (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  int fd;
  FILE *in;

  if (processing_flag==2) {
    send_info_message("Can not load file list at this moment.\n");
    return(TRUE);
  }

  fd = get_a_file( O_RDONLY , "Load file list" );
  if (fd==-1) return TRUE;

  in = fdopen (fd, "r");
  if (!in) return TRUE;
  fill_text(widget, in, 0);

  fclose(in);
  return TRUE;
}

gboolean
load_command_list                      (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  int fd;
  FILE *in;

  if (processing_flag==2) {
    send_info_message("Can not load command list at this moment.\n");
    return(TRUE);
  }

  fd = get_a_file( O_RDONLY , "Load command list" );  
  if (fd==-1) return TRUE;

  in = fdopen (fd, "r");
  if (!in) return TRUE;
  globalPref->ncom = fill_text(widget, in, 1);

  fclose(in);
  
  return TRUE;
}

void stop_processing() {
  //  G_LOCK(processing_flag);
  processing_flag = 1;
  //G_UNLOCK(processing_flag);
}

void sum(int si) {
  // use sto[5] as a buffer
  int ix,iy;
  float aa,bb,tt;

  // if si is 0, initialize sto[5]
  
  if (si==0) {
    if (sto[5].data && (dataset->width==sto[5].width
			|| dataset->height==sto[5].height)) {
      free(sto[5].data);
      sto[5].data=NULL;
    }
    if (!sto[5].data) {
      sto[5].data 
        = (DTYPE *)malloc(sizeof(DTYPE)*dataset->width*dataset->height);
     }
    sto[5].width = dataset->width;
    sto[5].height = dataset->height;
    sto[5].zmax = dataset->zmax;
    sto[5].zmin = dataset->zmin;
    sto[5].avg = dataset->avg;
    sto[5].sigma = dataset->sigma;
    sto[5].favg = dataset->favg;
 
    memcpy((void *)(sto[5].data), (void *)(dataset->data),
	     sizeof(DTYPE)*dataset->width*dataset->height);
    strcpy(sto[5].comments,"SUMM");

    return;
  }

  for (ix=0;ix<dataset->width;ix++) {
    for (iy=0;iy<dataset->height;iy++) {
      aa=dataset->data[ix+iy*dataset->width];
      bb=sto[5].data[ix+iy*dataset->width];
/* *********************      
      if (si==1) { // data stored in sto[5] still contain zingers
        dataset->data[ix+iy*dataset->width] 
          = sto[5].data[ix+iy*dataset->width]
          = (aa+bb)/2 - fabs(aa-bb)/2;
      } else { // sto[5] contain average of mutiple (si) sets 
        tt = (fabs(aa-bb)/2 - (aa-bb)/2);  // this will be non-zero is the pixel is a zinger
        // but it will also be non-zero if bb simply have higher intensity
        // no easy way to exclude this artifact
        // if (tt<sqrt(bb)) tt=0;  // this might produce discontinuity
        tt=0; // for now, sum only
	bb = aa - tt;  // this removes zingers in dataset
        dataset->data[ix+iy*dataset->width] 
          = sto[5].data[ix+iy*dataset->width]
          = (aa*si+bb)/(si+1);
      }
***************************/
       dataset->data[ix+iy*dataset->width]
          = sto[5].data[ix+iy*dataset->width] = aa+bb;
    }
  }        
 
}

gboolean
start_processing                      (GtkWidget       *widget,
				       GdkEventButton  *event,
				       gpointer         user_data) {
  int i,ic,line_count,fd,ti;
  gboolean result;
  char *data_file;
  GtkTextBuffer *buffer;
  GtkTextIter iter1,iter2;  
  GtkDialog *dialog;
  GtkWidget *pbar_conv;
  GError *err=NULL;
  float dq,dn,qmax,tf,tt;
  char sbuf[512];
  FILE *svfile;

  qmax = -1;
  //G_LOCK(processing_flag);
  if (processing_flag == 1) processing_flag = 2;
  else {
    send_info_message("Can not start processing.\n");
    return(TRUE);
  }
  //G_UNLOCK(processing_flag);

  sprintf(sbuf,"%s/view-sumb.dat",getenv(HOME_ENV));
  remove(sbuf);

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tvFileList));
  line_count = gtk_text_buffer_get_line_count (buffer);
  if (line_count==0) return(TRUE);

  dialog = GTK_DIALOG(gtk_dialog_new ());
  pbar_conv = gtk_progress_bar_new ();
  gtk_widget_show (pbar_conv);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), 
		      pbar_conv, FALSE, FALSE, 0);
  gtk_window_set_modal(GTK_WINDOW(dialog),TRUE);
  gtk_dialog_add_button(dialog,"stop",1);
  g_signal_connect((gpointer)dialog, "response",
		   G_CALLBACK(stop_processing), NULL);
  gtk_widget_show (GTK_WIDGET(dialog));
  
  gtk_text_buffer_get_start_iter ( buffer, &iter1 );
  gtk_text_buffer_get_start_iter ( buffer, &iter2 );
  for (i=0;i<line_count;i++) {
    sprintf(text,"converting file %d of %d",i+1, line_count);
    gtk_progress_bar_set_text (GTK_PROGRESS_BAR (pbar_conv), text);
    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (pbar_conv), 1.0*(i+1)/line_count);
    while (gtk_events_pending ()) gtk_main_iteration ();
    if (processing_flag == 1) break;
    gtk_text_view_scroll_to_iter ( GTK_TEXT_VIEW (tvFileList), &iter1, 0, FALSE, 0, 0);
    gtk_text_iter_forward_line (&iter2);
    data_file = gtk_text_buffer_get_text ( buffer, &iter1, &iter2, FALSE);

    ti=strlen(data_file);
    if (ti<2) continue;
    if (data_file[ti-1]=='\n') data_file[ti-1]=0;
    strcpy(filename, data_file);
    sprintf(text,"processing %s: \n", g_path_get_basename(filename));
    send_info_message(text);
    fd = open( data_file, OPEN_FILE_FLAG);
    if (fd<0) {
      send_info_message("failed to open file.\n");
      continue;
    }
    read_data(fd, &sto[6], 1); /**/
    close(fd);

    gtk_widget_queue_draw(da2DView);
    while (gtk_events_pending ()) gtk_main_iteration ();

    // first determine dq, this information is common to all datasets
    //dq = pos_to_q( globalPref->cx + globalPref->N_dq, globalPref->cy);

    // grid spacing given by et1DGrid
    dn = getValue(et1DGrid);
    if (dn<0) dn=1;
    setValue(et1DGrid,"%3.1f",dn);
    dn = getValue(et1DGrid);

    dq = 0.5* pos_to_q( globalPref->cx + dn, globalPref->cy);
    tf=1;
    while (dq<1) {
      dq*=10;
      tf/=10;
    }
    dq = tf * (int)(dq+0.5);
    // assume circular effective area
    tt=dataset->width/2 - globalPref->cx;
    tf=dataset->height/2 - globalPref->cy;
    tf=sqrt(tt*tt+tf*tf);
    //qmax = pos_to_q(dataset->width/2+tf+globalPref->cx,globalPref->cy);
    // UPDATE. Oct 2009, the assumption of circular area is not valid for PSI WAXS 
    // the highest q could be one of the corners
    qmax = pos_to_q(dataset->width*0.7+tf+globalPref->cx,globalPref->cy);
    //tf = atan2( dataset->width/2 - globalPref->cx,
    //	dataset->height/2 - globalPref->cy );
    //qmax = pos_to_q( dataset->width/2*(1.0+cos(tf)) + globalPref->cx,
    //	     dataset->height/2*(1.0+sin(tf)) + globalPref->cy);
    
    // processing this file 
    // "JPEG","EXTR","EXTB","CVQ1","SUMB","CVQ2","SUMM","PEAK","ANGD","SAVE"};
    for (ic=0;ic<globalPref->ncom;ic++) {

      //while (gtk_events_pending ()) gtk_main_iteration ();

      sprintf(text,"     performing %s.   ",batch_com[globalPref->com_list[ic]]);
      send_info_message(text);
      strcpy(text,data_file);
      switch (globalPref->com_list[ic]) {
      case 0: // "JPEG" save as a jpeg
	strcat(text,".jpg");
	while (dataset->status<DATA_STATUS_INDEXED && gtk_events_pending ()) 
	  gtk_main_iteration ();
	// this saves the whole frame
	//gdk_pixbuf_save(dataset->pixbuf,text,"jpeg",NULL);
	// this saves only the visible part
	gdk_pixbuf_save(globalPref->zoombuf,text,"jpeg",&err,NULL);
	break;
      case 1: // "EXTR", extract line profile
	strcat(text,".ext");
	extract_line(text,0,&tt);
	break;
      case 2: // "EXTB", extract line with background subtraction
	strcat(text,".ext");
	extract_line(text,1,&tt);
	break;	
      case 3: // "CVQ1", convert to 1D profile
	strcat(text,".ave");
	convert_to_1D(dq, (int)(qmax/dq)+0.5, text);
	break;
      case 4: // "SUMB", like "EXTB", but output the total intensity afterward, 
              // useful for rocking curve
	extract_line(0,1,&tt);
	sprintf(sbuf,"printf \"%3d  %10.2f   # %s\n\" >> %s/view-sumb.dat",
			i,tt,text,getenv(HOME_ENV));
	system(sbuf);
	break;
      case 5: // "CVQ2"
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ckbtnDetRec), 1);
        while (gtk_events_pending ()) gtk_main_iteration ();
        break;
      case 6: // "SUMM", add newfile to existing SUM using dezinger method
              // this command requires sto[5]
        sum(i);
        if (i) { // reload image
          dataset->status = DATA_STATUS_LOADED;
          gtk_widget_queue_draw(da2DView);
          while (gtk_events_pending ()) gtk_main_iteration ();
        }
        break;
      case 7: // "PEAK", find peaks
	strcat(text,".pks");
	show_peak(text);
	while (gtk_events_pending ()) gtk_main_iteration ();
	break;
      case 8: // "ANGD", angular distribution
	strcat(text,".ang");
	convert_to_ang_dist(text);
	break;	
      case 9: // "SAVE", save the 2D data on the screen
	strcat(text,".sav");
	svfile=fopen(text,"w+");
	fwrite(dataset->data,sizeof(DTYPE),dataset->width*dataset->height,svfile);
	fclose(svfile);
	break;
      }
      send_info_message("done.\n");
    }
    // finished processing for one file
    gtk_text_iter_forward_line (&iter1);
  }  
  gtk_widget_destroy(GTK_WIDGET(dialog));
  
  //G_LOCK(processing_flag);
  processing_flag = 1;
  //G_UNLOCK(processing_flag);
  
  return(TRUE);
}

void change_color (GtkWidget *widget,  
		   gpointer user_data)
{
  GtkWidget *dialog;
  GtkColorSelection *colorsel;
  gint response;
  GdkColor color;
  GtkStyle *style;

  dialog = gtk_color_selection_dialog_new ("Changing color");
  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (wControl));
  colorsel = GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel);
  gtk_window_set_modal (GTK_WINDOW(dialog),TRUE);  
  style=gtk_widget_get_style(widget);
  
  gtk_color_selection_set_previous_color (colorsel, &style->bg[0]);
  gtk_color_selection_set_current_color (colorsel, &style->bg[0]);
  
  gtk_color_selection_set_has_palette (colorsel, TRUE);
  response = gtk_dialog_run (GTK_DIALOG (dialog));

  if (response == GTK_RESPONSE_OK)
    {
      gtk_color_selection_get_current_color (colorsel, &color);
      if (color.red != style->bg[0].red || color.blue != style->bg[0].blue
	  || color.green != style->bg[0].green) {
	gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (widget, GTK_STATE_PRELIGHT, &color);
      
	/* also need to update the colormap here */
	update_RGBmap();
	gtk_widget_queue_draw(daColorTable);
	dataset->status=DATA_STATUS_LOADED;
	gtk_widget_queue_draw(da2DView);
      } 
   }
  
  gtk_widget_destroy (dialog);
  
}

gint button_press_event (GtkWidget *widget, GdkEventButton *event)
{
  if (event->button == 1) {
    xx0=yy0=-1;
    globalPref->mouse_pressed = 1;
    gdk_window_set_cursor(w2DView->window,globalPref->move_cursor);
    //draw_brush (widget, event->x, event->y);
  }
  return TRUE;
}

gint button_release_event (GtkWidget *widget, GdkEventButton *event)
{
  if (event->button == 1) {
    gdk_window_set_cursor(w2DView->window,globalPref->crosshair_cursor);
    globalPref->mouse_pressed = 0;
  }
    //draw_brush (widget, event->x, event->y);

  return TRUE;
}

gint motion_notify_event (GtkWidget *widget, GdkEventMotion *event)
{
  int x, y, window_width, window_height;
  GdkModifierType state;
  static char textbuf[256];
  float q,qr,qz,tx,ty;

  if (!dataset) {
    // data not available
    return;
  }

  if (event->is_hint)
    gdk_window_get_pointer (event->window, &x, &y, &state);
  else
    {
      x = event->x;
      y = event->y;
      state = event->state;
    }

  gtk_window_get_size ( GTK_WINDOW (w2DView), &window_width, &window_height );

  if (!globalPref->mouse_pressed) {
    if (dataset->status>DATA_STATUS_INDEXED) { // show pointer position
      //printf("%d\n",dataset->status);
      x = globalPref->viewport_cx + (x-window_width/2)/globalPref->zoom;
      y = globalPref->viewport_cy + (y-window_height/2)/globalPref->zoom;
      // (x,y could be out of bounds when zoom is small
      if (x<0 || y<0 || x>=dataset->width || y>=dataset->height) return;
      if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (ckbtnDetRec))) 
	{
	  qr=Qr0+dQ*x;
	  qz=Qz0-dQ*y;
	  sprintf(&textbuf[0],"(%4d,%4d), q=(%6.3f,%6.3f) : %8.2f\n",
		  x,y,qr,qz,sto[7].data[x+y*sto[7].width]);
	}
      else if (globalPref->samGeometry) 
	{
	  q=pos_to_qrz(x,y,&qr,&qz);
	  //qrz_to_pos(qr,qz,&tx,&ty);
	  sprintf(&textbuf[0],"(%4d,%4d): %8.2f, q=%6.4f (qr:%6.4f, qz: %6.4f)\n",
		  x,y,dataset->data[x+y*dataset->width],q,qr,qz);
	  //sprintf(&textbuf[0],"(%4d,%4d):(qr:%6.4f,qz:%6.4f):(%6.1f,%6.1f)\n",
	  //	  x,y,qr,qz,tx,ty);
	} else { 
	  q=pos_to_q(x,y);
	  sprintf(&textbuf[0],"(%4d,%4d) : %8.2f,    q=%6.4f\n",
		  x,y,dataset->data[x+y*dataset->width],q);
	}
      send_status_message(&textbuf[0],0);
    }
    return FALSE;
  }
  if (x<0 || y<0 || x>511 || y>511) return TRUE;
  if (xx0<0) {
    xx0=x; yy0=y;
    return TRUE;
  }
  globalPref->viewport_cx -= (x-xx0)/globalPref->zoom;
  globalPref->viewport_cy -= (y-yy0)/globalPref->zoom;
  //printf("offset = ( %4d,%4d )\n",x-xx0,y-yy0);
  xx0=x; yy0=y;
  if (dataset->status!=DATA_STATUS_EMPTY) { // must be indexed already
    dataset->status=DATA_STATUS_INDEXED;
    gtk_widget_queue_draw(da2DView);
  }
  /*
  if (state & GDK_BUTTON1_MASK && pixmap != NULL)
    draw_brush (widget, x, y);
  */
  return TRUE;
}

void
show_image_statistics                  (GtkButton       *button,
                                        gpointer         user_data)
{

  if (!dataset) {
    // data not available
    return;
  }

}

void
on_format_changed                      (GtkEditable     *editable,
                                        gpointer         user_data)
{
  globalPref->DataFormat = GPOINTER_TO_INT( user_data );
}

void
on_Zoom_changed                      (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
  float zoom[]={0.5,1,2,4,8};
  static int prev=-1;
  int selection; 

  assert(dataset);

  selection = gtk_combo_box_get_active (GTK_COMBO_BOX (combobox));
  if (prev>0 && prev==selection) return;
  
  globalPref->zoom = zoom[ prev=selection ];
  if (dataset->status) { // not empty
    dataset->status = DATA_STATUS_INDEXED;
    gtk_widget_queue_draw(da2DView);
  }
}

void send_info_message (char *text) {
  GtkTextIter iter;

  //gdk_threads_enter();
  gtk_text_buffer_get_end_iter ( tbuf_info, &iter );
  gtk_text_buffer_insert ( tbuf_info, &iter, text, -1);
  gtk_text_view_scroll_to_iter ( GTK_TEXT_VIEW (tvInfo), &iter, 0,
				 FALSE, 0, 0);
  //gdk_threads_leave();
}

void send_status_message (char *text, int which) {
  /* meant to have two GtkLabels here */

  switch (which) {
    case 0:
      gtk_label_set_text(GTK_LABEL(lbStatusMsg),text);
      break;
    case 1:
      gtk_label_set_text(GTK_LABEL(lbStatusMsg),text);
  }
}  

float getValue( GtkWidget *entry ) {
  float tmp;
  sscanf( gtk_entry_get_text (GTK_ENTRY(entry)), "%f", &tmp );
  return(tmp);
}

void setValue( GtkWidget *entry, const char *format, float v ) {
  char buf[256];
  if (strchr(format,'f') || strchr(format,'g')) sprintf(buf,format,v);
  else sprintf(buf,format,(int)v);
  gtk_entry_set_text(GTK_ENTRY(entry), buf);
}

void on_autoscale_toggled (GtkWidget *widget, gpointer user_data) {

  assert(dataset);
  if (dataset->status==DATA_STATUS_EMPTY) return;

  switch ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (widget) ) ) {
  case TRUE:
    if (dataset->status>DATA_STATUS_EMPTY) {
      /* old code 
      globalPref->zmax = dataset->avg+3*dataset->sigma;
      if (globalPref->zmax>65535) globalPref->zmax = 65535;
      globalPref->zmin = dataset->avg-3*dataset->sigma;
      if (globalPref->zmin<0) globalPref->zmin = 0;
      */
      
      find_MM(dataset,1);

      globalPref->zmax = dataset->zmax;
      globalPref->zmin = dataset->zmin;

      setValue(etZMin, "%5.0f", globalPref->zmin);
      setValue(etZMax, "%5.0f", globalPref->zmax);
      dataset->status = DATA_STATUS_LOADED;
    }
    gtk_widget_set_sensitive (etZMax, FALSE);
    gtk_widget_set_sensitive (etZMin, FALSE);   
    gtk_widget_queue_draw(da2DView);
    break;
  case FALSE:
    if (globalPref->zmin>0) setValue(etZMin,"%5.0f",globalPref->zmin);
    if (globalPref->zmax>0) setValue(etZMax,"%5.0f",globalPref->zmax);
    gtk_widget_set_sensitive (etZMax, TRUE);
    gtk_widget_set_sensitive (etZMin, TRUE);   
  }
  
}

void on_log_toggled (GtkToggleButton *widget, gpointer user_data) {
  switch ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (widget) ) ) {
  case TRUE:
    send_info_message ("Switching to log scale.\n");
    gtk_widget_set_sensitive (etGamma, TRUE);
    globalPref->log_gamma = getValue(etGamma);
    //printf("%f\n",globalPref->log_gamma);    
    if (globalPref->log_gamma <= 0.01) 
      setValue(etGamma, "%4.2f",globalPref->log_gamma = 0.3);
    break;
  case FALSE:
    send_info_message ("Switching to linear scale.\n");
    gtk_widget_set_sensitive (etGamma, FALSE);
    globalPref->log_gamma = -1;
  }
  if (dataset->status > DATA_STATUS_LOADED) dataset->status = DATA_STATUS_LOADED;
  gtk_widget_queue_draw(da2DView);
}

void on_editing_done (GtkWidget *widget, gpointer user_data) {
  float v;
  int data = GPOINTER_TO_INT( user_data );

  if (widget) v=getValue(widget); // can be NULL, called by load_Parameter_set

  switch (data) {
  case 0: // etZmin
    if (v>=0 && v<globalPref->zmax && globalPref->zmin!=v) { 
      globalPref->zmin=v;
      setValue(widget, "%5.0f", v);
      if (dataset->status >= DATA_STATUS_LOADED) dataset->status = DATA_STATUS_LOADED;
    }
    break;
  case 1: // etZmax
    if (v>=0 && v>globalPref->zmin && globalPref->zmax!=v) {
      globalPref->zmax=v;
      setValue(widget, "%5.0f", v);
      if (dataset->status >= DATA_STATUS_LOADED) dataset->status = DATA_STATUS_LOADED;
    }
    break;
  case 2: // etGamma
    //printf("%f\n",v);
    if (v>0.01) globalPref->log_gamma=v;
    setValue(etGamma, "%4.2f",globalPref->log_gamma);
    if (dataset->status >= DATA_STATUS_LOADED) dataset->status = DATA_STATUS_LOADED;
    break;
  case 3: // etWavelength
    if (v>0.01) globalPref->wavelength=v;
    setValue(widget, "%6.4f",globalPref->wavelength);
    break;
  case 4: // etDd
    if (v>0.01) globalPref->ratioDd=v;
    setValue(widget, "%5.3f",globalPref->ratioDd);
    break;
  case 5: // etDetSwing
    globalPref->detector_psi=v/180*Pi;
    setValue(widget, "%5.1f",v);
    break;
  case 6: // etCenX
    globalPref->cx=v;
    setValue(widget, "%5.1f",globalPref->cx);
    break;
  case 7: // etCenY
    globalPref->cy=v;
    setValue(widget, "%5.1f",globalPref->cy);
    break;
  case 8: // etSamNormal
    globalPref->normal=v;
    setValue(widget, "%5.1f",globalPref->normal);
    break;
  case 9: // etIncidentAng
    globalPref->incident=v;
    setValue(widget, "%5.2f",globalPref->incident);
    break;    
  case 10: // etZingerRadius
    if (v<1) v=1;
    if (v>20) v=20;
    globalPref->zr=v;
    setValue(widget, "%2d",globalPref->zr);
    break;
  case 11: // etZingerTolerance
    if (v<0.01) v=0.01;
    globalPref->zt=v;
    setValue(widget, "%4f",globalPref->zt);
    break;
  }
  calc_rot(globalPref);
  if (globalPref->refPattern && data>=3 && data<=7 || 
      globalPref->samGeometry && (data>=6 && data<=9 || data==256)) {
    // need to redraw the reference pattern for a different geometry
    if (dataset->status == DATA_STATUS_DRAWN) dataset->status = DATA_STATUS_NEED_DECOR;
  }
  // q_available=2; // original, don't know why
  q_available=0;
  if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (ckbtnDetRec))) 
    toggle_DetRec(GTK_TOGGLE_BUTTON (ckbtnDetRec),0);
  //else 
    gtk_widget_queue_draw(da2DView);

}

void
on_RefPattern_changed                (GtkComboBox    *combobox,
                                        gpointer         user_data) {
  FILE *ref_pat;
  char input[256];
  int i;

  globalPref->refPattern = gtk_combo_box_get_active(combobox);
  //  if (dataset->status == DATA_STATUS_DRAWN)  dataset->status = DATA_STATUS_INDEXED;
  if (dataset->status == DATA_STATUS_DRAWN)  dataset->status = DATA_STATUS_NEED_DECOR;
  //if (globalPref->refPattern == 0 && dataset->status > DATA_STATUS_INDEXED)
  //  dataset->status = DATA_STATUS_INDEXED;
  if (globalPref->refPattern==3) { 
    // read user defined pattern
    // allowed pattern:
    //     ring:    R q
    //     point:   P qr qz
    //     line:    L qr1 qz1 qr2 qz2 N
    if (globalPref->q_ref) free(globalPref->q_ref);
    sprintf(input,"%s/.view.gtk/ref_pattern",getenv(HOME_ENV));
    ref_pat = fopen(input,"r");
    if (ref_pat) {
      i = 0;
      while (!feof(ref_pat)) {
	input[0]=0;
	fgets(input,255,ref_pat);
	if (input[0]=='R' || input[0]=='P' || input[0]=='L') i++;
      }
      globalPref->n_ref=i;
      if (i) {
	rewind(ref_pat);
	globalPref->q_ref = (stRefPtn*)malloc(i*sizeof(stRefPtn));
	i=0;
	while (!feof(ref_pat)) {
	  input[0]=0;
	  fgets(input,255,ref_pat);
	  switch (input[0]) {
	  case 'R':
	    globalPref->q_ref[i].type='R';
	    sscanf(input,"R%f%d",&globalPref->q_ref[i].para[0],&globalPref->q_ref[i].color);
	    i++;
	    break;
	  case 'P':
	    globalPref->q_ref[i].type='P';
	    sscanf(input,"P%f%f%d",
		   &globalPref->q_ref[i].para[0],
		   &globalPref->q_ref[i].para[1],
		   &globalPref->q_ref[i].color);
	    i++;
	    break;
	  case 'L':
	    globalPref->q_ref[i].type='L';
	    sscanf(input,"L%f%f%f%f%d%d",
		   &globalPref->q_ref[i].para[0],&globalPref->q_ref[i].para[1],
		   &globalPref->q_ref[i].para[2],&globalPref->q_ref[i].para[3],
		   &globalPref->q_ref[i].N,&globalPref->q_ref[i].color);
	    if (globalPref->q_ref[i].N<5) globalPref->q_ref[i].N=5;
	    else if (globalPref->q_ref[i].N>100) globalPref->q_ref[i].N=100;
	    i++;
	    break;
	  }
	  if (globalPref->q_ref[i].color>7) globalPref->q_ref[i].color=7;
	  if (globalPref->q_ref[i].color<0) globalPref->q_ref[i].color=0;
	}
      }
      
      fclose(ref_pat);
    } else { 
      globalPref->n_ref=0;
      send_info_message("Cannot open user defined pattern in /tmp/ref_pattern.\n");
    }
  }

  gtk_widget_queue_draw(da2DView);

}

void
evaluate_expression                    (GtkButton       *button,
                                        gpointer         user_data)
{
  
  int i,nSet1,nSet2;
  float factor,bkg;

  factor = getValue(etScaleFactor);
  bkg = getValue(etConst);
  setValue(etScaleFactor, "%f", factor);
  setValue(etConst,"%f",bkg);

  nSet1 = gtk_spin_button_get_value_as_int(sbSet1);
  nSet2 = gtk_spin_button_get_value_as_int(sbSet2);

  if (!sto[nSet1].data || !sto[nSet2].data ||
      sto[nSet1].height!=sto[nSet2].height || sto[nSet1].width!=sto[nSet2].width) {
    send_info_message ("Incompatible sets!\n");
    return;
  }

  if (dataset->data && (dataset->width==sto[nSet1].width
			|| dataset->height==sto[nSet1].height)) {
    free(dataset->data);
    dataset->data=NULL;
  }
  dataset->width = sto[nSet1].width;
  dataset->height = sto[nSet1].height;
  if (!dataset->data) {
    dataset->data 
      = (DTYPE *)malloc(sizeof(DTYPE)*dataset->width*dataset->height);
  }
 
  for (i=0;i<dataset->width*dataset->height;i++) 
    dataset->data[i]=sto[nSet1].data[i]+factor*(sto[nSet2].data[i]-bkg);
  
  find_MM(dataset,1);
  post_loading_data(dataset,1);
  gtk_widget_queue_draw(da2DView);
}

void
save_current_set                       (GtkButton       *button,
                                        gpointer         user_data)
{
  // save data, binary, DTYPE
  
  int fd;

  fd = get_a_file(O_WRONLY|O_CREAT, "Save data to" );
  if (fd==-1) return;

  write(fd,dataset->data,sizeof(DTYPE)*dataset->width*dataset->height);
  
  close(fd);

  chmod(filename,0666);
  sprintf(text,"Data saved to %s\n",filename);// g_path_get_basename(filename));
  send_info_message (text);

}


void
store_current_set                      (GtkButton       *button,
                                        gpointer         user_data)
{
  int cur = gtk_spin_button_get_value_as_int(sbCurSet);
  // get the number on sbCurSet
  
  if (!dataset->data) return;
  if (sto[cur].data && (dataset->width==sto[cur].width
			|| dataset->height==sto[cur].height)) {
    free(sto[cur].data);
    sto[cur].data=NULL;
  }
  if (!sto[cur].data) {
    sto[cur].data 
      = (DTYPE *)malloc(sizeof(DTYPE)*dataset->width*dataset->height);
  }
  sto[cur].width = dataset->width;
  sto[cur].height = dataset->height;
  sto[cur].zmax = dataset->zmax;
  sto[cur].zmin = dataset->zmin;
  sto[cur].avg = dataset->avg;
  sto[cur].sigma = dataset->sigma;
  sto[cur].favg = dataset->favg;
 
  memcpy((void *)(sto[cur].data), (void *)(dataset->data),
	   sizeof(DTYPE)*dataset->width*dataset->height);

  /* change the comments if it is still "(empty)" */
  if (!strncmp(&sto[cur].comments[0],"(empty)",7)) {
    strcpy(sto[cur].comments,"(loaded)");
    gtk_entry_set_text(GTK_ENTRY(etSetComments),sto[cur].comments);
  }

}


void
recall_stored_set                      (GtkButton       *button,
                                        gpointer         user_data)
{
  int cur = gtk_spin_button_get_value_as_int(sbCurSet);
  
  if (!sto[cur].data) return;

  // need to calculate new q data
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ckbtnDetRec), q_available=0);

  // need new dataset if of different size
  if (dataset->data && (dataset->width!=sto[cur].width
			|| dataset->height!=sto[cur].height)) {
    free(dataset->data);
    dataset->data=NULL;
  }
  dataset->width = sto[cur].width;
  dataset->height = sto[cur].height;
  if (!dataset->data) {
    dataset->data 
      = (DTYPE *)malloc(sizeof(DTYPE)*dataset->width*dataset->height);
  }
  dataset->zmax = sto[cur].zmax;
  dataset->zmin = sto[cur].zmin;
  dataset->avg = sto[cur].avg;
  dataset->sigma = sto[cur].sigma;
  dataset->favg = sto[cur].favg;
 
  memcpy((void *)(dataset->data), (void *)(sto[cur].data),
	 sizeof(DTYPE)*dataset->width*dataset->height);  

  post_loading_data(dataset,1);
  gtk_widget_queue_draw(da2DView);

  
}


void
load_set_comments                      (GtkEditable     *editable,
                                        gpointer         user_data)
{
  int cur;
  cur = gtk_spin_button_get_value_as_int(sbCurSet);
  gtk_entry_set_text(GTK_ENTRY(etSetComments),&sto[cur].comments[0]);
}


void
update_set_comments                    (GtkCellEditable *celleditable,
                                        gpointer         user_data)
{
  int cur;
  cur = gtk_spin_button_get_value_as_int(sbCurSet);
  strncpy(&sto[cur].comments[0],gtk_entry_get_text(GTK_ENTRY(etSetComments)),255);
  sto[cur].comments[255]=0;
}

// sto[6] stores raw data, sto[7] is the data in reciprocal space
// old toggle_DetRec code copies data between dataset and sto[6]/sto[7]
// new code use dataset as a pointer that normally points to sto[6]
void
toggle_DetRec                          (GtkToggleButton  *button,
                                        gpointer         user_data)
{
  int ix,iy,iqr,iqz,count;
  float d_q,qrmax,qrmin,qzmax,qzmin,qr,qz,tmp,tx,ty;
  double av=0,sg=0;
  char buf[256];
  
  if (!dataset->data || globalPref->samGeometry==0) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ckbtnDetRec), 0);
    return;
  }

  switch ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (button) ) ) {
  case TRUE:
    dataset = &sto[7];   // sto[7] used to store data in qr-qz plane

    if (q_available) break;      // skip this if q data already calculated
    
    if (sto[6].width!=sto[7].width || sto[6].height!=sto[7].height) {
      if (sto[7].data) free(sto[7].data);
      if (sto[7].indexed_buf) free(sto[7].indexed_buf);
      sto[7].data=NULL;      
      sto[7].indexed_buf=NULL;
    }
      
    if (!sto[7].data) {
      sto[7].width = sto[6].width;
      sto[7].height = sto[6].height;
      sto[7].data 
        = (DTYPE *)malloc(sizeof(DTYPE)*sto[7].width*sto[7].height);
    }

    if (!sto[7].indexed_buf) {
      sto[7].indexed_buf = 
        (guchar *)malloc(RGBA*sizeof(guchar)*sto[7].width*sto[7].height);
    }

    // determine the best q-range for reciprocal space data
    for (ix=0;ix<=dataset->width;ix+=dataset->width/16) {
      for (iy=0;iy<=dataset->height;iy+=dataset->height/16) {
	pos_to_qrz(ix,iy,&qr,&qz);
	if (ix==0 && iy==0) {
	  qrmin=qrmax=qr;
	  qzmax=qzmin=qz;
	} else {
	  if (qr<qrmin) qrmin=qr;
	  if (qr>qrmax) qrmax=qr;
	  if (qz<qzmin) qzmin=qz;
	  if (qz>qzmax) qzmax=qz;
	}
      }
    }

    tmp=(qrmax-qrmin)/sto[7].width;
    d_q=(qzmax-qzmin)/sto[7].height;
    if (tmp>d_q) d_q=tmp;
    tmp=1;
    while (d_q>1) {tmp*=10;d_q/=10;}
    while (d_q<0.1) {tmp/=10;d_q*=10;}
    dQ=d_q=0.01*(int)(d_q*100+0.5)*tmp;

    iqr=(int)(qrmin/d_q);
    iqz=(int)(qzmax/d_q);
    sto[7].x0=Qr0=d_q*iqr;
    sto[7].y0=Qz0=d_q*iqz;
    sto[7].dx=sto[7].dy=d_q;

    send_info_message("converting data into reciprocal space:\n");
    sprintf(buf,"  dq=%f,  %dx%d\n",d_q,sto[7].width,sto[7].height);
    send_info_message (buf);
    sprintf(buf,"  upper-left corner at (%f, %f) \n",d_q*iqr,d_q*iqz);
    send_info_message (buf);    

    count=0;
    av=sg=0;
    for (ix=0;ix<sto[7].width;ix++) {
      for (iy=0;iy<sto[7].height;iy++) {
	//printf("%7.3f  %7.3f\n",dq*(ix+iqr),dq*(iy+iqz));
	tmp=sto[7].data[ix+iy*sto[7].width]=
	    qrz_to_pos(d_q*(ix+iqr),d_q*(iqz-iy),&tx,&ty);
	if (ix==0 && iy==0) sto[7].zmax=sto[7].zmin=tmp;
	else {
	  if (sto[7].zmax<tmp) sto[7].zmax=tmp;
	  if (sto[7].zmin>tmp) sto[7].zmin=tmp;
	}
	av += tmp;
	sg += tmp*tmp;
	count++;
      }
    }

    av /= count;
    sg /= count;
    sto[7].avg = av;
    sto[7].favg = av;
    sto[7].sigma = sqrt( sg - av*av ) + 0.5;    
    //printf("qrmin = %6.3f,  qrmax = %6.3f,  qzmin = %6.3f,  qzmax = %6.3f\n",
    //   qrmin,qrmax,qzmin,qzmax);
    q_available=TRUE;
    post_loading_data(dataset,1);   // need to redo sto[7]'s pixbuf
    break;
  case FALSE:
    // restore original data to display
    dataset = &sto[6];
  }
  
  // update display
  dataset->status = DATA_STATUS_LOADED;
  gtk_widget_queue_draw(da2DView);

}


void
MarkQrz                                (GtkButton       *button,
                                        gpointer         user_data)
{
  // must copy data into sto[6] first
}

void
extract_line_profile                   (GtkButton       *button,
                                        gpointer         user_data)
{
  float t;
  char buf[512];
  sprintf(buf,"%s/view-extracted.dat",getenv(HOME_ENV));
  extract_line(buf,2,&t);
}

void extract_line(char *fname, int fdraw, float *sm) {
  // draw the line if fdraw is 2
  // subtract background if fdraw is 1
  float xc,yc,ang,radius,sc,xf,yf,tfx,tfy;
  float wtx,wty;
  int ix,iy,idx,len,bkup_mask;
  float *sa,*wa,*sba,*wba,q,qr,qz;
  FILE *out;
  char obuf[256];
  BITMAP *bkup_bitmap;

  if (!dataset->data) {
     send_info_message ("load data first.\n");
    return;
  }   

  // get the numbers
  // angle is clock-wise
  xc=getValue(etLineProfileCx);
  yc=getValue(etLineProfileCy);
  len=getValue(etLineProfileLength);
  ang=getValue(etLineProfileOrient);
  radius=getValue(etLineProfileRadius);
  // in case of bad inputs
  setValue(etLineProfileCx,"%5.1f",xc);
  setValue(etLineProfileCy,"%5.1f",yc);
  setValue(etLineProfileLength,"%4d",len);
  setValue(etLineProfileOrient,"%5.2f",ang);
  setValue(etLineProfileRadius,"%3.1f",radius);
  if (len<=1 || radius<0.1) {
    send_info_message ("bad input for 'extract'.\n");
    return;
  }
  radius+=0.001;     // this avoids the irregular shape at 90deg ang

  if (fname) {
    out = fopen(fname,"w+");
    if (!out) {
      sprintf(obuf,"cannot open %s.\n",fname);
      send_info_message (obuf);
      return;
    }   
  }

  if (fdraw==2) {
    // backup bitmap first
    bkup_bitmap=globalPref->mask_bitmap;
    globalPref->mask_bitmap=new_bitmap(bkup_bitmap->width,bkup_bitmap->height);
    clear_bitmap(globalPref->mask_bitmap);
  }

  ang*=3.1415926/180.0;

  // include more points for background subtraction
  if (fdraw==1) {
    sc=1.5;
    if (radius<2) sc=2.0;
  }
  else sc=1.0;

  /* *** new code, 05/02/2006 ***
     alternatively, the mask functions can be utilized to first define which pixels 
     to operate on and then bin them into the line
     ******* */
  
  sa=(float *)malloc((2*len+3)*sizeof(float));
  wa=(float *)malloc((2*len+3)*sizeof(float));
  sba=(float *)malloc((2*len+3)*sizeof(float));
  wba=(float *)malloc((2*len+3)*sizeof(float));
  for (ix=0;ix<2*len+3;ix++) sa[ix]=wa[ix]=sba[ix]=wba[ix]=0;

  /* each pixel within the extraction line must be within 
     1. strip parallel to the line
     2. two vertical lines
               fabs(x-xc)<len*fabs(cos(ang))+2*radius*fabs(sin(ang))
     3. two horizontal lines
               fabs(y-yc)<len*fabs(sin(ang))+2*radius*fabs(cos(ang))

     fabs around cos and sin because ang and -ang should give the same results
     also ang and Pi+ang
  */

  tfx=len*fabs(cos(ang))+2*radius*sc*fabs(sin(ang))+2;
  tfy=len*fabs(sin(ang))+2*radius*sc*fabs(cos(ang))+2;  
  for (ix=xc-tfx;ix<=xc+tfx;ix++) {
    if (ix<0 || ix>=dataset->width) continue;
    for (iy=yc-tfy;iy<=yc+tfy;iy++) {
      if (iy<0 || iy>=dataset->height) continue;

      // first establish which pixels to include
      xf=cos(ang)*(ix-xc)-sin(ang)*(iy-yc);
      yf=sin(ang)*(ix-xc)+cos(ang)*(iy-yc);

      xf=floorf(xf*100000+0.5)*0.00001;
      yf=floorf(yf*100000+0.5)*0.00001;

      //printf("(%4d,%4d) -- (%f,%f) \n",ix,iy,xf,yf);
      if (fabs(xf)>len || fabs(yf)>radius*sc) {  // out of range
	//printf("\n");
	continue; 
      }
      wtx=xf-floorf(xf+0.5);
      idx=(int)floorf(xf+0.5)+len+1;
      //printf("%d\n",idx);
      if (fabs(yf)>radius*sc-1) wty=radius*sc-fabs(yf); // pixels on the boundary
      else wty=1;

      if (fabs(yf)>radius) {  // background
	sba[idx]+=dataset->data[ix+iy*dataset->width]*wty;
	wba[idx]+=wty;
	//printf("B\n");
      } else {  // data to extract
	wa[idx]+=wtx;
	wa[idx+1]+=(1-wtx);
	sa[idx]+=dataset->data[ix+iy*dataset->width]*wtx;
	sa[idx+1]+=dataset->data[ix+iy*dataset->width]*(1.0-wtx);
	//printf("D\n");
	if (fdraw==2) set_bit(globalPref->mask_bitmap,ix+dataset->width*iy);
      }	
    }
  }

  if (fname) {
    fprintf(out,"# index    intensity   q or (qr,qz)\n");
    fprintf(out,"# center=(%5.1f, %5.1f),  len=%d,  radius=%4.1f\n",xc,yc,len,radius);
  }

  *sm=0;
  for (idx=1;idx<=2*len+1;idx++) {  // dist = floorf(xf)+len+1
    if (wa[idx]>0) {
      sa[idx]/=wa[idx];
      if (wba[idx]>0) sa[idx]-=sba[idx]/wba[idx];
      *sm+=sa[idx];

      if (fname) {
	// now calculate the corresponding q
	ix=idx-(len+1);
	xf=xc+cos(ang)*ix;
	yf=yc-sin(ang)*ix;
	if (globalPref->samGeometry) {
	  if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (ckbtnDetRec))) {
            qr=dataset->x0+dataset->dx*xf;
            qz=dataset->y0-dataset->dy*yf;
          } else q=pos_to_qrz(xf,yf,&qr,&qz);
	  fprintf(out,"%5d   %8.2f   %7.4f   %7.4f\n",ix,sa[idx],qr,qz);
	} else { 
	  q=pos_to_q(xf,yf);
	  fprintf(out,"%5d   %8.2f   %7.4f\n",ix,sa[idx],q);
	}
      }
    }
  }

  fprintf(out,"# total counts: %f\n",*sm);

  free(wba);
  free(sba);
  free(wa);
  free(sa);

  if (fdraw==2) {
    // draw the line
    bkup_mask=globalPref->show_mask;
    globalPref->show_mask=1;
    if (dataset->status>DATA_STATUS_LOADED) dataset->status=DATA_STATUS_LOADED;
    gtk_widget_queue_draw(da2DView);  
    while (gtk_events_pending ()) gtk_main_iteration ();
    
    // wait
    g_usleep(1000000);    

    // restore bitmap, get rid of the line
    free(globalPref->mask_bitmap);
    globalPref->mask_bitmap=bkup_bitmap;
    globalPref->show_mask=bkup_mask;
    if (dataset->status>DATA_STATUS_LOADED) dataset->status=DATA_STATUS_LOADED;
    gtk_widget_queue_draw(da2DView);  
    while (gtk_events_pending ()) gtk_main_iteration ();
  } 

  if (fname) {
    sprintf(obuf,"line profile extracted to %s.\n",fname);
    send_info_message(obuf);
    fclose(out);
  }

}

void 
on_cbUpdate_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  switch ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (togglebutton) ) ) {
  case TRUE: 
    globalPref->update=1;
    break;
  case FALSE:
    globalPref->update=0;
  }
  updateImage();

}

gboolean updateImage() {
  struct stat sb;
  static time_t mtime=0;
  static int fd=-1;
  char txt[256];
  char *p;

  //printf("in updateImage()\n");

  if (globalPref->update==0) { 
    mtime=0;
    return FALSE;  // load new file will reset update to 0
  }

  // get filename from the "recent" list
  strncpy(filename,p=gtk_combo_box_get_active_text((GtkComboBox *)cbxRecent),255);
  free(p);
  fd = open(filename, OPEN_FILE_FLAG);
  //printf("%s\n",filename);
  if (fd==-1) {
    sprintf(txt,"Error reading image at %s",ctime(&mtime));
    send_info_message(txt);
    return FALSE;
  }

  // check file, load image if changed
  fstat(fd,&sb);
  //printf("file size: %d\n",sb.st_size);
  if (mtime!=sb.st_mtime) {
    //printf("file changed\n");
    mtime=sb.st_mtime;
    g_usleep(500000);
    //lseek(fd,0,SEEK_SET);
    //printf("read result: %d\n",read_data(fd, &sto[6], 1)); /**/
    if (!read_data(fd, &sto[6], 1)) 
      sprintf(txt,"Image updated at %s",ctime(&mtime));
    else 
      sprintf(txt,"Error reading image at %s",ctime(&mtime));
    send_info_message(txt);
    close(fd);
    globalPref->update=1;
  } else close(fd);
  gtk_widget_queue_draw(da2DView);

  // set up timer for next load
  g_timeout_add(1500,(GSourceFunc)updateImage, NULL);
  // need glib > 2.14 to use add_seconds
  //g_timeout_add_seconds(1,(GSourceFunc)updateImage, NULL);
  return FALSE;

}

void
on_dezinger_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  switch (globalPref->dezinger 
	  = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (button)) ) {
  case 0:
    send_info_message ("dezinger is now off.\n");
    break;
  case 1:
    send_info_message ("dezinger is now on.\n");
  }
  
}

void
on_mask_file_changed                   (GtkComboBox     *combobox,
                                        gpointer         user_data) 
{
  int fd,selection,show;

  selection = gtk_combo_box_get_active(combobox);
  show=1;

  if (dataset->status<DATA_STATUS_DRAWN) {
    gtk_combo_box_set_active(combobox,0);  // no mask without data
    return;
  }

  switch (selection) {
  case 0: // none, remove mask
    show=0;
    break;
  case 1: // load file, add entry if load successful 
    // get a filename, assign to mask_fn 
    fd = get_a_file( O_RDONLY, "Open mask file" );
    if (fd>0) {
      strncpy(globalPref->mask_fn,&filename[0],255);
      globalPref->mask_fn[255]=0;
      close(fd);
    }
    break;
  // clear list is not implemented
  //case 2: // clear list
  //  show=0;
  //  gtk_combo_box_set_active(combobox,0);
  //  
  //  break;
  default: // existing mask files, remove entry if load not successful 
    // copy entry to mask_fn
    strncpy(globalPref->mask_fn,gtk_combo_box_get_active_text(combobox),255);
    globalPref->mask_fn[255]=0;
  }

  if (show) {
    if (load_mask()>0) { // load successful
      if (selection==1) { // add entry to the drop list
        gtk_combo_box_insert_text(combobox, 2, &filename[0]);
	gtk_combo_box_set_active(combobox, 2);
      }
      show=1;
    } else { // load failed
      if (selection>1) { // remove entry from the drop list
        gtk_combo_box_remove_text(combobox,selection);
      }
      gtk_combo_box_set_active(combobox,0);
      show=0;
    }
  }

  if (show==0) {
    globalPref->mask_fn[0]=0;
    globalPref->show_mask=0;
  } else 
    globalPref->show_mask=1;

  if (dataset->status>DATA_STATUS_LOADED) dataset->status=DATA_STATUS_LOADED;
  gtk_widget_queue_draw(da2DView);  
}
/*
void
on_btLoadFlat_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  int fd;

  fd = get_a_file( OPEN_FILE_FLAG, "Open flat field data" );
  if (fd>0) {
    if (read_data(fd, &(globalPref->flatfield), 0)==0 
	&& globalPref->flatfield.avg>FLAT_MIN) { 
      // refuse correction if the statistics is bad
      gtk_entry_set_text(GTK_ENTRY(etFlatFile), &filename[0]);
       gtk_widget_set_sensitive (ckbtnFlat, TRUE);
    } else {
      globalPref->flatfield.status = DATA_STATUS_EMPTY;
      gtk_widget_set_sensitive (ckbtnFlat, FALSE);
    }
    close(fd);
  }
}
*/
/*
void on_flat_toggled (GtkWidget *widget, gpointer user_data) {
  
  switch ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (widget) ) ) {
  case TRUE: 
    globalPref->use_ffield=1;
    break;
  case FALSE:
    globalPref->use_ffield=0;
  }
}
*/

void
on_ffield_file_changed                 (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
  int fd,use,selection;

  use=1;
  selection = gtk_combo_box_get_active(combobox);

  if (dataset->status<DATA_STATUS_DRAWN) {
    gtk_combo_box_set_active(combobox,0);  // no mask without data
    return;
  }

  switch (selection) {
  case 0: // none, no flat field correction
    use=0;
    break;
  case 1: // load file, add entry if load successful 
    // get a filename  
    fd = get_a_file( OPEN_FILE_FLAG, "Open flat field file" );
    break;
  // clear list is not implemented
  //case 2: // clear list
  //  use=0;
  //  gtk_combo_box_set_active(combobox,0);
  //  
  //  break;
  default: // existing flat field files, remove entry if load not successful 
    // copy entry to mask_fn
    strncpy(&filename[0],gtk_combo_box_get_active_text(combobox),255);
    fd = open(filename,OPEN_FILE_FLAG,0);
    if (fd<0) use=0; // load unsuccessful
  }

  if (use) {
    if (read_data(fd, &(globalPref->flatfield), 0)==0 
       && globalPref->flatfield.avg>FLAT_MIN) { // everything works fine
      if (selection==1) { // newly loaded file, add entry to the drop list
        gtk_combo_box_insert_text(combobox, 2, &filename[0]);
	gtk_combo_box_set_active(combobox, 2);
      }
    } else  // flat field data is no good
      use=0;
    close(fd);
  }

  if (use==0) { // selected none or no valid flat field data found
    if (selection>1) { // remove entry from the drop list
      gtk_combo_box_remove_text(combobox,selection);
    }
    gtk_combo_box_set_active(combobox,0);
    globalPref->use_ffield=0;
  } else 
    globalPref->use_ffield=1;

  // change will take effect next when data is loaded

  //if (dataset->status>DATA_STATUS_LOADED) dataset->status=DATA_STATUS_LOADED;
  //gtk_widget_queue_draw(da2DView);  

}

void
on_distortion_file_changed             (GtkComboBox     *combobox,
                                        gpointer         user_data) {
}


void
change_sample_geometry                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {

  int choice;

  choice = gtk_toggle_button_get_active (togglebutton);

  if (globalPref->samGeometry == choice) return;
  
  switch (globalPref->samGeometry = choice) {
  case 0: // powder
    send_info_message ("Powder sample, will only show q.\n");
    gtk_widget_set_sensitive (etSamNormal, FALSE);
    gtk_widget_set_sensitive (etIncidentAng, FALSE);
    break;
  case 1: // aligned
    send_info_message ("Aligned sample, will show qr and qz.\n");
    gtk_widget_set_sensitive (etSamNormal, TRUE);
    gtk_widget_set_sensitive (etIncidentAng, TRUE);    
    break;
  }
  // redraw the normal vector
  if (dataset->status == DATA_STATUS_DRAWN)  dataset->status = DATA_STATUS_NEED_DECOR;
  gtk_widget_queue_draw(da2DView); 

}

void
load_recent_image                      (GtkComboBox     *combobox,
                                        gpointer         user_data) 
{
  int fd,selection;
  //static int flag=0;
  char *p;

  //if (flag==1) {
  //  flag=0;
  //  return;
  //}

  p=gtk_combo_box_get_active_text(combobox);
  if (!p) return;
  strncpy(filename,p,255);
  free(p);
  selection = gtk_combo_box_get_active(combobox);

  fd = open(filename, OPEN_FILE_FLAG);
  read_data(fd, &sto[6], 1); /**/
  close(fd);
  send_info_message ("Done.\n");
  
  if (selection) {
      gtk_combo_box_remove_text(combobox,selection);
      gtk_combo_box_prepend_text(combobox,filename);
      gtk_combo_box_set_active((gpointer)cbxRecent,0);
  };

  gtk_widget_queue_draw(da2DView);
}

void
save_Parameter_set                     (GtkButton       *button,
                                        gpointer         user_data)
{
  FILE *outfile;
  char buf[256];  
  int cur;

  cur = gtk_spin_button_get_value_as_int(sbParaSet);
  sprintf(buf,"%s/.view.gtk/conf.%d",getenv(HOME_ENV),cur);
  outfile = fopen(buf,"w+");
  if (outfile) {
    fprintf(outfile,"# view.gtk-conf\n");
    fprintf(outfile,"%5.3f   # wavelength \n",globalPref->wavelength);
    fprintf(outfile,"%5.3f   # D/d\n",globalPref->ratioDd);
    fprintf(outfile,"%5.1f   # detector_psi \n",globalPref->detector_psi);
    fprintf(outfile,"%5.1f   # cx \n",globalPref->cx);
    fprintf(outfile,"%5.1f   # cy \n",globalPref->cy);
    fprintf(outfile,"%5.1f   # sample normal \n",globalPref->normal);
    fprintf(outfile,"%5.1f   # incident \n",globalPref->incident);
    fprintf(outfile,"%5.1f   # detector orient \n",globalPref->detector_orient);
    fprintf(outfile,"%5.1f   # detector tilt \n",globalPref->detector_tilt);
    fprintf(outfile,"%5.1f   # detector phi \n",globalPref->detector_phi);
    fprintf(outfile,"%5.1f   # detector tilt x\n",globalPref->beam_tilt_x);
    fprintf(outfile,"%5.1f   # detector tilt y\n",globalPref->beam_tilt_y);
    fclose(outfile);
  } else send_info_message ("could not open config file to write.");

}

void
load_Parameter_set                     (GtkButton       *button,
                                        gpointer         user_data)
{
  FILE *infile;
  char buf[256];  
  int cur;

  cur = gtk_spin_button_get_value_as_int(sbParaSet);
  sprintf(buf,"%s/.view.gtk/conf.%d",getenv(HOME_ENV),cur);

  infile=fopen(buf,"r");
  if (!infile) {
    send_info_message ("could not open config file.");
    return;
  }

  fgets(buf,255,infile);
  if (strncmp(buf,"# view.gtk-conf",15)==0) { // need some sort of token as proof of valid file format
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->wavelength);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->ratioDd);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->detector_psi);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->cx);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->cy);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->normal);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->incident);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->detector_orient);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->detector_tilt);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->detector_phi);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->beam_tilt_x);
    fgets(buf,255,infile);
    sscanf(buf,"%f",&globalPref->beam_tilt_y);

    setValue(etWavelength, "%5.3f", globalPref->wavelength);
    setValue(etDd, "%5.3f", globalPref->ratioDd);
    //setValue(etDetSwing, "%5.1f", globalPref->detector_psi);
    setValue(etCenX, "%5.1f", globalPref->cx);
    setValue(etCenY, "%5.1f", globalPref->cy);
    setValue(etSamNormal, "%5.1f", globalPref->normal);
    setValue(etIncidentAng, "%5.1f", globalPref->incident);
    setValue(etDetOrient, "%5.1f", globalPref->detector_orient);
    setValue(etDetTilt, "%5.1f", globalPref->detector_tilt);
    setValue(etDetPhi, "%5.1f", globalPref->detector_phi);
    setValue(etBeamTiltX, "%5.1f", globalPref->beam_tilt_x);
    setValue(etBeamTiltY, "%5.1f", globalPref->beam_tilt_y);
    send_info_message ("config file read successfully.\n");
  }
  fclose(infile);

  calc_rot(globalPref);

  on_editing_done(NULL,GINT_TO_POINTER(256));
}

void show_peak(char *filename) {

  // take image/dataset
  // examine each pixel that has more counts than the thershold
  // a peak pixel must be higher than all it nearest neigbors
  // a peak pixel should also be significantly higher than all distant neighbors
  // the peak position is weighted center of a block of pixels

  // a zinger is necessarily a peak, and only one peak 
  // a pixel on a powder ring or a scattering rod should not be identified as a peak

  // should have an option of searching only within the visible area
  // should skip masked pixels

  int ix,iy,ix1,iy1,flag,dist,nn;
  unsigned int thresh,index,index1,ct,ct1;
  DTYPE t,max,min;
  float fx,fy,weight,weight2,avg,std,q,qr,qz,tol;
  char textbuf[256];
  FILE *out;

  thresh = getValue(etPeakThresh);
  if (thresh<0) return;

  setValue(etPeakThresh, "%d", thresh);

  dist = getValue(etZingerRadius);
  if (dist<1) dist=5; // definition of distant pixels 
  setValue(etZingerRadius, "%d", dist);
  globalPref->zr = dist;

  tol = getValue(etZingerTolerance);
  if (tol<0) tol=3.0;
  setValue(etZingerTolerance, "%.1f", tol);
  globalPref->zt = tol;  

  nn=3;   // definition of nearest neighbors
  globalPref->peak_N=0;

  out=fopen(filename,"w+");
  if (!out) {
    sprintf(textbuf,"Cannot open file to write: %s.",filename);
    send_info_message(textbuf);
    return;
  }
  
  for (ix=0;ix<dataset->width;ix++) {
    for (iy=0;iy<dataset->height;iy++) {
      index = ix+iy*dataset->width;
      if ((t=dataset->data[index])<thresh) continue;
      
      // now check nearest neighbors
      flag=-1;
      weight=weight2=0;ct=ct1=0;
      for (ix1=ix-nn;ix1<=ix+nn;ix1++) {
	if (ix1<0 || ix1>=dataset->width) continue;
	for (iy1=iy-nn;iy1<=iy+nn;iy1++) {
	  if (iy1<0 || iy1>=dataset->height) continue;
	  index1 = ix1+iy1*dataset->width;
	  if (dataset->data[index]<dataset->data[index1]
	      || (dataset->data[index]==dataset->data[index1] && index<index1) ) 
	    {flag=1;break;}
	  if (index!=index1) {
	    weight+=dataset->data[index1];
	    weight2+=dataset->data[index1]*dataset->data[index1];
	    ct++;
	  }
	}
	if (flag>0) break;
      }
      if (flag>0) continue; // lower than one of its nearest neighbor

      // peak should have finite width, low std. dev. within the nearest neighbors
      //if (ct<nn) continue;
      avg=weight/ct;
      std=sqrt(weight2/ct-avg*avg);
      if (dataset->data[index]-avg>std*tol) continue;
      //printf("(%d, %d): %f, ct=%d, avg=%f, std=%f\n",ix,iy,dataset->data[index],ct,avg,std);

      // now check distant neighbors, dist pixels from the center 
      // get max, min, 
      max=0;
      min=65535;
      avg=0;
      weight=avg=0;
      iy1=iy-dist;
      if (iy1>=0) {
	for (ix1=ix-dist;ix1<=ix+dist;ix1++) {
	  if (ix1<0 || ix1>=dataset->width) continue;
	  index1 = ix1+iy1*dataset->width;
	  if (dataset->data[index1]<min) min=dataset->data[index1];
	  else if (dataset->data[index1]>max) max=dataset->data[index1];
	  weight++;
	  avg+=dataset->data[index1];
	}
      }
      if (max>=dataset->data[index]) continue; // not a peak
      iy1=iy+dist;
      if (iy1<dataset->height) {
	for (ix1=ix-dist;ix1<=ix+dist;ix1++) {
	  if (ix1<0 || ix1>=dataset->width) continue;
	  index1 = ix1+iy1*dataset->width;
	  if (dataset->data[index1]<min) min=dataset->data[index1];
	  else if (dataset->data[index1]>max) max=dataset->data[index1];
	  weight++;
	  avg+=dataset->data[index1];
	}
      }
      if (max>=dataset->data[index]) continue; // not a peak
      ix1=ix-dist;
      if (ix1>=0) {
	for (iy1=iy-dist;iy1<=iy+dist;iy1++) {
	  if (iy1<0 || iy1>=dataset->height) continue;
	  index1 = ix1+iy1*dataset->width;
	  if (dataset->data[index1]<min) min=dataset->data[index1];
	  else if (dataset->data[index1]>max) max=dataset->data[index1];
	  weight++;
	  avg+=dataset->data[index1];
	}
      }
      if (max>=dataset->data[index]) continue; // not a peak
      ix1=ix+dist;
      if (ix1<dataset->width) {
	for (iy1=iy-dist;iy1<=iy+dist;iy1++) {
	  if (iy1<0 || iy1>=dataset->height) continue;
	  index1 = ix1+iy1*dataset->width;
	  if (dataset->data[index1]<min) min=dataset->data[index1];
	  else if (dataset->data[index1]>max) max=dataset->data[index1];
	  weight++;
	  avg+=dataset->data[index1];
	}
      }
      if (max>=dataset->data[index]) continue; // not a peak
      if (weight>0) avg/=weight;
      // this needs to be re-considered
      //if (dataset->data[index]-avg<tol*(max-avg)+0.3*thresh) continue;
      if (dataset->data[index]-avg<thresh) continue;
      //if (sqrt(dataset->data[index]-avg)<(max-avg)) continue; 

      // now calculate the weighted center position
      weight=0; 
      fx=fy=0;
      flag=0;
      for (ix1=ix-2*nn;ix1<=ix+2*nn;ix1++) {
	if (ix1<0 || ix1>=dataset->width) continue;
	for (iy1=iy-2*nn;iy1<=iy+2*nn;iy1++) {
	  if (iy1<0 || iy1>=dataset->height) continue;
	  index1 = ix1+iy1*dataset->width;
	  if (dataset->data[index1]>dataset->data[index]) {
	    flag=1;
	    break;
	  }
	  //weight += dataset->data[index1]-avg;
	  //fx+= (float)ix1*(dataset->data[index1]-avg);
	  //fy+= (float)iy1*(dataset->data[index1]-avg);
	  weight += dataset->data[index1];
	  fx+= (float)ix1*(dataset->data[index1]);
	  fy+= (float)iy1*(dataset->data[index1]);
 	}
	if (flag) break;
      }
      fx/=weight;
      fy/=weight;

      // now should record the peak position somehow
      //printf("possible peak: (%4d, %4d), max=%f, min=%f, avg=%f\n",ix,iy,max,min,avg);
      
      //fprintf(out,"(%d,%d)->",ix,iy);
      if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (ckbtnDetRec))) 
	{ // data displayed is already converted into reciprocal space
	  qr=Qr0+dQ*fx;
	  qz=Qz0-dQ*fy;
	  sprintf(&textbuf[0],"(%6.1f, %6.1f), q=(%6.3f, %6.3f) : %10.2f",
		  fx,fy,qr,qz,weight);
	}
      else {
	if (globalPref->samGeometry) 
	  {
	    q=pos_to_qrz(fx,fy,&qr,&qz);
	    sprintf(&textbuf[0],"(%6.1f, %6.1f) : %10.2f, q=%6.4f (%6.4f, %6.4f)",
		    fx,fy,weight,q,qr,qz);
	    //qrz_to_pos(qr,qz,&tx,&ty);
	    //printf("(%4d, %4d) -> (%6.2f, %6.2f)",x,y,tx,ty);
	  } else { 
	  q=pos_to_q(fx,fy);
	  sprintf(&textbuf[0],"(%6.1f, %6.1f) : %10.2f,    q=%6.4f",
		  fx,fy,weight,q);
	}
	fprintf(out,"%s\n",&textbuf[0]);
	globalPref->peak_x[globalPref->peak_N]=fx;
	globalPref->peak_y[globalPref->peak_N]=fy;
	globalPref->peak_N++;
	if (globalPref->peak_N==256) return;  // can only remember up to 256 peaks
      }
    }
  }
  fclose(out);
}

void
on_showpeak_clicked                    (GtkButton *button,
                                        gpointer   user_data)
{
  char buf[256];
  unsigned int thresh;

  if (dataset->status<=DATA_STATUS_INDEXED) return;  // no displayed data

  thresh = getValue(etPeakThresh);
  if (thresh<50) { // turn peaks off
    dataset->status = DATA_STATUS_INDEXED;
    globalPref->showpeak=0;
    gtk_widget_queue_draw(da2DView);
    return;
  }

  sprintf(buf,"%s/view-data.pks",getenv(HOME_ENV));
  show_peak(buf);

  if (globalPref->peak_N) {
    // need to redraw the reference pattern for a different geometry
    if (dataset->status == DATA_STATUS_DRAWN) dataset->status = DATA_STATUS_NEED_DECOR;
    globalPref->showpeak=1;
    gtk_widget_queue_draw(da2DView);
  }

}


void
config_DetGeo(GtkButton *button, gpointer user_data) {
  gtk_widget_show (dlgDetGeo);
  gtk_grab_add(dlgDetGeo);
}


void
DetGeo_config_done(GtkButton *button, gpointer user_data) {
  float v;

  // update parameters
  v=getValue(etDetOrient);
  globalPref->detector_orient=v;
  setValue(etDetOrient, "%5.1f",v);
  v=getValue(etDetTilt);
  globalPref->detector_tilt=v;
  setValue(etDetTilt, "%5.1f",v);
  v=getValue(etDetPhi);
  globalPref->detector_phi=v;
  setValue(etDetPhi, "%5.1f",v);
  v=getValue(etBeamTiltX);
  globalPref->beam_tilt_x=v;
  setValue(etBeamTiltX, "%5.1f",v);
  v=getValue(etBeamTiltY);
  globalPref->beam_tilt_y=v;
  setValue(etBeamTiltY, "%5.1f",v);

  gtk_grab_remove(dlgDetGeo);
  gtk_widget_hide (dlgDetGeo);
  calc_rot(globalPref);

  // same code as in on_editing_done
  if (globalPref->refPattern || globalPref->samGeometry) {
    // need to redraw the reference pattern for a different geometry
    if (dataset->status == DATA_STATUS_DRAWN) dataset->status = DATA_STATUS_NEED_DECOR;
  }
  // q_available=2; // original, don't know why
  q_available=0;
  if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (ckbtnDetRec))) 
    toggle_DetRec(GTK_TOGGLE_BUTTON (ckbtnDetRec),0);
  else 
    gtk_widget_queue_draw(da2DView);
 

}
