#include <unistd.h>
#include <sys/types.h>
//#include <linux/unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <math.h>
#include <glib.h>
#include "marCCD.h"
#include "princeton.h"
#include "data.h"
#include "support.h"
#include "callbacks.h"
#include "bitmap.h"
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <errno.h>
#include <assert.h>
#include <string.h>
#include <tiff.h>
#include <tiffio.h>

#define DATA_LEN 8192

extern char * scale_xpm[];
extern GtkWidget *w2DView;
extern GtkWidget *wControl;
extern GtkWidget *dlgDetGeo;
extern stPreference *globalPref;
extern int errno;
extern int q_available;

extern int WINDOW_SIZE;

typedef union {
  unsigned int n;
  char c[4];
} NC;

char default_color[5][3] = {
  {  0,   0,  65},
  {189,   2, 107},
  {255,  55,   0},
  {255, 255,   0},
  {255, 255, 255}};

char datafilename[256];

GtkWidget *btColor[5],*daColorTable,*sclContrast,*sclBrightness,*cbtnLog;
GtkWidget *etGamma,*etZMax,*etZMin,*tbAutoZScale,*daColorTable,*da2DView;
GtkWidget *tvInfo,*tvFileList,*lbStatusMsg;
GtkWidget *etWavelength, *etDd, *etDetSwing, *etCenX, *etCenY;
GtkWidget *etDetOrient, *etDetTilt, *etDetPhi,*etBeamTiltX,*etBeamTiltY;
GtkWidget *etSamNormal,*etIncidentAng, *et1DGrid, *etScaleFactor, *etConst, *etSetComments;
GtkWidget *etLineProfileCx,*etLineProfileCy,*etLineProfileLength;
GtkWidget *etLineProfileOrient,*etLineProfileRadius;
GtkWidget *cbxZoom,*ckbtnGeo,*cbxRef,*cbxMask,*cbxFlatField,*cbxDistortion,*cbxRecent;
GtkSpinButton *sbParaSet, *sbCurSet, *sbSet1, *sbSet2;
GtkWidget *etPeakThresh, *etZingerRadius, *etZingerTolerance;
GtkWidget *ckbtnDetRec;
GtkTextBuffer *tbuf_info;
DataSet *dataset,sto[8];
GdkColor colors[8]={
  {0,65535,65535,65535},
  {0,65535,    0,    0},
  {0,    0,65535,    0},
  {0,    0,    0,65535},
  {0,    0,65535,65535},
  {0,65535,    0,65535},
  {0,65535,65535,    0},
  {0,35535,35535,35535}
};
//StoredData sto[8];

static struct {
  char type;
  float center_x,center_y;
  float R,a,b,tilt;
} mask[128];  

int load_mask() {
  int i,n,x,y,x1,x2,y1,fd;
  float xf,yf;
  float ft;
  char buf[256];
  FILE *mask_file;

  if (globalPref->mask_fn[0]==0) return(-1);

  n=0;
  if (dataset->status<DATA_STATUS_LOADED) return(0);
  clear_bitmap(globalPref->mask_bitmap);
  mask_file=fopen(globalPref->mask_fn,"r");

  if (!mask_file) {
    printf("errno = %d\n",errno);
    return(-1);
  }

  for (i=0;i<128;i++) {
    buf[0]=0;
    if (feof(mask_file)) break;
    fgets(buf,255,mask_file);
    if (buf[0]==0) break;
    switch (buf[0]) {
    case 'h':
      sscanf(buf,"%c %f %f %f",&mask[n].type,
	     &mask[n].center_x,
	     &mask[n].center_y,
	     &mask[n].R);
      for (x=0;x<dataset->width;x++) {
	for (y=0;y<dataset->height;y++) {
	  ft=(mask[n].center_x-x)*(mask[n].center_x-x)+
	    (mask[n].center_y-y)*(mask[n].center_y-y);
	  if (ft>=mask[n].R*mask[n].R)
	    set_bit(globalPref->mask_bitmap,x+dataset->width*y);
	}
      }
      n++;
      break;
    case 'f':  // fan
      sscanf(buf,"%c %f %f %f %f",&mask[n].type,
	     &mask[n].center_x, &mask[n].center_y,
	     &mask[n].a, &mask[n].b);
      mask[n].b+=mask[n].a;
      //printf("%6.2f   %6.2f\n",mask[n].a,mask[n].b);
      mask[n].a*=Pi/180;
      mask[n].b*=Pi/180;
      for (y=0;y<dataset->height;y++) {
	x1=(int)(mask[n].center_x+(mask[n].center_y-y)*tan(mask[n].a)+0.5);
	x2=(int)(mask[n].center_x+(mask[n].center_y-y)*tan(mask[n].b)+0.5);
	if (x1<0) x1=0;
	if (x2<0) x2=0;
	if (x1>=dataset->width) x1=dataset->width-1;
	if (x2>=dataset->width) x2=dataset->width-1;
	if (x1>x2) {x=x1;x1=x2;x2=x;}
	if (cos(mask[n].a)*cos(mask[n].b)>0) {
	  for (x=x1;x<=x2;x++) 
	    set_bit(globalPref->mask_bitmap,x+dataset->width*y);
	} else {
	  for (x=0;x<=x1;x++)
	    set_bit(globalPref->mask_bitmap,x+dataset->width*y);
	  for (x=x2;x<dataset->width;x++)
	    set_bit(globalPref->mask_bitmap,x+dataset->width*y);
	}
      }
      break;
    case 'c':
      sscanf(buf,"%c %f %f %f",&mask[n].type,
	     &mask[n].center_x,
	     &mask[n].center_y,
	     &mask[n].R);
      /*
      printf("cir: (%7.2f,%7.2f) w=%7.2f)",
	     mask[n].center_x,
	     mask[n].center_y,
	     mask[n].R);
      */
      for (x=mask[n].center_x-mask[n].R;x<=mask[n].center_x+mask[n].R;x++) {
	for (y=mask[n].center_y-mask[n].R;y<=mask[n].center_y+mask[n].R;y++) {
	  if (x<0 || x>=dataset->width || y<0 || y>=dataset->height) continue;
	  ft=(mask[n].center_x-x)*(mask[n].center_x-x)+
	    (mask[n].center_y-y)*(mask[n].center_y-y);
	  if (ft<=mask[n].R*mask[n].R)
	    set_bit(globalPref->mask_bitmap,x+dataset->width*y);
	}
      }

      n++;
      break;
    case 'r':
      sscanf(buf,"%c %f %f %f %f %f",&mask[n].type,
	     &mask[n].center_x, &mask[n].center_y,
	     &mask[n].a, &mask[n].b, 
	     &mask[n].tilt);
      mask[n].tilt*=3.1415926/180;
      mask[n].R = sqrt(mask[n].a*mask[n].a + mask[n].b*mask[n].b)/2;
      /*
      printf("rect: (%7.2f,%7.2f) w=%7.2f  h=%7.2f  tilt=%7.2f \n",
	    mask[n].center_x,mask[n].center_y,
	    mask[n].a,mask[n].b,mask[n].tilt);
      */
      for (x=mask[n].center_x-mask[n].R;x<=mask[n].center_x+mask[n].R;x++) {
	for (y=mask[n].center_y-mask[n].R;y<=mask[n].center_y+mask[n].R;y++) {
	  if (x<0 || x>=dataset->width || y<0 || y>=dataset->height) continue;
	  xf=cos(mask[n].tilt)*(x-mask[n].center_x)
	    -sin(mask[n].tilt)*(y-mask[n].center_y);
	  yf=sin(mask[n].tilt)*(x-mask[n].center_x)
	    +cos(mask[n].tilt)*(y-mask[n].center_y);
	  if (fabs(xf)*2<mask[n].a && fabs(yf)*2<mask[n].b) {
	    set_bit(globalPref->mask_bitmap,x+dataset->width*y);
	  }
	}
      }

      n++;
      break;
    default:
      sprintf(buf,"Invalid mask entry: %s",buf);
      send_info_message(buf);
    }
  }  

  if (n==0) return (n);
  fclose(mask_file);

  return (globalPref->Nmask=n); 
}

int read_data(int fd, DataSet *ds, gboolean show) {
  // show=0 when loading flat field
  int ret,i,j,count;
  float s;
  //DataSet *ds;

  if (globalPref->update) globalPref->update=0;

  ds->status=DATA_STATUS_EMPTY;

  // need to calculate new q data
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ckbtnDetRec), q_available=0);

  switch (globalPref->DataFormat) {
  case 0: // Mar CCD
    ret = load_marCCD_tiff(fd, ds);
    break;
  case 1: // WAXS tiff
    ret = load_WAXS_tiff(fd, ds);
    break;
  case 2: // PILATUS tiff
    ret = load_pilatus_tiff(fd, ds);
    break;
  case 3: // raw binary, format specified in HOME_ENV/.view.gtk/data_format
    ret = load_raw_binary(fd, ds);
    break;
  case 4: // Bruker
    ret = load_Bruker_binary(fd, ds);
    break;
  case 5: // princeton/Roper
    ret = load_Roper_binary(fd, ds);
  }

  if (ret || ds->data==NULL) {
    send_info_message("Failed to load data.");
    return(ret);
  }
  
  globalPref->N_dq=1;
  
  ds->zmax=-1; // require updating zinger_map and zmax/zmin
  return(post_loading_data(ds,show));
}

// check whether pixel (xx,yy) in the data is within the viewport
int check_visible(int xx, int yy) {
  static int xx_min,xx_max,yy_min,yy_max;
  int window_width, window_height;
  
  // invalid position force the re-calculation of limits
  gtk_window_get_size ( GTK_WINDOW (w2DView), &window_width, &window_height );
  if (xx<0 || yy<0) {
    xx_min = globalPref->viewport_cx - window_width/2/globalPref->zoom;
    xx_max = globalPref->viewport_cx + window_width/2/globalPref->zoom;
    yy_min = globalPref->viewport_cy - window_height/2/globalPref->zoom;
    yy_max = globalPref->viewport_cy + window_height/2/globalPref->zoom;
    return 0;
  }

  if (xx<xx_min || xx>xx_max || yy>yy_max || yy<yy_min) return(0);
  return(1);
}

void find_MM(DataSet *ds, gboolean show) {
  double av=0,sg=0;
  int count,ix,iy,i,tt,zr;

  ds->zmin = 65535;
  ds->zmax = -1;
  count = 0;

  check_visible(-1,-1);

  if (show && globalPref->use_ffield && globalPref->flatfield.status) {
    if (globalPref->flatfield.favg>FLAT_MIN) {
      // do the flat field correction only when the flat image 
      // has better than 10% statistics      
      for (i=0;i<ds->width*ds->height;i++)
	ds->data[i]/=globalPref->flatfield.data[i];
    }
  }

  for (ix=0;ix<ds->width;ix++) {
    for (iy=0;iy<ds->height;iy++) {
      i = ix+iy*dataset->width;
      zr=dezinger(ds,ix,iy,show);
      if (zr) set_bit(ds->zinger_map,i);

      if (show || ds->data[i]>FLAT_MIN && zr==0) {
	av += ds->data[i];
	sg += ds->data[i]*ds->data[i];

	/* full frame
	if (ds->data[i]>ds->zmax && zr==0) ds->zmax = ds->data[i];
	else if (ds->data[i]<ds->zmin && ds->data[i]>0 && zr==0) ds->zmin = ds->data[i];
	*/

	/* within the viewport only */
	if (check_visible(ix,iy)) {
	  if (ds->data[i]>ds->zmax && zr==0) ds->zmax = ds->data[i];
	  else if (ds->data[i]<ds->zmin && ds->data[i]>0 && zr==0) ds->zmin = ds->data[i];
	}

	count++;
      }
    }
  }
  av /= count;
  sg /= count;
  ds->avg = av;
  ds->favg = av;
  ds->sigma = sqrt( sg - av*av ) + 0.5;

  if (show==0 && ds->avg>FLAT_MIN) { // flat field image
    for (ix=0;ix<ds->width;ix++) {
      for (iy=0;iy<ds->height;iy++) {
	i = ix+iy*dataset->width;
	if (ds->data[i]<FLAT_MIN || get_bit(ds->zinger_map,i)) ds->data[i]=1;
	else ds->data[i]/=ds->favg;
      }
    }
  }

}

int post_loading_data(DataSet *ds, gboolean show) { 
  float av;
  int ix,iy,index,w,h;

  if (show) {

    // reallocate the supporting data if image size has changed
    // ds->width and ds->height are reset when reading the data
    // size of the previous data can be found from ds->pixbuf
    if (ds->pixbuf) {
      w=gdk_pixbuf_get_width(ds->pixbuf);
      h=gdk_pixbuf_get_height(ds->pixbuf);
      if (ds->width!=w || ds->height!=h) {
	gdk_pixbuf_unref(ds->pixbuf);
	ds->pixbuf=NULL;
      }
    }
      
    if (ds->indexed_buf && !ds->pixbuf) {
      free(ds->indexed_buf);
      ds->indexed_buf=NULL;
    }
    if (!ds->indexed_buf)
      ds->indexed_buf = (guchar *)malloc(RGBA*sizeof(guchar)*ds->width*ds->height);
    
    if (!ds->pixbuf && globalPref->mask_bitmap) {
      destroy_bitmap(globalPref->mask_bitmap);
      globalPref->mask_bitmap=NULL;
    }
    if (!globalPref->mask_bitmap) {
      globalPref->mask_bitmap = new_bitmap(ds->width,ds->height);
      load_mask();
    }
    
    // new zinger map for newly loaded data
    if (!ds->pixbuf && ds->zinger_map) {
      destroy_bitmap(ds->zinger_map);
      ds->zinger_map=NULL;
    }
    if (!ds->zinger_map) {
      ds->zinger_map=new_bitmap(ds->width,ds->height);
    }
    if (ds->zmax<0) find_MM(ds,show);
    
    if ((globalPref->zmax<0 || 
	 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (tbAutoZScale)) ) ) {
      
      /* old code 
	 av = ds->avg+3*ds->sigma;
	 globalPref->zmax = (av<65535 ? av : 65535);
	 av = ds->avg-3*ds->sigma;
	 globalPref->zmin = (av>0 ? av : 0);
      */
      
      globalPref->zmax = ds->zmax;
      globalPref->zmin = ds->zmin;
      setValue(etZMax, "%5.0f", globalPref->zmax);
      setValue(etZMin, "%5.0f", globalPref->zmin);
    }
  }

  ds->status = DATA_STATUS_LOADED;
  
  return(0);
}

int load_Bruker_binary(int fd, DataSet *ds)   
// int viewCtrl::readfile_header_bin(const char *filename, DataSet2D *dataset) 
  /* data is in binaray format i.e. have to read 'bytewise' */
  /* in case of 2bytes (4bytes) per pixel (-> NPIXELB),     */
  /* the least significant comes first (little endian)      */
{
  int  ovrfl,pixel;
  int  hdr,nvf,bpp,npr,npi,maxcount,npc;
  int  i,j;
  char v1[512];
  char *pntr,*buf;
  NC   nc;
  
  read(fd,v1,512);

  /* -------------- get basic file info   ----------- */
  /*  get  Header size  -> hdr  */
  pntr=strstr(v1,"HDRBLKS");
  if (!pntr) return(-1); // incorrect format
  sscanf(&pntr[8],"%d",&hdr);
  hdr*=512;

  /*   read in all of the Header  -> buf */
  buf=malloc(hdr);
  lseek(fd,0,SEEK_SET);
  read(fd,buf,hdr);
  
  /*   get imageformat (bytes per pixel) -> bpp  */
  pntr=strstr(buf,"NPIXELB");
  sscanf(&pntr[8],"%d",&bpp);

  /*    get imagesize (# of pixelrows and pixelcolums) -> npr,npc  */
  pntr=strstr(buf,"NROWS");
  sscanf(&pntr[8],"%d",&npr);

  pntr=strstr(buf,"NCOLS");
  sscanf(&pntr[8],"%d",&npc);

  npi=npc*npr;
  /*
  if (ds->data && (ds->height!=npr || ds->width!=npc)) 
    {free(ds->data);ds->data=0;}
  */
  ds->height=npr;
  ds->width=npc;
  //if (!ds->data) 
  ds->data=(DTYPE*)malloc(npi*sizeof(DTYPE));  
  
  pntr=strstr(buf,"MAXIMUM");
  sscanf(&pntr[8],"%d",&maxcount);
  
  /*   check if any overflow, if so read overflow table   */
  pntr=strstr(buf,"NOVERFL");
  sscanf(&pntr[8],"%d",&nvf);
  
  free(buf);

  /* allocate data as 32-bit npi pixels per image and im_data 3colors 8*bpp */
  /* then load frame depending on the bytes per pixel format     */
  /* finally read add overflow  */
  //lseek(fd,hdr,SEEK_SET);
  for (i=0;i<npi;i++) {
    nc.n=0;
    read(fd,&nc,bpp);
    ds->data[i]=nc.n;
    //    printf("%6n\t%f\n",i,ds->data[i]);
  }
  
  /* overflow table */
  v1[16]=0;
  if (nvf>0) {
    for (j=0;j<nvf;j++) {
      read(fd,v1,16);
      sscanf(v1,"%d%d",&ovrfl,&pixel);
      if (pixel>0 && pixel<npi) ds->data[pixel]=ovrfl;
      else printf("%s -- %d, %d\n",v1,ovrfl,pixel);
    }
  }

  //printf("nw=%d nh=%d npi=%d   data=%p\n",ds->width, ds->height, npi, ds->data ); 

  return(0);
}

// raw binary, the data format has to be specified in HOME_ENV/.view.gtk/data_format
// this file should read something like this:
// 640 480 16I LL
// size, data type (16I, 32I, and 32F), and byte order (LL)
int load_raw_binary(int fd, DataSet *ds)   
{
  unsigned long int len;
  guint32 ti32;
  DTYPE s;
  guchar *dbuf;
  int i,j,w,h,bpp,binning,pw,ph;
  unsigned int t=0;
  guint16 ti16;
  FILE *fmt;
  float tf32;
  char sbuf[512],dt_str[16],bo_str[16];

  sprintf(sbuf,"%s/.view.gtk/data_format",getenv(HOME_ENV));
  fmt=fopen(sbuf,"r");
  if (!fmt) {
    sprintf(sbuf,"failed to open %s.\n",sbuf);
    send_info_message(sbuf);
    return(-1);
  }

  sbuf[0]=0;
  while (!feof(fmt)) {
    fgets(&sbuf[0],256,fmt);
    if (sbuf[0]==0 || sbuf[0]=='#') continue;
    break;
  }
  fclose(fmt);

  if (sbuf[0]==0) {
    send_info_message("Could not read data format specification.");
    return(-1);
  }

  sscanf(sbuf,"%d %d %3s %2s",&w,&h,dt_str,bo_str);
  sscanf(dt_str,"%2d",&bpp);
  bpp/=8;

  sprintf(sbuf,"data size: %d x %d, data type: %s, bpp = %d, byte order: %s\n",w,h,dt_str,bpp,bo_str);
  send_info_message(sbuf);

  if (strcmp(dt_str,"8I") && strcmp(dt_str,"16I") && strcmp(dt_str,"32I") && strcmp(dt_str,"32F")) {
    sprintf(sbuf,"cannot handle data format: %s\n",dt_str);
    send_info_message(sbuf);
    return(-1);
  }

  if (strcmp(bo_str,"BE") && strcmp(bo_str,"LE")) {
    sprintf(sbuf,"unknown byte order: %s\n",bo_str);
    send_info_message(sbuf);
    return(-1);
  }

  len = lseek(fd, 0, SEEK_END);
  if (w*h*bpp!=len) {
    sprintf(sbuf,"file size (%ld) does not agree with specified format (%d x %d x %dbpp).\n",len,w,h,bpp);
    send_info_message(sbuf);
    return(-1);
  }

  lseek(fd, 0, SEEK_SET);
  ds->height = h;
  ds->width = w;

  ds->data=(DTYPE*)malloc(ds->width*ds->height*sizeof(DTYPE));  
  dbuf = malloc(len);
  read(fd,dbuf,len);
  memset((char *)(ds->data),0,w*h*sizeof(DTYPE));

  if (strcmp(dt_str,"8I")==0) {
    for (i=0;i<w*h;i++) ds->data[i]=dbuf[i];
  } else if (strcmp(dt_str,"16I")==0) {
    if (strcmp(bo_str,"BE")==0) {  // Big Endian
      for (i=0;i<w*h;i++) {
	ti16=((guint16 *)&(dbuf[bpp*i]))[0];
	ds->data[i]=GUINT16_FROM_BE(ti16);
	//if (i%1000==0) printf("%8d  ->  %8d\n",ti16,data[i]);
      }
    } else { // Little Endian
      for (i=0;i<w*h;i++) {
	ti16=((guint16 *)&(dbuf[bpp*i]))[0];
	ds->data[i]=GUINT16_FROM_LE(ti16);
	//printf("%8d  ->  %8d\n",ti16,data[i]);
      }      
    } 
  } else if (strcmp(dt_str,"32I")==0) {
    if (strcmp(bo_str,"BE")==0) {  // Big Endian
      for (i=0;i<w*h;i++) {
	ti32=((guint32 *)&(dbuf[bpp*i]))[0];
	ds->data[i]=GUINT32_FROM_BE(ti32);
      }
    } else { // Little Endian
      for (i=0;i<w*h;i++) {
	ti32=((guint32 *)&(dbuf[bpp*i]))[0];
	ds->data[i]=GUINT32_FROM_LE(ti32);
      }      
    } 
  } else if (strcmp(dt_str,"32F")==0) {
    for (i=0;i<w*h;i++) {
      tf32=((gfloat *)&(dbuf[bpp*i]))[0];
      ds->data[i]=tf32;
    }
  }

  // resize to 1024 x 1024
  // bin the pixels if image is too big
  /*
  binning = 1;
  t=(w>h?w:h);
  while (t>1024) {
    binning*=2;
    t/=2;
  }
  */
  /*
  pw = (ds->width - w/binning)/2;
  ph = (ds->height - h/binning)/2;
  for (i=0;i<w;i++) {
    for (j=0;j<h;j++) {
      ds->data[(j/binning+ph)*ds->width+i/binning+pw]=data[j*w+i]/(binning*binning);
    }
  }
  */
 
  free(dbuf);
  //free(data);
  
  return(0);

} 
    
int load_pilatus_tiff(int fd, DataSet *ds) {
  // to be implemented
  // 487 x 195, 32 bpp
  // convert each pixel into 4 16bpp pixels, divide the counts 
  // and fit into 1024 x 1024 array
  // date/time: 32:0x1E
  // image description: 732:0x234

  int x,y,x1,y1;
  char dt[32],descp[732];
  uint32 w, h;
  size_t npixels;
  uint32* tif_data;
  TIFF *ccd_data;
  
  ccd_data = TIFFFdOpen(fd, datafilename, "r");
  if (!ccd_data) {
    printf("failed to read TIFF image.\n");
    TIFFError(0,"%s","error reading file.");
    return(-1);
  }

  TIFFGetField(ccd_data, TIFFTAG_IMAGEWIDTH, &w);
  TIFFGetField(ccd_data, TIFFTAG_IMAGELENGTH, &h);
  printf("w=%d, h=%d\n",w,h);
  npixels = w * h;
  ds->width=w; ds->height=h;
  ds->data = (DTYPE *)malloc(sizeof(DTYPE)*npixels);  
  tif_data = (uint32*) malloc(npixels * sizeof (uint32));
  if (tif_data != NULL) {
    lseek(fd,0x1000,SEEK_SET);
    read(fd,(void *)tif_data,w*h*sizeof(uint32));
    for (x=0;x<w*h;x++) ds->data[x]=tif_data[x];
      //}
    free(tif_data);
  }
  TIFFClose(ccd_data);

  /* old code
  ds->width = ds->height = 1024;
  ds->data = (DTYPE *)malloc(sizeof(DTYPE)*ds->width*ds->height);  
  //ccd_data = (short *)malloc(sizeof(short)*ds->width*ds->height);
  tif_data = (long *)malloc(sizeof(long)*487*195);
  lseek(fd,0x1000,SEEK_SET);
  read(fd,(void *)tif_data,487*195*sizeof(long));
  lseek(fd,0x1e,SEEK_SET);
  read(fd,(void *)dt,32);
  lseek(fd,0x234,SEEK_SET);
  read(fd,(void *)descp,732);
  */
  //printf("%s\n",dt);
  //printf("%s\n",descp);

  /* 
# Pixel_size 172e-6 m x 172e-6 m
# Silicon sensor, thickness 0.000320 m
# Exposure_time 1.000000 s
# Exposure_period 1.000000 s
# Tau = 0 s
# Count_cutoff 1048574 counts
# Threshold_setting 0 eV
# N_excluded_pixels = 0
# Excluded_pixels: (nil)
# Flat_field: (nil)
# Trim_directory: (nil)
# Wavelength 1.5400 A
# Energy_range (0, 0) eV
# Detector_distance 1.00000 m
# Detector_Voffset 0.00000 m
# Beam_xy (243.50, 97.50) pixels
# Flux 0.0000 ph/s
# Filter_transmission 1.0000
# Start_angle 0.0000 deg.
# Angle_increment 0.1000 deg.
# Detector_2theta 0.0000 deg.
# Polarization 0.990
# Alpha 0.0000 deg.
# Kappa 0.0000 deg.
# Phi 0.0000 deg.
# Chi 0.0000 deg.
# Oscillation_axis X, CW
# N_oscillations 1
  */
  /* old code for reformating the image
  for (y=0;y<195;y++) { 
    // in 1024 array y starts from 512-195=317
    y1=317+y*2;
    for (x=0;x<487;x++) { 
      // in 1024 array x starts from 512-487=25
      x1=25+x*2;
      ds->data[x1+y1*1024]
	=ds->data[x1+1+y1*1024]
	=ds->data[x1+(y1+1)*1024]
	=ds->data[x1+1+(y1+1)*1024]
	=tif_data[x+y*487]/4;
      ds->data[x1+y1*1024]+=tif_data[x+y*487]%4;
    }
  }
  */

  return(0);

}

int load_WAXS_tiff(int fd, DataSet *ds) {
  // 8 byte tiff header, followed by 1042 x 1042, 16 bpp
  // ignore tiff directory at the end of the file
  // crop down to 1024 x 1024
  // image description: 63:0x2122C8
  // date/time: 27:0x212308

  int i,j;

  // updated Aug 2009, use LibTiff

  TIFF *ccd_data;
  
  ccd_data = TIFFFdOpen(fd, datafilename, "r");
  if (!ccd_data) {
    printf("failed to read TIFF image.\n");
    TIFFError(0,"%s","error reading file.");
    return(-1);
  }

  uint32 w,h;
  tdata_t dbuf;
  uint32 row;
  
  TIFFGetField(ccd_data, TIFFTAG_IMAGEWIDTH, &w);
  TIFFGetField(ccd_data, TIFFTAG_IMAGELENGTH, &h);

  /*
  if (imagelength!=1042) {
    printf("this program currently cannot handle WAXS TIFF image size other than 1042 x 1042.\n");
    TIFFClose(ccd_data);   
    return(-1);
  }
  */

  // need to flip across diagonal
  ds->width = h; ds->height = w;
  ds->data = (DTYPE *)malloc(sizeof(DTYPE)*ds->width*ds->height);  

  dbuf = _TIFFmalloc(TIFFScanlineSize(ccd_data));
  for (row = 0; row < ds->height; row++) {
    //if (1042-row>=ds->height) continue;
    if (TIFFReadScanline(ccd_data, dbuf, row, 0)<0) { // error
      _TIFFfree(dbuf);
      TIFFClose(ccd_data);
      return(-1);
    }
    for (j=0;j<ds->width;j++)
      ds->data[j*ds->height+ds->width-row]=((short*)dbuf)[ds->width-j];
  }
  _TIFFfree(dbuf);
  TIFFClose(ccd_data);

  /* old code, cannot handle different byte order 
  //ccd_data = (short *)malloc(sizeof(short)*ds->width*ds->height);
  ccd_data = (short *)malloc(sizeof(short)*1042*1042);
  lseek(fd,8,SEEK_SET);
  read(fd,(void *)ccd_data,1042*1042*sizeof(short));

  //for (i=0;i<ds->height;i++) {
  //  lseek(fd,i*1042+9+8,SEEK_SET);  // 8 for the header, 9 for (1042-1024)/2
  //  read(fd,(void *)(&ccd_data[i*ds->width]),ds->width*sizeof(short));
  //}

  for (i=0;i<ds->height;i++) {
    for (j=0;j<ds->width;j++) {
      // original: need to swap x,y 
      //ds->data[i*1024+j]=ccd_data[i*1042+j+9];
      // Updated: May 2009, x,y swapped, also adjusted offset to retain low q data
      ds->data[j*1024+i]=ccd_data[(1042-i)*1042+(1024-j)];
    }
  }
  free(ccd_data);
  */

  return(0);

}


// 4100 bytes header
// **** do not use the RoperHeader structure defined in princeton.h ***
// pixel data type specified by the header 
// in case of multiple frames in one data file, only the first frame is read
int load_Roper_binary(int fd, DataSet *ds) {
  char header[4100],*p,datasize;
  short datatype;     // 108 experiment datatype
                      // 0 = FLOATING POINT	1 = LONG INTEGER
                      // 2 = INTEGER		3 = UNSIGNED INTEGER
  unsigned short xdim, ydim, dim;
  long num_frames;
  int ix,iy,ax,ay,count,xoff,yoff;
  void *dbuf;
  union {
    float flt;
    long dwd;
    short wd;
    unsigned short uwd;
  } pixel;

  count = read(fd,(void *)(header),4100);

  xdim = *((unsigned short *)(header+42));
  ydim = *((unsigned short *)(header+656));
  num_frames = *((long *)(header+1446));
  datatype = *((short *)(header+108)); 

  // determine data size
  // this program only handles square arrays, 512x512, 1024x1024, and 2048x2048
  // for any other array size, a dataset of equal width and height will be created
  // the data will be cropped if the actual size is less than 120% of 512/1024/2048
  // otherwise the data will be fitted into a larger array, padded with zeros
  
  if (datatype>1) datasize=2;
  else datasize=4;

  ds->width = xdim; ds->height = ydim;
  ds->data = (DTYPE*)malloc(sizeof(DTYPE)*ds->width*ds->height);  

  for (iy=ydim-1;iy>=0;iy--) {
    for (ix=0;ix<xdim;ix++) {
      count = read(fd,(void *)(&pixel),datasize);
      switch (datatype) {
      case 0: // float
	ds->data[(ydim-1-iy)*xdim+ix]=pixel.flt;
	break;
      case 1: // long int
	ds->data[(ydim-1-iy)*xdim+ix]=pixel.dwd;
	break;
      case 2: // short
	ds->data[(ydim-1-iy)*xdim+ix]=pixel.wd;
	break;
      case 3: // unsigned short
	ds->data[(ydim-1-iy)*xdim+ix]=pixel.uwd;
      }
    }
  }
  
  return(0);

}   
/*
  4096 bytes header
  nfast*nslow*depth byte image 
*/
//***** Not completed yet!!!
//*****   need to know data type from header. assume SHORT for now
int load_marCCD_tiff(int fd, DataSet *ds)   
{
  marCCD_Frame_Header fr_header;
  TIFF_Header tf_header;
  int i,count;
  unsigned short *ccd_data; 
    
  /*  read the header */
  count=read(fd,(void *)(&tf_header),sizeof(tf_header));
  //printf("%d bytes read for tf_header, %d bytes expected.\n",count,sizeof(tf_header));
  count=read(fd,(void *)(&fr_header),sizeof(fr_header));
  //printf("%d bytes read for fr_header, %d bytes expected.\n",count,sizeof(fr_header));

  if (fr_header.depth!=2) {
    printf("can't handle a depth other than 2, found %d instead",fr_header.depth);
    close(fd);
    return(-1);
  }
  if (fr_header.nslow!=fr_header.nfast || 
      (fr_header.nslow!=512 && fr_header.nslow!=1024 && fr_header.nslow!=2048)) {
    printf("data is not 512x512 or 1024x1024.");
    close(fd);
    return(-1);
  }

  ds->width = fr_header.nslow;
  ds->height = fr_header.nfast;
  ds->data = (DTYPE*)malloc(sizeof(DTYPE)*ds->width*ds->height);
  ccd_data = (unsigned short *)
    malloc(sizeof(unsigned short)*ds->width*ds->height);

  count=read(fd,(void *)ccd_data,fr_header.nslow*fr_header.nfast*sizeof(unsigned short));
  for (i=0;i<ds->width*ds->height;i++) ds->data[i]=ccd_data[i];
  free(ccd_data);

  //close(fd);  // do not close the data file
  return(0);
}

void conv_raw_indexed(DataSet *ds, DTYPE max, DTYPE min) {
  int i;
  float ft,c;
  unsigned char t;

  c=exp(1.0/globalPref->log_gamma)-1;
  for (i=0;i<ds->width*ds->height;i++) {
    if (ds->data[i]<=min) t=0; 
    else if (ds->data[i]>=max) t=NCOLOR-1;
    else {
      ft=(ds->data[i]-min)/(max-min);
      if (globalPref->log_gamma>0) {  // log color scale
	/*
	  (I - Imin) / (Imax - Imin) = (exp(ft/gamma) - 1) / (exp(1/gamma) - 1), 0<ft<1
	  t = log ( (I - Imin) / (Imax - Imin) * (exp(1/gamma) - 1) + 1 )* gamma
	*/
	ft=log(ft*c+1)*globalPref->log_gamma;
      } 
      if (ft<0 || ft>=1) {
	printf("%10f  %10f  %10f  %10f\n",ds->data[i],max,min,(ds->data[i]-min)/(max-min));
      }
      t=ft*NCOLOR;
    }
    if (globalPref->show_mask) {
      if (get_bit(globalPref->mask_bitmap,i)) {
	t=NCOLOR-1;
	//t=NCOLOR-1-t;
	//t=(t+NCOLOR/2)%NCOLOR;
	//printf("%d\n",i);
      }
    }
    
    ds->indexed_buf[RGBA*i] = globalPref->red[t];   
    ds->indexed_buf[RGBA*i+1] = globalPref->green[t];   
    ds->indexed_buf[RGBA*i+2] = globalPref->blue[t];
  }
  conv_indexed_pixbuf(ds);
  ds->status = DATA_STATUS_INDEXED;
}

void conv_indexed_pixbuf(DataSet *ds) {
  if (ds->pixbuf) gdk_pixbuf_unref(ds->pixbuf);
  ds->pixbuf = gdk_pixbuf_new_from_data (&ds->indexed_buf[0],
					 GDK_COLORSPACE_RGB,
                                         FALSE,
					 8,
                                         ds->width, ds->height,
					 ds->width*RGBA, /* rowstride, RGBA */
					 NULL, NULL);

  /*
  gdk_draw_indexed_image (ds->pixmap, globalPref->gc,
			  0, 0,
			  ds->width, ds->height,
			  GDK_RGB_DITHER_NORMAL,
			  &ds->indexed_buf[0],
			  ds->width*sizeof(guchar),
			  globalPref->RGBmap );
  */
  //ds->status = DATA_STATUS_DRAWN;
}

void initialize() {
  GError *error=NULL;
  int i;
  FILE *infile;
  char buf[256];

  btColor[0] = lookup_widget (wControl, "btColor0");
  btColor[1] = lookup_widget (wControl, "btColor1");
  btColor[2] = lookup_widget (wControl, "btColor2");
  btColor[3] = lookup_widget (wControl, "btColor3");
  btColor[4] = lookup_widget (wControl, "btColor4");
  daColorTable = lookup_widget (wControl, "daColorTable");
  da2DView = lookup_widget (w2DView, "da2DView");
  sclContrast = lookup_widget (wControl, "sclContrast");
  sclBrightness = lookup_widget (wControl, "sclBrightness");
  cbtnLog = lookup_widget (wControl, "cbtnLog");
  etZMax = lookup_widget (wControl, "etZmax");
  etZMin = lookup_widget (wControl, "etZmin");
  etGamma = lookup_widget (wControl, "etGamma");
  tbAutoZScale = lookup_widget (wControl, "tbAutoZScale");
//  etMaskFile = lookup_widget (wControl, "etMaskFile");
//  ckbtnMask  = lookup_widget (wControl, "ckbtnMask");
//  etFlatFile = lookup_widget (wControl, "etFlatFile");
//  ckbtnFlat  = lookup_widget (wControl, "ckbtnFlat");
  sbParaSet = (GtkSpinButton*)lookup_widget (wControl, "sbParaSet");
  cbxZoom  = lookup_widget (wControl, "cbxZoom");
  ckbtnGeo  = lookup_widget (wControl, "ckbtnGeo");
  cbxRecent = lookup_widget (wControl, "cbxRecent");
  cbxRef  = lookup_widget (wControl, "cbxRef");
  cbxMask  = lookup_widget (wControl, "cbxMask");
  cbxFlatField  = lookup_widget (wControl, "cbxFlatField");
  cbxDistortion  = lookup_widget (wControl, "cbxDistortion"); 
  tvInfo = lookup_widget (wControl, "tvInfo");
  tbuf_info = gtk_text_view_get_buffer (GTK_TEXT_VIEW(tvInfo));
  lbStatusMsg = lookup_widget (wControl, "lbStatusMsg");
  etWavelength = lookup_widget (wControl, "etWavelength");
  etDd  = lookup_widget (wControl, "etDd");
  //etDetSwing = lookup_widget (wControl, "etDetSwing");
  etDetTilt = lookup_widget (dlgDetGeo, "etDetTilt");
  etBeamTiltX = lookup_widget (dlgDetGeo, "etBeamTiltX");
  etBeamTiltY = lookup_widget (dlgDetGeo, "etBeamTiltY");
  etDetPhi = lookup_widget (dlgDetGeo, "etDetPhi");
  etDetOrient = lookup_widget (dlgDetGeo, "etDetOrient");
  etCenX = lookup_widget (wControl, "etCenX"); 
  etCenY = lookup_widget (wControl, "etCenY"); 
  tvFileList = lookup_widget (wControl, "tvFileList");
  etSamNormal = lookup_widget (wControl, "etSamNormal");
  etIncidentAng = lookup_widget (wControl, "etIncidentAng");
  et1DGrid = lookup_widget (wControl, "et1DGrid");
  etScaleFactor = lookup_widget (wControl, "etScaleFactor");
  etConst = lookup_widget (wControl, "etConst");
  etSetComments = lookup_widget (wControl, "etSetComments");
  sbCurSet = (GtkSpinButton*)lookup_widget (wControl, "sbCurSet");
  sbSet1 = (GtkSpinButton*)lookup_widget (wControl, "sbSet1");
  sbSet2 = (GtkSpinButton*)lookup_widget (wControl, "sbSet2");
  ckbtnDetRec = lookup_widget (wControl, "cbDetRec");
  etLineProfileCx = lookup_widget (wControl, "etLineProfileCx");
  etLineProfileCy = lookup_widget (wControl, "etLineProfileCy");
  etLineProfileLength = lookup_widget (wControl, "etLineProfileLength");
  etLineProfileOrient = lookup_widget (wControl, "etLineProfileOrient");
  etLineProfileRadius = lookup_widget (wControl, "etLineProfileRadius");
  etPeakThresh = lookup_widget (wControl, "etPeakThresh");
  etZingerRadius = lookup_widget (wControl, "etZingerRadius");
  etZingerTolerance = lookup_widget (wControl, "etZingerTolerance");

  // these are defalt values
  globalPref->wavelength = 1.55;
  globalPref->ratioDd = 3.0;
  globalPref->detector_psi = 0;
  globalPref->detector_tilt= 0;
  globalPref->beam_tilt_x= 0;
  globalPref->beam_tilt_y= 0;
  globalPref->detector_orient = 0;
  globalPref->detector_phi = 0;
  globalPref->cx = 512;
  globalPref->cy = 512;
  globalPref->normal = 0;
  globalPref->incident = 0.5;
  globalPref->peak_N = 0;
  globalPref->showpeak = 0;

  sprintf(buf,"%s/.view.gtk/conf.0",getenv(HOME_ENV));
  infile = fopen(buf,"r");  
  if (infile) {
    fgets(buf,255,infile);
    if (strncmp(buf,"# view.gtk-conf",15)==0) { // need some sort of token as proof of valid file format
      fscanf(infile,"%f",&globalPref->wavelength);
      fscanf(infile,"%f",&globalPref->ratioDd);
      fscanf(infile,"%f",&globalPref->detector_psi);
      fscanf(infile,"%f",&globalPref->cx);
      fscanf(infile,"%f",&globalPref->cy);
      fscanf(infile,"%f",&globalPref->normal);
      fscanf(infile,"%f",&globalPref->incident);
      fscanf(infile,"%f",&globalPref->detector_orient);
      fscanf(infile,"%f",&globalPref->detector_tilt);
      fscanf(infile,"%f",&globalPref->detector_phi);
      fscanf(infile,"%f",&globalPref->beam_tilt_x);
      fscanf(infile,"%f",&globalPref->beam_tilt_y);
    }
    fclose(infile);
  } 
   
  calc_rot(globalPref);

  // set the font
  sprintf(buf,"%s/.view.gtk/conf.font",getenv(HOME_ENV));
  infile = fopen(buf,"r");  
  if (infile) {
    fgets(buf,255,infile);
    change_widget_font(wControl,pango_font_description_from_string (buf));
    fclose(infile);
  }

  setValue(etWavelength, "%5.3f", globalPref->wavelength);
  setValue(etDd, "%5.3f", globalPref->ratioDd);
  //setValue(etDetSwing, "%5.1f", globalPref->detector_psi);
  setValue(etCenX, "%5.1f", globalPref->cx);
  setValue(etCenY, "%5.1f", globalPref->cy);
  setValue(etSamNormal, "%5.1f", globalPref->normal);
  setValue(etIncidentAng, "%5.1f", globalPref->incident);
  setValue(etDetOrient, "%5.1f", globalPref->detector_orient);
  setValue(etDetTilt, "%5.1f", globalPref->detector_tilt);
  setValue(etBeamTiltX, "%5.1f", globalPref->beam_tilt_x);
  setValue(etBeamTiltY, "%5.1f", globalPref->beam_tilt_y);
  setValue(etDetPhi, "%5.1f", globalPref->detector_phi);

  for (i=0;i<5;i++) {
    //    globalPref->bColor[i].red = globalPref->bColor[i].blue 
    //  = globalPref->bColor[i].green = 65535/4*(4-i);
    globalPref->bColor[i].red = default_color[i][0]*256;
    globalPref->bColor[i].green = default_color[i][1]*256;
    globalPref->bColor[i].blue = default_color[i][2]*256;
    gtk_widget_modify_bg (btColor[i], GTK_STATE_NORMAL, &globalPref->bColor[i]);
    gtk_widget_modify_bg (btColor[i], GTK_STATE_PRELIGHT, &globalPref->bColor[i]);
  }
  globalPref->gc = gdk_gc_new(wControl->window);
  globalPref->zoombuf = gdk_pixbuf_new ( GDK_COLORSPACE_RGB,
                                         FALSE,
					 8,
					 WINDOW_SIZE, WINDOW_SIZE );
  globalPref->pixmap = gdk_pixmap_new ( w2DView->window,
					WINDOW_SIZE, WINDOW_SIZE, -1 );
  globalPref->cr = gdk_cairo_create ( w2DView->window );
  globalPref->gc = gdk_gc_new ( globalPref->pixmap );
  globalPref->color_refPat.red = 0;
  globalPref->color_refPat.green = 0;
  globalPref->color_refPat.blue = 60000;
  globalPref->color_normal.red = 60000;
  globalPref->color_normal.green = 0;
  globalPref->color_normal.blue = 0;
  globalPref->color_peaks.red = 0;
  globalPref->color_peaks.green = 60000;
  globalPref->color_peaks.blue = 0;

  update_RGBmap();
  
  globalPref->zoom = 0.5;
  globalPref->viewport_cx = globalPref->viewport_cy = 512;

  globalPref->dezinger = 1;
  globalPref->zr = 5;
  globalPref->zt = 3;

  globalPref->crosshair_cursor = gdk_cursor_new(GDK_CROSSHAIR);
  globalPref->move_cursor = gdk_cursor_new(GDK_FLEUR);
  globalPref->mouse_pressed = 0;
  globalPref->mouse_function = 0;
  globalPref->log_gamma = -1;
  globalPref->refPattern = 0;
  globalPref->show_mask = 0;
  globalPref->use_ffield = 0;
  globalPref->mask_fn[0] = 0;
  globalPref->mask_bitmap = NULL;
  globalPref->flatfield.data = NULL;
  globalPref->flatfield.status = DATA_STATUS_EMPTY;
  globalPref->zmax = globalPref->zmin = -1000;
  getcwd(&globalPref->cwd[0],255);
  strcat(&globalPref->cwd[0],"/");
  globalPref->q_ref = 0;
  globalPref->DataFormat = 0;
  globalPref->update=0;

  gtk_widget_set_sensitive (etSamNormal, FALSE);
  gtk_widget_set_sensitive (etIncidentAng, FALSE);
  globalPref->samGeometry = 0;

  for (i=0;i<8;i++) {
    sto[i].data=NULL;
    sto[i].zinger_map=NULL;
    strcpy(&sto[i].comments[0],"(empty)");
    sto[i].status = DATA_STATUS_EMPTY;
    sto[i].data = NULL;
    sto[i].zinger_map = NULL;
    sto[i].indexed_buf = NULL;
    sto[i].pixbuf = NULL;
  }
  dataset=&sto[6];

  gtk_combo_box_set_active (GTK_COMBO_BOX (cbxZoom), 0);
  gtk_toggle_button_set_active ((gpointer)ckbtnGeo, 0);
  gtk_combo_box_set_active (GTK_COMBO_BOX (cbxRef), 0);
  gtk_combo_box_set_active (GTK_COMBO_BOX (cbxMask), 0);
  gtk_combo_box_set_active (GTK_COMBO_BOX (cbxFlatField), 0);
  gtk_combo_box_set_active (GTK_COMBO_BOX (cbxDistortion), 0);  

  //g_thread_create(proc_thread, // GThreadFunc func,
  //		  (void *)pbar_conv,        //  gpointer data
  //		  FALSE,       //  gboolean joinable
  //		  NULL         //  GError **error
  //		  );
  
}

// qx and qy are perpendicular to the beam (z-axis)
// (qx, qy, 0) is not visible on the detetor
DTYPE qxy_to_pos(float q, float phi, float *px, float *py) {
  float tt,dr,theta,sum,weight;
  float x1,y1,z1,x2,y2,z2,x,y,z,dx,dy,dz,rn;
  float R13,R23,R33,v[3];
  int ix,iy,ch,idx,idy;

  theta = globalPref->wavelength*q/(4.0*Pi);
  if (theta>=1.0) return(-1);
  theta = asin(theta);

  if (fabs(theta)<1e-6) { // beam center
    *px = globalPref->cx;
    *py = globalPref->cy;
  } else {

    rn = globalPref->normal*Pi/180.0;
    phi -= rn;
    
    // now determine the physical position of the pixel
    // non-trivial, see doc
    
    R13 = globalPref->det_rot[0][2];
    R23 = globalPref->det_rot[1][2];
    R33 = globalPref->det_rot[2][2];

    tt = (R13*cos(phi)+R23*sin(phi))*tan(2.*theta);
    dr = globalPref->ratioDd*sto[6].width;

    // pixel position in lab referece frame
    v[2] = dz = dr*tt/(tt-R33);
    v[0] = (dr-dz)*tan(2.0*theta)*cos(phi); 
    v[1] = (dr-dz)*tan(2.0*theta)*sin(phi); 
    
    // transform to detector frame
    matrix_dot_vector(globalPref->det_rot_T,v);

    // pixel position, note reversed y-axis
    *px = v[0] + globalPref->cx;
    *py = -v[1] + globalPref->cy;   

  }

  ix = (int)(*px); iy = (int)(*py);
  if (ix>(*px)) ix--;
  if (iy>(*py)) iy--;
  dx = (*px)-ix;   dy = (*py)-iy;  
  //printf("dx=%5.3f dy=%5.3f  [",dx,dy);
  
  sum=0;//rn=0;
  for (ch=0;ch<4;ch++) {
    idx=ch&0x01;
    idy=(ch&0x02)>>1;
    
    weight=fabs((-dx+1-idx)*(-dy+1-idy));
    //rn+=weight;
    //printf("%6.4f(%2d,%2d) ",weight,1-idx,1-idy); 
    
    if (ix+idx<0 || ix+idx>=sto[6].width ||
	iy+idy<0 || iy+idy>=sto[6].height) continue;
    
    sum+=weight*sto[6].data[(ix+idx)+(iy+idy)*sto[6].width];
  }
  //printf("] -> %6.4f\n",rn);
  //if (fabs(rn-1.0)>0.001) exit(1);

  return(sum);
}

DTYPE qrz_to_pos(float qr, float qn, float *px, float *py) {
  float phi,theta,alpha,dr,tt;
  float q,qx,qy,qz;

  if (!sto[6].data) { // sto[6] is scattering pattern
    printf("sto[6] is empty\n");
    exit(1);
  }

  alpha = globalPref->incident*Pi/180.0;

  q = sqrt(qr*qr+qn*qn);
  theta = globalPref->wavelength*q/(4.0*Pi);
  if (theta>=1.0) return(-1);
  theta = asin(theta);

  // conversion from qr, qz to projection on to the lab frame axis 
  // see doc
  qz = q*sin(theta);
  qy = (qn-qz*sin(alpha))/cos(alpha);

  tt = q*q-qz*qz-qy*qy;
  if (tt<0) return(-1);
  qx = sqrt(tt)*(qr<0?-1:1);

  phi = atan2(qy,qx);
  return(qxy_to_pos(q,phi,px,py));

}

// new version, with different definition of x,y,z
float pos_to_qrz(float col, float row, float *pqr, float *pqz) {
  float q,qn,qr,qx,qy,qz;
  float phi,theta,alpha,rn,dr;
  float x,y,z,tt;
  float v[3];

  v[0] = col - globalPref->cx;
  v[1] = -(row - globalPref->cy); 
  v[2] = 0;

  matrix_dot_vector(globalPref->det_rot,v);
  
  x = v[0];
  y = v[1];
  z = v[2];
  dr = globalPref->ratioDd*sto[6].width;

  tt = sqrt(x*x+y*y);
  theta = 0.5*atan(tt/(dr-z));
  q=4.0*Pi*sin(theta)/globalPref->wavelength;
  
  // pos_to_q() calls pos_to_qrz(x,y,0,0)
  if (!pqr || !pqz) return(q);

  phi = atan2(y,x);

  // sample normal is supposed to be vertical up (y-axis)
  // in reality it may be off by rn, clockwise is positive 
  rn = globalPref->normal*Pi/180.0;
  phi += rn;
  //printf("phi = %7.2f \n", phi/Pi*180); 

  qz = q*sin(theta);
  qy = q*cos(theta)*sin(phi);  
  qx = q*cos(theta)*cos(phi);

  alpha = globalPref->incident*Pi/180.0;
  qn = qy*cos(alpha)+qz*sin(alpha);
  qr = sqrt(q*q-qn*qn)*(qx<0?-1:1);

  *pqr = qr;
  *pqz = qn;
  return(q);
}


float pos_to_q(float x, float y) {
  return(pos_to_qrz(x,y,0,0));
}

// get the average intensity within a circle of radius rad at (x,y)
float get_average_intensity(float x, float y, float rad) {
  int dx,dy,ix,iy;
  float count,sum;

  count=0;
  sum=0;
  for (dx=-rad-1;dx++;dx<rad+1) {
    for (dy=-rad-1;dy++;dy<rad+1) {
      ix=dx+x; iy=dy+y;
      if (ix<0 || iy<0 || ix<=dataset->width || iy>=dataset->height) continue;
      if (dx*dx+dy*dy>rad*rad) continue;
      if (dataset->data[ix+iy*dataset->width]>0) {
	count++;
	sum+=dataset->data[ix+iy*dataset->width];
      }
    }
  }
  if (count>0) sum/=count;
  return(sum);
}

// extract a line centered at (cx,cy), length 2*len, each data point averaged 
// within a circle of radius rad 
void extract_data(FILE *out, int cx, int cy, float th, int len, float rad, float delta) {
  float x,y;
  int i,d;

  d=(float)len/rad;
  for (i=-d;i<=d;i++) {
    x=cx+cos(th)*i*rad;
    y=cy+sin(th)*i*rad;
    fprintf(out,"%7.4f   %f\n",rad*i,get_average_intensity(x,y,rad)); 
  }
  
}

gboolean analyze(DTYPE v0, DTYPE v1, DTYPE v2) {
  float dv,av,se;
  int cnt=0;

  if (v0<0) return(TRUE);
  av=(v0+v1+v2)/3;
  dv=sqrt(((v0-av)*(v0-av)+(v1-av)*(v1-av)+(v2-av)*(v2-av))/3);
  se=sqrt(av);  // statistical error
  dv=sqrt(se*se+dv+dv);

  //printf("v0=%5d, v1=%5d, v2=%5d,   dv=%f cnt=%d",v0,v1,v2,dv,cnt);

  v0=fabs(av-v0);v1=fabs(av-v1);v2=fabs(av-v2);
  if (v0>dv*globalPref->zt && v0>v1 && v0>v2) {
    //printf("zinger!!!\n");
    return(TRUE);
  }
  //printf("not a zinger\n");
  return(FALSE);
}

gboolean dezinger(DataSet *ds, int ix, int iy, int use_mask) {
  // return true if the pixel at (ix,iy) is determined to be a zinger
  DTYPE v0,v1,v2;
  int i0,i1,i2;

  // rule for identifying zingers
  // take 3 points, the point in question itself v0
  // and two other points on the same line: v1 and v2
  // 
  // test 1:
  //       if v1 is between v0 and v2, v0 is not a zinger
  //       Note: this won't work if v0 is a zinger (>>v1, v2) and v1>v2
  // test 2:
  //       in case test 1 fails
  //       if v0, v1 and v2 are similar, not a zinger
  //       criteria for being similar:
  //           average of the 3: av, standard deviation: dv
  //           v1 is a zinger if v1 deviate from ve by more than dv*globalPref->zt
  //  See the function above

  i0 = ix+iy*ds->width;
  v0=ds->data[i0];

  //if (ix<globalPref->cx) {
  // Updated May 2009
  if (ix<ds->width/2) {
    i1=ix+globalPref->zr+iy*ds->width;
    i2=ix+2*globalPref->zr+iy*ds->width;
    v1=ds->data[i1];
    v2=ds->data[i2];
   } else {
    i1=ix-globalPref->zr+iy*ds->width;
    i2=ix-2*globalPref->zr+iy*ds->width;
    v1=ds->data[i1];
    v2=ds->data[i2];
  }
  //v2=(v0+v2)/2/v1;
  //if (v2>2 && v0>v1) return(TRUE);
  if (globalPref->show_mask && use_mask) {
    if (get_bit(globalPref->mask_bitmap,i1)==0 && 
	get_bit(globalPref->mask_bitmap,i2)==0) {
      if (analyze(v0,v1,v2)) return(TRUE);
    }
  } else if (analyze(v0,v1,v2)) return(TRUE); 

  //  if (iy<globalPref->cy) {
  // Updated May 2009
  if (iy<ds->height/2) {
    i1=ix+(iy+globalPref->zr)*ds->width;
    i2=ix+(iy+2*globalPref->zr)*ds->width;
    v1=ds->data[i1];
    v2=ds->data[i2];
  } else {
    i1=ix+(iy-globalPref->zr)*ds->width;
    i2=ix+(iy-2*globalPref->zr)*ds->width;
    v1=ds->data[i1];
    v2=ds->data[i2];
  }
  //v2=(v2+v2)/2/v1;
  //if (v2>2 && v0> v1) return(TRUE);
  if (globalPref->show_mask && use_mask) {
    if (get_bit(globalPref->mask_bitmap,i1)==0 && 
	get_bit(globalPref->mask_bitmap,i2)==0) {
      if (analyze(v0,v1,v2)) return(TRUE);
    }
  } if (analyze(v0,v1,v2)) return(TRUE);    

  return(FALSE);
}

void convert_to_ang_dist (char *filename) {
  int ix,iy,i,index,k,ki,len;
  float dx,dy,phi,dphi,r,t,sigma,weight[256],y[256];
  FILE *output;

  len=240;dphi=360.0/len;

  output = fopen(filename, "w+");
  if (!output) {
    send_info_message("Can't open output file. ");
    return;
  }

  for (k=0;k<len+1;k++) y[k]=weight[k]=0;

  for (ix=0;ix<dataset->width;ix++) {
    for (iy=0;iy<dataset->height;iy++) {
      index = ix+iy*dataset->width;
      if (globalPref->show_mask &&
	  get_bit(globalPref->mask_bitmap,index)) continue;
      if (globalPref->dezinger) { // this gets rid of zingers
	if (dezinger(dataset,ix,iy,1)) continue;
      }
      dx = ix - globalPref->cx;
      dy = -(iy - globalPref->cy);
      phi = atan2(dy,dx)*180.0/3.1415926;
      while (phi<0) phi+=360;
      if (phi>360) printf("%f  %f  %f\n",dx,dy,phi);
      r=sqrt(dx*dx+dy*dy);
      t=phi/dphi;
      k=(int)(t);
      sigma=sqrt(log(2.0))/r*180.0/3.1415926/dphi;
      for (i=-2;i<=2;i++) {
	ki=(k+i+len)%len;
	t=(-phi/dphi+k+i)/sigma;
	t=exp(-t*t);
	y[ki]+=t*dataset->data[index];
	weight[ki]+=t;
      }
    }
  }

  y[0]+=y[len];weight[0]+=weight[len]; // 360deg is the same as 0deg
  for (k=0;k<len;k++) {
    t=sqrt(y[k]);
    if (weight[k]>1.0e-8) { 
      y[k]/=weight[k];
      t/=weight[k];
      if (output) 
	fprintf(output, "%8.4f   %8.4f   %8.4f\n",dphi*k,y[k],weight[k]);
    }
  }
  
  fclose(output);
}

void convert_to_1D (float dq, int len, char *filename) {
  int ix,iy,index,k;
  float v0,v1,v2,q,t,*t_ij;
  float weight[DATA_LEN],ave[DATA_LEN],err_abs[DATA_LEN],err_rms[DATA_LEN];
  FILE *output;

  assert(len<DATA_LEN);

  output = fopen(filename, "w+");
  if (!output) {
    send_info_message("Can't open output file. ");
    return;
  }
  t_ij = (float *)malloc(dataset->width*dataset->height*sizeof(float));
  for (k=0;k<len;k++) ave[k]=weight[k]=err_abs[k]=err_rms[k]=0;

  for (ix=0;ix<dataset->width;ix++) {
    for (iy=0;iy<dataset->height;iy++) {
      index = ix+iy*dataset->width;
      t_ij[index]=-1;
      if (globalPref->show_mask &&
	  get_bit(globalPref->mask_bitmap,index)) continue;
      if (globalPref->dezinger) { // this gets rid of zingers
	if (dezinger(dataset,ix,iy,1)) continue;
      }
      q = pos_to_q(ix,iy);
      t=t_ij[index]=q/dq;
      k=(int)(t);
      t-=k;
      if (k<len) {
	ave[k]+=t*dataset->data[index];
	weight[k]+=t;
	if (k<len-1) {
	  ave[k+1]+=(1.0-t)*dataset->data[index];
	  weight[k+1]+=1.0-t;
	}
      }
    }
  }

  // now calculate the average
  for (k=0;k<len;k++) {
    if (weight[k]>1.0e-8) ave[k]/=weight[k];
  }

  // now calculate the error
  for (ix=0;ix<dataset->width;ix++) {
    for (iy=0;iy<dataset->height;iy++) {
      index = ix+iy*dataset->width;
      if ((t=t_ij[index])<0) continue;
      k=(int)(t);
      t-=k;
      if (k<len) {
	q=dataset->data[index]-ave[k];
	//if (k==100) printf("%10f   %10f   %10f\n",
	//	   ave[k],dataset->data[index],q);
	//err_abs[k]+=t*fabs(q);
	err_rms[k]+=t*q*q;
	if (k<len-1) {
	  q=dataset->data[index]-ave[k+1];
	  //err_abs[k+1]+=(1.0-t)*fabs(q);
	  err_rms[k+1]+=(1.0-t)*q*q;
	}
      }
    }
  }
  free(t_ij);
  
  // output
  if (output) 
    fprintf(output, "#     q            I           err_rms       err_abs\n");
  for (k=0;k<len;k++) {
    if (weight[k]>1) {
      // standard deviation = sum ( (x_ave - x_i)^2 ) / (N-1)
      err_rms[k]=sqrt(err_rms[k]/(weight[k]-1));
      // standard deviation of the average = standard deviation / sqrt(N)
      err_rms[k]/=sqrt(weight[k]);
      //err_abs[k]/=weight[k];
      if (output) 
	fprintf(output, "%10.5f   %10.5g     %10.5g\n",
		dq*k,ave[k],err_rms[k]);
    }
  }  

  fclose(output);
}

void rot_matrix(float (*a)[3],char axis,float angle) {

  switch (axis) {
  case 'x':
  case 'X':
    a[0][0]=1;            a[0][1]=0;              a[0][2]=0;
    a[1][0]=0;            a[1][1]=cos(angle);     a[1][2]=-sin(angle);
    a[2][0]=0;            a[2][1]=sin(angle);     a[2][2]=cos(angle);    
    break;
  case 'y':
  case 'Y':
    a[0][0]=cos(angle);   a[0][1]=0;              a[0][2]=sin(angle);
    a[1][0]=0;            a[1][1]=1;              a[1][2]=0;
    a[2][0]=-sin(angle);  a[2][1]=0;              a[2][2]=cos(angle);    
    break;
  case 'z':
  case 'Z':
    a[0][0]=cos(angle);   a[0][1]=-sin(angle);    a[0][2]=0;
    a[1][0]=sin(angle);   a[1][1]= cos(angle);    a[1][2]=0;
    a[2][0]=0;            a[2][1]=0;              a[2][2]=1;    
    break;
  default:
    printf("invalid axis.");
    exit(1);
  }

  return;
}

void matrix_dot_matrix(float (*a)[3],float (*b)[3]) {
  float c[3][3];
  int i,j;
    
  c[0][0]=a[0][0]*b[0][0]+a[0][1]*b[1][0]+a[0][2]*b[2][0];
  c[0][1]=a[0][0]*b[0][1]+a[0][1]*b[1][1]+a[0][2]*b[2][1];
  c[0][2]=a[0][0]*b[0][2]+a[0][1]*b[1][2]+a[0][2]*b[2][2];

  c[1][0]=a[1][0]*b[0][0]+a[1][1]*b[1][0]+a[1][2]*b[2][0];
  c[1][1]=a[1][0]*b[0][1]+a[1][1]*b[1][1]+a[1][2]*b[2][1];
  c[1][2]=a[1][0]*b[0][2]+a[1][1]*b[1][2]+a[1][2]*b[2][2];

  c[2][0]=a[2][0]*b[0][0]+a[2][1]*b[1][0]+a[2][2]*b[2][0];
  c[2][1]=a[2][0]*b[0][1]+a[2][1]*b[1][1]+a[2][2]*b[2][1];
  c[2][2]=a[2][0]*b[0][2]+a[2][1]*b[1][2]+a[2][2]*b[2][2];

  for (i=0;i<3;i++) {
    for (j=0;j<3;j++) b[i][j]=c[i][j];
  }

  return;

}

void matrix_dot_vector(float (*m)[3],float *v) {
  float r[3];
  
  r[0]=m[0][0]*v[0]+m[0][1]*v[1]+m[0][2]*v[2];
  r[1]=m[1][0]*v[0]+m[1][1]*v[1]+m[1][2]*v[2];
  r[2]=m[2][0]*v[0]+m[2][1]*v[1]+m[2][2]*v[2];

  v[0]=r[0]; v[1]=r[1]; v[2]=r[2];
  return;

}

void calc_rot(stPreference *sp) {
  float dOrt,dTlt,dPhi,bTltX,bTltY;
  float tm[3][3],det_rot[3][3]={{1.,0,0},{0,1.,0},{0,0,1.}};
  int i,j;

  dOrt=globalPref->detector_orient/180*Pi;
  dTlt=globalPref->detector_tilt/180*Pi;
  dPhi=globalPref->detector_phi/180*Pi;
  bTltX=globalPref->beam_tilt_x/180*Pi;
  bTltY=globalPref->beam_tilt_y/180*Pi;

  // tilt by dTlt along axis at dOrt from the y-axis
  rot_matrix(tm,'z',-dOrt);
  matrix_dot_matrix(tm,det_rot);
 
  rot_matrix(tm,'y',dTlt);
  matrix_dot_matrix(tm,det_rot);

  rot_matrix(tm,'z',dOrt);
  matrix_dot_matrix(tm,det_rot);
  
  rot_matrix(tm,'z',dPhi);
  matrix_dot_matrix(tm,det_rot);
 
    rot_matrix(tm,'x',bTltX);
  matrix_dot_matrix(tm,det_rot);

  rot_matrix(tm,'y',bTltY);
  matrix_dot_matrix(tm,det_rot);

  //printf("dOrt=%.2f, dTlt=%.2f, dPhi=%.2f, bTltX=%.2f, bTltY=%.2f\n",dOrt,dTlt,dPhi,bTltX,bTltY);

  for (i=0;i<3;i++) {
    for (j=0;j<3;j++) sp->det_rot[i][j] = det_rot[i][j];
    for (j=0;j<3;j++) sp->det_rot_T[i][j] = det_rot[j][i];
    //printf("%12.3f   %12.3f   %12.3f\n",det_rot[i][0],det_rot[i][1],det_rot[i][2]);  
  }

  return;
 
}

