#ifndef DATA_H
#define DATA_H

//#define DTYPE short

// why DTYPE has to be float???
#define DTYPE float

#include <gtk/gtk.h>
#include <stdio.h>
#include <unistd.h>
#include <cairo.h>
#include "bitmap.h"

#define NCOLOR 64
#define DATA_STATUS_EMPTY 0
#define DATA_STATUS_LOADED 1
#define DATA_STATUS_INDEXED 2
#define DATA_STATUS_NEED_DECOR 4
#define DATA_STATUS_DRAWN 8
#define MAX_COM 16
#define FLAT_MIN 100

//#define WINDOW_SIZE 712
//#define WINDOW_SIZE 512
int WINDOW_SIZE;
#define Pi 3.1415926

#define RGBA 3

#ifdef WIN32
#define HOME_ENV "USERPROFILE"
#else
#define HOME_ENV "HOME"
#endif

/*
  for each data set, need to store the raw data, converted 8-bit data
  and other information
*/
typedef struct {
  DTYPE *data;
  guchar *indexed_buf;
  int width,height;
  char comments[256];
  int status;
  GdkPixbuf *pixbuf;
  DTYPE zmax,zmin,avg,sigma;
  float favg;
  float x0,y0,dx,dy;
  BITMAP *zinger_map;
  /* other statistics */
} DataSet;

/* now use DataSet type for sto[8] as well
typedef struct {
  DTYPE *data;
  int width,height;
  char comments[256];
  DTYPE zmax,zmin,avg,sigma;
  BITMAP *zinger_map;
  float favg;
} StoredData;
*/

/* definition of parameters
 *  ratioDd = distance between the sample and the detector / diameter of the detector
 *  detector_psi = the angle by which the detector arm deviate from the beam direction ,
 *           counter-clockwise (view from top) is positive
 *
 *  updated: May 2009, additional parameters to describe detector geometry
 *  detector_orient: physical orientation of pixel rows in the detector, 45deg for WAXS
 *  detector_tilt: similar to detector_psi
 *  detector_phi: rotation of the (WAXS) detector upward, rotation axis is the beam direction
 *
 */

typedef struct {
  float para[4];
  char type;
  int N;
  int color;
} stRefPtn;

typedef struct {
  GdkColor bColor[5];
  guchar red[NCOLOR],green[NCOLOR],blue[NCOLOR];
  GdkRgbCmap* RGBmap;
  GdkPixbuf *zoombuf;
  GdkPixmap *pixmap;
  cairo_t *cr;
  DataSet flatfield;
  GdkGC *gc;
  GdkColor color_refPat,color_normal,color_mask,color_peaks;
  float wavelength,ratioDd,detector_psi,detector_phi,detector_tilt;
  float beam_tilt_x,beam_tilt_y,detector_orient,cx,cy;
  float det_rot[3][3],det_rot_T[3][3];
  float normal,incident;
  int zr;
  float zt;
  char recent[5][256];
  float zoom,log_gamma,zmax,zmin;
  int n_ref,DataFormat,N_dq;
  stRefPtn *q_ref;
  char mask_fn[256];
  gint refPattern,Nmask,show_mask,use_ffield;
  gint samGeometry,dezinger,update;
  gint mouse_pressed, mouse_function;
  gint viewport_cx,viewport_cy;
  gint viewport_width,viewport_height;
  char com_list[MAX_COM];
  gint ncom;
  GdkCursor *crosshair_cursor,*move_cursor; 
  BITMAP *mask_bitmap;
  char cwd[256];
  float peak_x[256],peak_y[256];
  int peak_N,showpeak;
} stPreference;

int read_data(int fd, DataSet *ds,gboolean show);
void find_MM(DataSet *,gboolean);
int post_loading_data(DataSet *,gboolean);
int load_WAXS_tiff(int fd, DataSet *ds);
int load_pilatus_tiff(int fd, DataSet *ds);
int load_marCCD_tiff(int fd, DataSet *ds);
int load_square_binary(int fd, DataSet *ds, int bpp);   
int load_Bruker_binary(int fd, DataSet *ds);
int load_ascii(int fd, DataSet *ds);
void conv_raw_indexed(DataSet *ds, DTYPE max, DTYPE min);
void conv_indexed_pixbuf(DataSet *ds);
void initialize();
void update_preference();
void save_preference();
float pos_to_q(float,float);
float pos_to_qrz(float,float,float*,float*);
DTYPE qxy_to_pos(float,float,float*,float*);
DTYPE qrz_to_pos(float,float,float*,float*);
void convert_to_1D(float, int, char *);
void convert_to_ang_dist(char *);
int load_mask();
gint dezinger(DataSet *ds, int ix, int iy, int use_mask);
void calc_rot(stPreference *sp);
void matrix_dot_vector(float (*m)[3],float *v);

#endif 

