#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <math.h>
#include "data.h"

#define N_SUCROSE 10
#define N_AGBH 12
#define QO_AgBH 0.1076
#define N_cir 72

float q_sucrose[N_SUCROSE]={0.5933,0.8289,0.9054,0.9336,1.1000};

extern GtkWidget *w2DView;
extern GtkWidget *wControl;
extern GtkWidget *daColorTable;
extern GtkWidget *da2DView;
extern stPreference *globalPref;
extern GtkWidget *btColor[5];
extern DataSet *dataset;
extern int q_available;
extern GtkWidget *ckbtnDetRec;
extern GdkColor colors[8];

/*
   General Bezier curve
   Number of control points is n+1
   0 <= mu < 1    IMPORTANT, the last point is not computed
*/
double Bezier(guchar p[], int n, double mu)
{
   int k,kn,nn,nkn;
   double blend,muk,munk;
   double sm;

   sm = 0;
   muk = 1;
   munk = pow(1-mu,(double)n);

   for (k=0;k<=n;k++) {
      nn = n;
      kn = k;
      nkn = n - k;
      blend = muk * munk;
      muk *= mu;
      munk /= (1-mu);
      while (nn >= 1) {
         blend *= nn;
         nn--;
         if (kn > 1) {
            blend /= (double)kn;
            kn--;
         }
         if (nkn > 1) {
            blend /= (double)nkn;
            nkn--;
         }
      }
      sm += blend*p[k];
   }
   return(sm);
}


void update_RGBmap() {
  int i;
  GtkStyle *style;
  guchar tr,tg,tb,r[5],g[5],b[5];
  guint32 bRGB[5],RGBtable[NCOLOR];
  double s;
  
  for (i=0;i<5;i++) {
    style=gtk_widget_get_style(btColor[i]);
    r[i] = style->bg[0].red / 256;
    g[i] = style->bg[0].green / 256;
    b[i] = style->bg[0].blue / 256;
    bRGB[i] = 65536*r[i] + 256*g[i] + b[i]; 
    /* printf("%06X\n", bRGB[i]); */ 
  }

  RGBtable[0] = bRGB[0];
  RGBtable[NCOLOR-1] = bRGB[4];
  globalPref->red[0] = r[0]; globalPref->red[NCOLOR-1] = r[4]; 
  globalPref->green[0] = g[0]; globalPref->green[NCOLOR-1] = g[4]; 
  globalPref->blue[0] = b[0]; globalPref->blue[NCOLOR-1] = b[4]; 
  for (i=1;i<NCOLOR-1;i++) {
    s = 1.0/(NCOLOR-1)*i;
    tr = globalPref->red[i] = Bezier(r,4,s);
    tg = globalPref->green[i] = Bezier(g,4,s);
    tb = globalPref->blue[i] = Bezier(b,4,s);
    RGBtable[i]  = 65536*tr + 256*tg + tb;
    /** printf("%06X\n", RGBtable[i]); **/
  }
  
  /*if (globalPref->RGBmap) gdk_rgb_cmap_free (globalPref->RGBmap); */
  
  globalPref->RGBmap = gdk_rgb_cmap_new (&RGBtable[0], NCOLOR);
  
  /* now draw the colortable and 2D data */
    
}

gboolean draw_color_table(GtkWidget *da,
			  GdkEventExpose *event,
			  gpointer data) {
  int i,width,height;
  guchar buf[256];

  width = daColorTable->allocation.width;
  if (width >256) width=256;
  height = daColorTable->allocation.height;
  for (i=0;i<width;i++) buf[i]=NCOLOR*i/width;

  if (globalPref->RGBmap!=NULL) 
    gdk_draw_indexed_image (da->window , globalPref->gc,
			    0, 0, 
			    width, height,
			    GDK_RGB_DITHER_NORMAL,
			    buf, 0, 
			    globalPref->RGBmap);

  return(TRUE);
}

/* 
 * these conv functions assume that the detector is perpendicular to the beam 
 * and that dataset->width = dataset->height
 * q = 4Pi sin(theta) / lambda, r = D tan(2 theta)
 */
float conv_to_q(float r) {
  float theta,q;
  theta = 0.5 * atan2( dataset->width*globalPref->ratioDd, r );
  q = 4.0*Pi*sin(theta)/globalPref->wavelength;
  return(q);
}
float conv_to_r(float q) {
  float r,theta;
  theta = asin(globalPref->wavelength*q/(4.0*Pi));
  r = dataset->width*globalPref->ratioDd*tan(2.0*theta);
  return(r);
}

#define NCIRCLE 90
#define CROSSSIZE 3

void draw_circle( float x, float y, float r) {
  /*
    this is supposed to draw a circle at (x,y):r
    eventually need to consider the tilt angle of the detector
  */

  float phi,tx,tz;
  int i;
  GdkPoint points[NCIRCLE];

  for (i=0;i<NCIRCLE;i++) {
    phi = 2.0*Pi*i/NCIRCLE;
    tx = r*cos(phi);
    tz = -dataset->width*globalPref->ratioDd;
    // see pos_to_q
    points[i].x =  x + tx*cos(-globalPref->detector_psi) + tz*sin(-globalPref->detector_psi);
    points[i].y = y + r*sin(phi);
  }
  // gdk_draw_polygon is supposed to be deprecated
  gdk_draw_polygon(globalPref->pixmap, globalPref->gc,0,
		   &points[0],NCIRCLE);
}

gboolean draw_2DView(GtkWidget *da,
			  GdkEventExpose *event,
			  gpointer data) {

  int i, j, dest_x, dest_y, dest_w, dest_h, offset_x, offset_y;
  int window_width, window_height, flag;
  float x1,x2,y1,y2,q,q0,alpha,qx,qy,kk,th0,beta;
  float center_x, center_y, scale_factor, radius, theta, dx, dy, dx0, dy0, qr, qz;
  GdkGC *gc;
  GdkColor color;

  gtk_window_get_size ( GTK_WINDOW (w2DView), &window_width, &window_height );
  globalPref->viewport_width = window_width / globalPref->zoom ; 
  globalPref->viewport_height = window_height / globalPref->zoom ; 

  switch (dataset->status) {
  case DATA_STATUS_EMPTY:
    return FALSE;
  case DATA_STATUS_LOADED: 
    conv_raw_indexed(dataset, globalPref->zmax, globalPref->zmin);
  case DATA_STATUS_INDEXED: 
    //conv_indexed_pixbuf(dataset);
    /*
      1. need to know which part of the pixbuf will be drawn to the 
      screen, given the size of the window and zoom
      2. scale the visible pixbuf to zoombuf and render onto the screen
      
      operations like zoom and move changes the parameters
      the screen is then redrawn based on these parameters
      a. pixel position that correspond to the screen center
      b. zoom
      
      the on-screen part of the data (vireport) will be drawn into the upper-left 
      corner of zoombuf. in data coordinates, the center of the viewport is 
      viewport_cx and the size is window.width/zoom. 
    */
    
    /* need to redraw the pixmap when
       1. zoom
       2. window resize
       3. move
       3. pixbuf itself changed (color changed, file loaded)
    */
      
    /*
      First, make sure the viewport does not go out of bounds
    */
    
    flag=0; // flag for filling/clearing the pixbuf
    if ( window_width > dataset->width*globalPref->zoom ) { 
      /* this will never be executed at fixed window size and zoom>0.5 */
      //globalPref->viewport_cx = globalPref->viewport_width / 2;
      globalPref->viewport_cx = dataset->width / 2;
      //dest_x = window_width/2 - dataset->width*globalPref->zoom / 2;
      dest_w = dataset->width*globalPref->zoom;
      //offset_x = 0;
      //dest_x = 0;
      dest_x = offset_x = window_width/2 - dataset->width*globalPref->zoom / 2;
      flag=1;
      //printf("dest_x=%d, dest_w=%d, offset_x=%d\n",dest_x,dest_w,offset_x);
    } else {
      if ( globalPref->viewport_cx < globalPref->viewport_width / 2 )
	globalPref->viewport_cx = globalPref->viewport_width / 2;
      else if ( globalPref->viewport_cx + globalPref->viewport_width / 2 
		> dataset->width)
	globalPref->viewport_cx = dataset->width - globalPref->viewport_width / 2;
      offset_x = window_width/2 - globalPref->viewport_cx*globalPref->zoom;
      dest_w = window_width;
      dest_x = 0;
    }
    
    if ( window_height > dataset->height*globalPref->zoom ) {
      //globalPref->viewport_cy = globalPref->viewport_height / 2;
      globalPref->viewport_cy = dataset->height / 2;
      offset_y = dest_y = window_height/2 - dataset->height*globalPref->zoom / 2;
      dest_h = dataset->height*globalPref->zoom;
      //offset_y = 0;
      flag=1;
      //printf("dest_y=%d, dest_h=%d, offset_y=%d\n",dest_x,dest_w,offset_x);
    } else {
      if ( globalPref->viewport_cy < globalPref->viewport_height / 2 )
	globalPref->viewport_cy = globalPref->viewport_height / 2;
      else if ( globalPref->viewport_cy + globalPref->viewport_height / 2 
		> dataset->height)
	globalPref->viewport_cy = dataset->height - globalPref->viewport_height / 2;
      offset_y = window_height/2 - globalPref->viewport_cy*globalPref->zoom;
      dest_h = window_height;
      dest_y = 0;
    }
    
    if (flag) gdk_pixbuf_fill(globalPref->zoombuf,0);
    gdk_pixbuf_scale ( dataset->pixbuf,
		       globalPref->zoombuf,
		       dest_x, dest_y, // dest x and y 
		       dest_w, dest_h, // dest w and h 
		       offset_x, offset_y, // offset x and y 
		       globalPref->zoom, globalPref->zoom,
		       GDK_INTERP_TILES );
    
    //dataset->status = DATA_STATUS_DRAWN;
  case DATA_STATUS_NEED_DECOR:
    /* this function is supposed to be deprecated */
    gdk_pixbuf_render_to_drawable ( globalPref->zoombuf,
				    globalPref->pixmap, 
				    da2DView->style->fg_gc[GTK_WIDGET_STATE (da2DView)],
				    0, 0,
				    0, 0,
				    window_width, window_height,
				    GDK_RGB_DITHER_NONE,
				    0, 0 ); 
    /**/
    // gdk_cairo_set_source_pixbuf (globalPref->cr, globalPref->zoombuf, 0, 0);
     
    if (!gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (ckbtnDetRec))) { 

      /* 
       *	 now draw the reference pattern, detector image only 
       */
      // +0.5 because (x,y) is the upper left corner of the pixel when magnified
      center_x = window_width/2 + (globalPref->cx+0.5 - globalPref->viewport_cx)*globalPref->zoom;
      center_y = window_height/2 + (globalPref->cy+0.5 - globalPref->viewport_cy)*globalPref->zoom;
      //printf("center is (%d,%d)\n",(int)center_x,(int)center_y);
      
      if (globalPref->samGeometry) {  // need to show sample normal
	gdk_gc_set_rgb_fg_color         (globalPref->gc,
					 &globalPref->color_normal);
	gdk_gc_set_line_attributes      (globalPref->gc,
					 1, // line_width
					 GDK_LINE_SOLID, // line_style
					 GDK_CAP_NOT_LAST, // cap_style
					 GDK_JOIN_ROUND); // join_style
	/* Original code
	theta = globalPref->normal*Pi/180.0;
	offset_x = (int)(0.5 + 300.0*sin(theta));
	offset_y = (int)(0.5 + 300.0*cos(theta));
	gdk_draw_line (globalPref->pixmap, globalPref->gc,
		       center_x, center_y, 
		       center_x + offset_x, center_y - offset_y);
	*/
	// updated May 2009
	// qr-axis
	q0=pos_to_q(globalPref->cx+1024,globalPref->cy);
	flag=0;
	for (j=-N_cir;j<=N_cir;j++) {
	  q=q0*j/N_cir;
	  if (qxy_to_pos(q,0,&dx,&dy)<0) {
	    // point not visible on the detector
	    flag=0;
	    continue; 
	  }
	  dx = window_width/2 + (dx+0.5 - globalPref->viewport_cx)*globalPref->zoom;
	  dy = window_height/2 + (dy+0.5 - globalPref->viewport_cy)*globalPref->zoom;
	  if (flag) gdk_draw_line (globalPref->pixmap, globalPref->gc,dx0, dy0, dx, dy);
	  dx0=dx; dy0=dy;
	  flag=1;
	}	  
	// "horizon"
	// K_in has fixed z component, the x component is zero and the y component (along the beam) varies 
	//    the z component is -k sin(alpha), the y component is k cos(alpha)
	// K_out is within the sample plane, therefore has zero z component but non-zero x,y components
	//    the y component is k cos(theta), the x component is k sin(theta)
	// qz = (K_out - K_in)_z = k sin(alpha)
	// qy = k cos(theta) - k cos(alpha)
	// qx = k sin(theta)
	//
	// more strictly, K_out has a finite z component too, 
	// with the exit angle beta=critical angle
	// the components of K_out are
	// x: k sin(theta)
	// y: k cos(theta)cos(beta)
	// z: k cos(theta)sin(beta)
	// therefore
	// qz = k cos(theta)sin(beta) + k sin(alpha)
	// qy = k cos(theta)cos(beta) - k cos(alpha)
	// qx = k sin(theta)
	flag=0;
	th0 = globalPref->wavelength*q0/(4.0*Pi);
	alpha=globalPref->incident*Pi/180.0;
	beta=0.2*Pi/180.0;
	kk=2*Pi/globalPref->wavelength;
	for (j=-N_cir;j<=N_cir;j++) {
	  theta=th0*j/N_cir*2;
	  qz=kk*(cos(theta)*sin(beta)+sin(alpha));
	  qy=kk*(cos(theta)*cos(beta)-cos(alpha));
	  qx=kk*sin(theta);
	  qr=sqrt(qx*qx+qy*qy);
	  if (theta<0) qr*=-1;
	  if (qrz_to_pos(qr,qz,&dx,&dy)<0) {
	    // point not visible on the detector
	    flag=0;
	    continue; 
	  }
	  dx = window_width/2 + (dx+0.5 - globalPref->viewport_cx)*globalPref->zoom;
	  dy = window_height/2 + (dy+0.5 - globalPref->viewport_cy)*globalPref->zoom;
	  if (flag) gdk_draw_line (globalPref->pixmap, globalPref->gc,dx0, dy0, dx, dy);
	  dx0=dx; dy0=dy;
	  flag=1;
	}	    
	// qz-axis
	flag=0;
	for (j=0;j<N_cir;j++) {
	  q=q0*j/N_cir;
	  theta=q*globalPref->wavelength/(4.0*Pi);
	  if (qxy_to_pos(q,Pi/2,&dx,&dy)<0) {
	    // point not visible on the detector
	    flag=0;
	    continue; 
	  }
	  dx = window_width/2 + (dx+0.5 - globalPref->viewport_cx)*globalPref->zoom;
	  dy = window_height/2 + (dy+0.5 - globalPref->viewport_cy)*globalPref->zoom;
	  if (flag) gdk_draw_line (globalPref->pixmap, globalPref->gc,dx0, dy0, dx, dy);
	  dx0=dx; dy0=dy;
	  flag=1;
	}	    
      }
      
      if (globalPref->refPattern) {
	gdk_gc_set_rgb_fg_color         (globalPref->gc,
					 &globalPref->color_refPat);
	gdk_gc_set_line_attributes      (globalPref->gc,
					 1, // line_width
					 GDK_LINE_SOLID, // line_style
					 GDK_CAP_NOT_LAST, // cap_style
					 GDK_JOIN_ROUND); // join_style
	
	switch (globalPref->refPattern) {
	case 1:  // sucrose
	  for (i=0;i< N_SUCROSE;i++) { // N_SUCROSE
	    /* OBSOLETE
	    theta = q_sucrose[i] * globalPref->wavelength / (4.0 * Pi);
	    if (theta>=0.5) continue; // scattering angle is too large
	    theta = asin (theta);
	    radius = globalPref->ratioDd * tan(2.0*theta) 
	      * dataset->width * globalPref->zoom;
	    draw_circle(center_x,  center_y, radius);
	    //scale_factor = ;
	    */
	    flag=0;
	    for (j=0;j<=N_cir;j++) {
	      if (qxy_to_pos(q_sucrose[i],2.0*Pi/N_cir*j,&dx,&dy)<0) {
		// point not visible on the detector
		flag=0;
		continue; 
	      }
	      dx = window_width/2 + (dx+0.5 - globalPref->viewport_cx)*globalPref->zoom;
	      dy = window_height/2 + (dy+0.5 - globalPref->viewport_cy)*globalPref->zoom;
	      if (flag) gdk_draw_line (globalPref->pixmap, globalPref->gc,dx0, dy0, dx, dy);
	      dx0=dx; dy0=dy;
	      flag=1;
	    }	    
	  }
	  break;
	case 2:  // AgBH
	  for (i=1;i< N_AGBH;i++) {
	    /* original code: ring of constant q = circle 
	    theta = QO_AgBH * i * globalPref->wavelength / (4.0 * Pi);
	    if (theta>=0.5) continue; // scattering angle is too large
	    theta = asin (theta);
	    radius = globalPref->ratioDd * tan(2.0*theta) 
	      * dataset->width * globalPref->zoom;
	    draw_circle(center_x,  center_y, radius);
	    */
	    //scale_factor = ;
	    flag=0;
	    for (j=0;j<=N_cir;j++) {
	      /* obsolete code, updated 10/20/09
	      qr = QO_AgBH*i*cos(2.0*Pi*j/N_cir);
	      qz = QO_AgBH*i*sin(2.0*Pi*j/N_cir);
	      if (qxy_to_pos(qr,qz,&dx,&dy)<0) {// point not visible on the detector
	      */ 
	      if (qxy_to_pos(QO_AgBH*i,2.0*Pi/N_cir*j,&dx,&dy)<0) {
		// point not visible on the detector
		flag=0;
		continue; 
	      }
	      dx = window_width/2 + (dx+0.5 - globalPref->viewport_cx)*globalPref->zoom;
	      dy = window_height/2 + (dy+0.5 - globalPref->viewport_cy)*globalPref->zoom;
	      if (flag) gdk_draw_line (globalPref->pixmap, globalPref->gc,dx0, dy0, dx, dy);
	      dx0=dx; dy0=dy;
	      flag=1;
	    }
	  }
	  break;
	case 3: //user defined, read from HOME_ENV/.view.gtk/ref_pattern
	  if (globalPref->n_ref==0) break;
	  for (i=0;i<globalPref->n_ref;i++) {
	    if (globalPref->q_ref[i].color<0 || globalPref->q_ref[i].color>7) {
	      printf("ref pattern color out of bounds: %d\n",globalPref->q_ref[i].color);
	      globalPref->q_ref[i].color%=8;
	    }
	    gdk_gc_set_rgb_fg_color(globalPref->gc,&colors[globalPref->q_ref[i].color]);
	    switch (globalPref->q_ref[i].type) {
	    case 'R': // ring at q
	      /* original code, ring = circle
	      theta = globalPref->q_ref[i].para[0] * globalPref->wavelength / (4.0 * Pi);
	      if (theta>=0.5) continue; // scattering angle is too large
	      theta = asin (theta);
	      radius = globalPref->ratioDd * tan(2.0*theta) 
		* dataset->width * globalPref->zoom;
	      draw_circle(center_x,  center_y, radius);
	      */
	      //  updated May 2009
	      // due to detector tilt, a ring of constant q is no longer circular
	      flag=0;
	      for (j=0;j<=N_cir;j++) {
		/* obsolete code, updated 10/20/09
		qr = globalPref->q_ref[i].para[0]*cos(2.0*Pi*j/N_cir);
		qz = globalPref->q_ref[i].para[0]*sin(2.0*Pi*j/N_cir);
		if (qrz_to_pos(qr,qz,&dx,&dy)<0) {// point not visible on the detector
		*/
		if (qxy_to_pos(globalPref->q_ref[i].para[0],2.0*Pi/N_cir*j,&dx,&dy)<0) {
		    flag=0;
		  continue; 
		}
		dx = window_width/2 + (dx+0.5 - globalPref->viewport_cx)*globalPref->zoom;
		dy = window_height/2 + (dy+0.5 - globalPref->viewport_cy)*globalPref->zoom;
		if (flag) gdk_draw_line (globalPref->pixmap, globalPref->gc,dx0, dy0, dx, dy);
		dx0=dx; dy0=dy;
		flag=1;
	      }
	      break;
	    case 'P': // point at (qr, qz)
	      if (qrz_to_pos(globalPref->q_ref[i].para[0],globalPref->q_ref[i].para[1],
			     &dx,&dy)<0) break; // point not visible on the detector
	      dx = window_width/2 + (dx+0.5 - globalPref->viewport_cx)*globalPref->zoom;
	      dy = window_height/2 + (dy+0.5 - globalPref->viewport_cy)*globalPref->zoom;
	      gdk_draw_line (globalPref->pixmap, globalPref->gc,
			     dx-CROSSSIZE, dy, dx+CROSSSIZE, dy);
	      gdk_draw_line (globalPref->pixmap, globalPref->gc,
			     dx, dy-CROSSSIZE, dx, dy+CROSSSIZE);
	      break;
	    case 'L':
	      flag=0;
	      for (j=0;j<globalPref->q_ref[i].N;j++) {
		qr = (globalPref->q_ref[i].para[2]-globalPref->q_ref[i].para[0])
		  *j/globalPref->q_ref[i].N+globalPref->q_ref[i].para[0];
		qz = (globalPref->q_ref[i].para[3]-globalPref->q_ref[i].para[1])
		  *j/globalPref->q_ref[i].N+globalPref->q_ref[i].para[1];
		if (qrz_to_pos(qr,qz,&dx,&dy)<0) {// point not visible on the detector
		  flag=0;
		  continue; 
		}
		dx = window_width/2 + (dx+0.5 - globalPref->viewport_cx)*globalPref->zoom;
		dy = window_height/2 + (dy+0.5 - globalPref->viewport_cy)*globalPref->zoom;
		if (flag) gdk_draw_line (globalPref->pixmap, globalPref->gc,dx0, dy0, dx, dy);
		dx0=dx; dy0=dy;
		flag=1;
	      }
	      break;
	    }
	  }
	}  
      }

      if (globalPref->showpeak && globalPref->peak_N>0) { 
	// show the diffraction peaks that are found by clicking "show peaks"
	gdk_gc_set_rgb_fg_color         (globalPref->gc,
					 &globalPref->color_peaks);
	gdk_gc_set_line_attributes      (globalPref->gc,
					 1, // line_width
					 GDK_LINE_SOLID, // line_style
					 GDK_CAP_NOT_LAST, // cap_style
					 GDK_JOIN_ROUND); // join_style
	for (i=0;i<globalPref->peak_N;i++) {
	  dx = window_width/2 + (globalPref->peak_x[i]+0.5 - globalPref->viewport_cx)*globalPref->zoom;
	  dy = window_height/2 + (globalPref->peak_y[i]+0.5 - globalPref->viewport_cy)*globalPref->zoom;
	  gdk_draw_line (globalPref->pixmap, globalPref->gc,
			 dx-CROSSSIZE, dy, dx+CROSSSIZE, dy);
	  gdk_draw_line (globalPref->pixmap, globalPref->gc,
			 dx, dy-CROSSSIZE, dx, dy+CROSSSIZE);
	}
      }
      
    }
    dataset->status = DATA_STATUS_DRAWN;
  case DATA_STATUS_DRAWN:
    gdk_draw_pixmap ( da2DView->window, 
		      da2DView->style->fg_gc[GTK_WIDGET_STATE (da2DView)],
		      globalPref->pixmap,
		      event->area.x, event->area.y,
		      event->area.x, event->area.y,
		      event->area.width, event->area.height);
  }    
  
  return TRUE;

}
