#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "draw.h"
#include "data.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

GtkWidget*
create_w2DView (int wsize)
{
  GtkWidget *w2DView;
  GtkWidget *drawing_area;

  w2DView = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request (w2DView, wsize, wsize);
  gtk_window_set_resizable (GTK_WINDOW (w2DView), FALSE);
  gtk_window_set_title (GTK_WINDOW (w2DView), _("2D view"));

  drawing_area = gtk_drawing_area_new ();
  gtk_widget_show (drawing_area);
  gtk_container_add (GTK_CONTAINER (w2DView), drawing_area);

  g_signal_connect ((gpointer) w2DView, "delete-event",
                    G_CALLBACK (gtk_true),
                    NULL);
  g_signal_connect ((gpointer) drawing_area, "expose_event",
		    G_CALLBACK (draw_2DView), 
		    NULL);
  gtk_signal_connect (GTK_OBJECT (drawing_area), "motion_notify_event",
		      (GtkSignalFunc) motion_notify_event, NULL);
  gtk_signal_connect (GTK_OBJECT (drawing_area), "button_press_event",
		      (GtkSignalFunc) button_press_event, NULL);
  gtk_signal_connect (GTK_OBJECT (drawing_area), "button_release_event",
		      (GtkSignalFunc) button_release_event, NULL);
  
  gtk_widget_set_events (drawing_area, GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_POINTER_MOTION_MASK
			 | GDK_POINTER_MOTION_HINT_MASK);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (w2DView, w2DView, "w2DView");
  GLADE_HOOKUP_OBJECT (w2DView, drawing_area, "da2DView");

  return w2DView;
}

GtkWidget*
create_wControl (void)
{
  GtkWidget *wControl;
  GtkWidget *vbox_control_window;
  GtkWidget *menubar1;
  GtkWidget *menuitem1;
  GtkWidget *menuitem1_menu;
  GtkWidget *open1;
  GtkWidget *save_as_jpg1;
  GtkWidget *separatormenuitem1;
  GtkWidget *quit1;
  GtkWidget *menuitem3;
  GtkWidget *menuitem3_menu;
  GSList *format_mar_group = NULL;
  GtkWidget *format_mar;
  GtkWidget *format_waxs;
  GtkWidget *format_pilatus;
  GtkWidget *format_binary_short;
  GtkWidget *format_bruker;
  GtkWidget *format_PI;
  GtkWidget *menuitem4;
  GtkWidget *menuitem4_menu;
  GtkWidget *about1;
  GtkWidget *fontSel;
  GtkWidget *notebook1;
  GtkWidget *vbox14;
  GtkWidget *hbox25;
  GtkWidget *frame_experimental;
  GtkWidget *vbox12;
  GtkWidget *table3;
  GtkWidget *label15;
  GtkWidget *label16;
  GtkWidget *label17;
  GtkWidget *label18;
  GtkWidget *label19;
  GtkWidget *etCenY;
  GtkWidget *etCenX;
  //GtkWidget *etDetSwing;
  GtkWidget *etDd;
  GtkWidget *etWavelength;
  GtkWidget *btConfDetGeo;
  GtkWidget *ckbtnGeo;
  GtkWidget *hbox23;
  GtkWidget *label76;
  GtkWidget *etSamNormal;
  GtkWidget *label77;
  GtkWidget *etIncidentAng;
  GtkWidget *hbox24;
  GtkWidget *label91;
  GtkObject *sbParaSet_adj;
  GtkWidget *sbParaSet;
  GtkWidget *btnSavePara;
  GtkWidget *btnLoadPara;
  GtkWidget *label13;
  GtkWidget *frame_color;
  GtkWidget *vbox_color;
  GtkWidget *hbox4;
  GtkWidget *label24;
  GtkWidget *etZmin;
  GtkWidget *label25;
  GtkWidget *etZmax;
  GtkWidget *tbAutoZScale;
  GtkWidget *hbox5;
  GtkWidget *btColor0;
  GtkWidget *btColor1;
  GtkWidget *btColor2;
  GtkWidget *btColor3;
  GtkWidget *btColor4;
  GtkWidget *daColorTable;
  GtkWidget *table2;
  GtkWidget *label11;
  GtkWidget *label12;
  GtkWidget *sclContrast;
  GtkWidget *sclBrightness;
  GtkWidget *hbox3;
  GtkWidget *cbtnLog;
  GtkWidget *label22;
  GtkWidget *etGamma;
  GtkWidget *label5;
  GtkWidget *hbox26;
  GtkWidget *frame_zoom;
  GtkWidget *cbxZoom;
  GtkWidget *label14;
  GtkWidget *frame32;
  GtkWidget *fixed2;
  GtkWidget *cbxRecent;
  GtkWidget *label89;
  GtkWidget *label1;
  GtkWidget *vbox_ext_tab;
  GtkWidget *hbox9;
  GtkWidget *frame14;
  GtkWidget *hbox18;
  GtkWidget *cbxRef;
  GtkWidget *label35;
  GtkWidget *frame15;
  GtkWidget *btShowStat;
  GtkWidget *label36;
  GtkWidget *hbox16;
  GtkWidget *frame30;
  GtkWidget *fixed1;
  GtkWidget *table6;
  GtkWidget *cbxFlatField;
  GtkWidget *label80;
  GtkWidget *label79;
  GtkWidget *label78;
  GtkWidget *cbxDistortion;
  GtkWidget *cbxMask;
  GtkWidget *label83;
  GtkWidget *frame31;
  GtkWidget *vbox8;
  GtkWidget *hbox17;
  GtkWidget *label67;
  GtkWidget *et1DGrid;
  GtkWidget *label68;
  GtkWidget *hbox19;
  GtkWidget *cbUpdate;
  GtkWidget *label84;
  GtkWidget *frame25;
  GtkWidget *hbox20;
  GtkWidget *cbtnDezinger;
  GtkWidget *label70;
  GtkWidget *etZingerRadius;
  GtkWidget *label71;
  GtkWidget *label72;
  GtkWidget *etZingerTolerance;
  GtkWidget *label69;
  GtkWidget *label2;
  GtkWidget *vbox_multi_tab;
  GtkWidget *frame7;
  GtkWidget *scrolledwindow8;
  GtkWidget *tvFileList;
  GtkWidget *label28;
  GtkWidget *hbox21;
  GtkWidget *frame28;
  GtkWidget *scrolledwindow7;
  GtkWidget *tvComList;
  GtkWidget *label81;
  GtkWidget *frame33;
  GtkWidget *vbox13;
  GtkWidget *hbox22;
  GtkWidget *btnFirst;
  GtkWidget *image6;
  GtkWidget *btnBack;
  GtkWidget *image7;
  GtkWidget *btnForward;
  GtkWidget *image8;
  GtkWidget *btnLast;
  GtkWidget *image9;
  GtkWidget *btnGo;
  GtkWidget *label90;
  GtkWidget *label3;
  GtkWidget *vbox_manip_tab;
  GtkWidget *frame19;
  GtkWidget *hbox13a;
  GtkWidget *btnStore;
  GtkWidget *btnRecall;
  GtkObject *sbCurSet_adj;
  GtkWidget *sbCurSet;
  GtkWidget *etSetComments;
  GtkWidget *label45;
  GtkWidget *frame20;
  GtkWidget *hbox14;
  GtkObject *sbSet1_adj;
  GtkWidget *sbSet1;
  GtkWidget *label48;
  GtkObject *sbSet2_adj;
  GtkWidget *sbSet2;
  GtkWidget *label49;
  GtkWidget *etConst;
  GtkWidget *label86;
  GtkWidget *etScaleFactor;
  GtkWidget *label87;
  GtkWidget *btnCalculate;
  GtkWidget *btnSaveData;
  GtkWidget *label46;
  GtkWidget *frame_GISAXS;
  GtkWidget *hbox15;
  GtkWidget *cbDetRec;
  GtkWidget *vseparator1;
  GtkWidget *btMarkQrz;
  GtkWidget *label53;
  GtkWidget *etQr;
  GtkWidget *label54;
  GtkWidget *etQz;
  GtkWidget *label55;
  GtkWidget *label_GISAXS;
  GtkWidget *frame21;
  GtkWidget *table4;
  GtkWidget *label57;
  GtkWidget *etLineProfileCx;
  GtkWidget *label59;
  GtkWidget *etLineProfileLength;
  GtkWidget *label60;
  GtkWidget *etLineProfileOrient;
  GtkWidget *label61;
  GtkWidget *etLineProfileRadius;
  GtkWidget *btnExtract;
  GtkWidget *etLineProfileCy;
  GtkWidget *label58;
  GtkWidget *label88;
  GtkWidget *label_ex;
  GtkWidget *label44;
  GtkWidget *scrolledwindow6;
  GtkWidget *tvInfo;
  GtkWidget *hseparator1;
  GtkWidget *lbStatusMsg;
  GtkAccelGroup *accel_group;
  GtkWidget *frame34;
  GtkWidget *hbox27;
  GtkWidget *label93;
  GtkWidget *etPeakThresh;
  GtkWidget *tbShowPeak;
  GtkWidget *label92;

  accel_group = gtk_accel_group_new ();

  wControl = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request (wControl, 400, 400);
  gtk_window_set_title (GTK_WINDOW (wControl), _("Control"));
  //gtk_window_set_resizable (GTK_WINDOW (wControl), FALSE);

  vbox_control_window = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox_control_window);
  gtk_container_add (GTK_CONTAINER (wControl), vbox_control_window);

  menubar1 = gtk_menu_bar_new ();
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (vbox_control_window), menubar1, FALSE, FALSE, 0);

  menuitem1 = gtk_menu_item_new_with_mnemonic (_("_File"));
  gtk_widget_show (menuitem1);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem1);

  menuitem1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem1), menuitem1_menu);

  open1 = gtk_image_menu_item_new_from_stock ("gtk-open", accel_group);
  gtk_widget_show (open1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), open1);

  save_as_jpg1 = gtk_menu_item_new_with_mnemonic (_("Save as JPG"));
  gtk_widget_show (save_as_jpg1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), save_as_jpg1);

  fontSel = gtk_menu_item_new_with_mnemonic (_("Change font"));
  gtk_widget_show (fontSel);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), fontSel);

  separatormenuitem1 = gtk_separator_menu_item_new ();
  gtk_widget_show (separatormenuitem1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), separatormenuitem1);
  gtk_widget_set_sensitive (separatormenuitem1, FALSE);

  quit1 = gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
  gtk_widget_show (quit1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), quit1);

  menuitem3 = gtk_menu_item_new_with_mnemonic (_("_Format"));
  gtk_widget_show (menuitem3);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem3);

  menuitem3_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem3), menuitem3_menu);

  format_mar = gtk_radio_menu_item_new_with_mnemonic (format_mar_group, _("Mar CCD"));
  format_mar_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (format_mar));
  gtk_widget_show (format_mar);
  gtk_container_add (GTK_CONTAINER (menuitem3_menu), format_mar);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (format_mar), TRUE);

  format_waxs = gtk_radio_menu_item_new_with_mnemonic (format_mar_group, _("WAXS tiff 16bpp"));
  format_mar_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (format_waxs));
  gtk_widget_show (format_waxs);
  gtk_container_add (GTK_CONTAINER (menuitem3_menu), format_waxs);

  format_pilatus = gtk_radio_menu_item_new_with_mnemonic (format_mar_group, _("PILATUS tiff 32bpp"));
  format_mar_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (format_pilatus));
  gtk_widget_show (format_pilatus);
  gtk_container_add (GTK_CONTAINER (menuitem3_menu), format_pilatus);

  format_binary_short = gtk_radio_menu_item_new_with_mnemonic (format_mar_group, _("raw binary"));
  format_mar_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (format_binary_short));
  gtk_widget_show (format_binary_short);
  gtk_container_add (GTK_CONTAINER (menuitem3_menu), format_binary_short);

  format_bruker = gtk_radio_menu_item_new_with_mnemonic (format_mar_group, _("Bruker"));
  format_mar_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (format_bruker));
  gtk_widget_show (format_bruker);
  gtk_container_add (GTK_CONTAINER (menuitem3_menu), format_bruker);

  format_PI = gtk_radio_menu_item_new_with_mnemonic (format_mar_group, _("Princeton / Roper"));
  format_mar_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (format_PI));
  gtk_widget_show (format_PI);
  gtk_container_add (GTK_CONTAINER (menuitem3_menu), format_PI);

  menuitem4 = gtk_menu_item_new_with_mnemonic (_("_Help"));
  gtk_widget_show (menuitem4);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem4);

  menuitem4_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem4), menuitem4_menu);

  about1 = gtk_menu_item_new_with_mnemonic (_("_About"));
  gtk_widget_show (about1);
  gtk_container_add (GTK_CONTAINER (menuitem4_menu), about1);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (vbox_control_window), notebook1, FALSE, FALSE, 0);
  //gtk_widget_set_size_request (notebook1, 176, 260);

  vbox14 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox14);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox14);

  hbox25 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox25);
  gtk_box_pack_start (GTK_BOX (vbox14), hbox25, TRUE, TRUE, 0);

  frame_experimental = gtk_frame_new (NULL);
  gtk_widget_show (frame_experimental);
  gtk_box_pack_start (GTK_BOX (hbox25), frame_experimental, TRUE, FALSE, 0);
  //gtk_widget_set_size_request (frame_experimental, 150, 170);

  vbox12 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox12);
  gtk_container_add (GTK_CONTAINER (frame_experimental), vbox12);

  table3 = gtk_table_new (5, 2, FALSE);
  gtk_widget_show (table3);
  gtk_box_pack_start (GTK_BOX (vbox12), table3, FALSE, TRUE, 0);
  gtk_table_set_col_spacings (GTK_TABLE (table3), 2);

  label15 = gtk_label_new (_("wavelength (ang)"));
  gtk_widget_show (label15);
  gtk_table_attach (GTK_TABLE (table3), label15, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label15), 0, 0.5);

  label16 = gtk_label_new (_("D/d"));
  gtk_widget_show (label16);
  gtk_table_attach (GTK_TABLE (table3), label16, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label16), 0, 0.5);

  label17 = gtk_label_new (_("detector geometry"));
  gtk_widget_show (label17);
  gtk_table_attach (GTK_TABLE (table3), label17, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label17), 0, 0.5);

  label18 = gtk_label_new (_("beam center (x)"));
  gtk_widget_show (label18);
  gtk_table_attach (GTK_TABLE (table3), label18, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label18), 0, 0.5);

  label19 = gtk_label_new (_("beam center (y)"));
  gtk_widget_show (label19);
  gtk_table_attach (GTK_TABLE (table3), label19, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label19), 0, 0.5);

  etCenY = gtk_entry_new ();
  gtk_widget_show (etCenY);
  gtk_table_attach (GTK_TABLE (table3), etCenY, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 1, 0);
  //gtk_widget_set_size_request (etCenY, 35, 21);
  gtk_entry_set_width_chars (GTK_ENTRY (etCenY), 5);
  gtk_entry_set_text (GTK_ENTRY (etCenY), _("512"));

  etCenX = gtk_entry_new ();
  gtk_widget_show (etCenX);
  gtk_table_attach (GTK_TABLE (table3), etCenX, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 1, 0);
  //gtk_widget_set_size_request (etCenX, 35, 21);
  gtk_entry_set_width_chars (GTK_ENTRY (etCenX), 5);
  gtk_entry_set_text (GTK_ENTRY (etCenX), _("512"));

  /*
  etDetSwing = gtk_entry_new ();
  gtk_widget_show (etDetSwing);
  gtk_table_attach (GTK_TABLE (table3), etDetSwing, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 1, 0);
  gtk_widget_set_size_request (etDetSwing, 35, 21);
  gtk_entry_set_text (GTK_ENTRY (etDetSwing), _("0"));
  */

  btConfDetGeo = gtk_button_new_with_mnemonic (_("config"));
  gtk_widget_show (btConfDetGeo);
  gtk_table_attach (GTK_TABLE (table3), btConfDetGeo, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  etDd = gtk_entry_new ();
  gtk_widget_show (etDd);
  gtk_table_attach (GTK_TABLE (table3), etDd, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 1, 0);
  //gtk_widget_set_size_request (etDd, 35, 21);
  gtk_entry_set_width_chars (GTK_ENTRY (etDd), 5);
  gtk_entry_set_text (GTK_ENTRY (etDd), _("3.00"));

  etWavelength = gtk_entry_new ();
  gtk_widget_show (etWavelength);
  gtk_table_attach (GTK_TABLE (table3), etWavelength, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 1, 0);
  //gtk_widget_set_size_request (etWavelength, 35, 21);
  gtk_entry_set_width_chars (GTK_ENTRY (etWavelength), 5);
  gtk_entry_set_text (GTK_ENTRY (etWavelength), _("1.54"));

  ckbtnGeo = gtk_check_button_new_with_mnemonic (_("aligned sample"));
  gtk_widget_show (ckbtnGeo);
  gtk_box_pack_start (GTK_BOX (vbox12), ckbtnGeo, FALSE, FALSE, 0);

  hbox23 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox23);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox23, TRUE, TRUE, 0);

  label76 = gtk_label_new (_("sample\nnormal"));
  gtk_widget_show (label76);
  gtk_box_pack_start (GTK_BOX (hbox23), label76, TRUE, FALSE, 2);

  etSamNormal = gtk_entry_new ();
  gtk_widget_show (etSamNormal);
  gtk_box_pack_start (GTK_BOX (hbox23), etSamNormal, TRUE, FALSE, 3);
  //gtk_widget_set_size_request (etSamNormal, 25, -1);
  gtk_widget_set_sensitive (etSamNormal, FALSE);
  gtk_entry_set_width_chars (GTK_ENTRY (etSamNormal), 5);
  gtk_entry_set_text (GTK_ENTRY (etSamNormal), _("0.0"));

  label77 = gtk_label_new (_("incident\nangle"));
  gtk_widget_show (label77);
  gtk_box_pack_start (GTK_BOX (hbox23), label77, TRUE, FALSE, 2);
  gtk_label_set_justify (GTK_LABEL (label77), GTK_JUSTIFY_CENTER);

  etIncidentAng = gtk_entry_new ();
  gtk_widget_show (etIncidentAng);
  gtk_box_pack_start (GTK_BOX (hbox23), etIncidentAng, TRUE, FALSE, 5);
  //gtk_widget_set_size_request (etIncidentAng, 25, -1);
  gtk_widget_set_sensitive (etIncidentAng, FALSE);
  gtk_entry_set_width_chars (GTK_ENTRY (etIncidentAng), 5);
  gtk_entry_set_text (GTK_ENTRY (etIncidentAng), _("0.3"));

  hbox24 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox24);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox24, TRUE, TRUE, 0);

  label91 = gtk_label_new (_("config #"));
  gtk_widget_show (label91);
  gtk_box_pack_start (GTK_BOX (hbox24), label91, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label91), GTK_JUSTIFY_RIGHT);

  sbParaSet_adj = gtk_adjustment_new (0, 0, 5, 1, 10, 0);
  sbParaSet = gtk_spin_button_new (GTK_ADJUSTMENT (sbParaSet_adj), 1, 0);
  gtk_widget_show (sbParaSet);
  gtk_box_pack_start (GTK_BOX (hbox24), sbParaSet, FALSE, FALSE, 0);
  //gtk_widget_set_size_request (sbParaSet, 26, 18);
  gtk_entry_set_width_chars (GTK_ENTRY(sbParaSet), 1); 
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (sbParaSet), TRUE);

  btnSavePara = gtk_button_new_with_mnemonic (_("save"));
  gtk_widget_show (btnSavePara);
  gtk_box_pack_start (GTK_BOX (hbox24), btnSavePara, FALSE, FALSE, 0);

  btnLoadPara = gtk_button_new_with_mnemonic (_("load"));
  gtk_widget_show (btnLoadPara);
  gtk_box_pack_start (GTK_BOX (hbox24), btnLoadPara, FALSE, FALSE, 0);

  label13 = gtk_label_new (_("experimental"));
  gtk_widget_show (label13);
  gtk_frame_set_label_widget (GTK_FRAME (frame_experimental), label13);

  frame_color = gtk_frame_new (NULL);
  gtk_widget_show (frame_color);
  gtk_box_pack_start (GTK_BOX (hbox25), frame_color, TRUE, FALSE, 0);

  vbox_color = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox_color);
  gtk_container_add (GTK_CONTAINER (frame_color), vbox_color);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox_color), hbox4, FALSE, FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (hbox4), 2);

  label24 = gtk_label_new (_("min"));
  gtk_widget_show (label24);
  gtk_box_pack_start (GTK_BOX (hbox4), label24, FALSE, FALSE, 0);

  etZmin = gtk_entry_new ();
  gtk_widget_show (etZmin);
  gtk_entry_set_width_chars (GTK_ENTRY (etZmin), 5);
  gtk_box_pack_start (GTK_BOX (hbox4), etZmin, TRUE, FALSE, 2);

  label25 = gtk_label_new (_("max"));
  gtk_widget_show (label25);
  gtk_box_pack_start (GTK_BOX (hbox4), label25, FALSE, FALSE, 0);

  etZmax = gtk_entry_new ();
  gtk_widget_show (etZmax);
  gtk_entry_set_width_chars (GTK_ENTRY (etZmax), 5);
  gtk_box_pack_start (GTK_BOX (hbox4), etZmax, TRUE, FALSE, 2);

  tbAutoZScale = gtk_toggle_button_new_with_mnemonic (_("auto"));
  gtk_widget_show (tbAutoZScale);
  gtk_box_pack_start (GTK_BOX (hbox4), tbAutoZScale, FALSE, TRUE, 1);

  hbox5 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox_color), hbox5, FALSE, FALSE, 0);
  //gtk_widget_set_size_request (hbox5, 150, 20);

  btColor0 = gtk_button_new_with_mnemonic ("");
  gtk_widget_show (btColor0);
  gtk_box_pack_start (GTK_BOX (hbox5), btColor0, FALSE, TRUE, 0);

  btColor1 = gtk_button_new_with_mnemonic ("");
  gtk_widget_show (btColor1);
  gtk_box_pack_start (GTK_BOX (hbox5), btColor1, FALSE, TRUE, 0);

  btColor2 = gtk_button_new_with_mnemonic ("");
  gtk_widget_show (btColor2);
  gtk_box_pack_start (GTK_BOX (hbox5), btColor2, FALSE, TRUE, 0);

  btColor3 = gtk_button_new_with_mnemonic ("");
  gtk_widget_show (btColor3);
  gtk_box_pack_start (GTK_BOX (hbox5), btColor3, FALSE, TRUE, 0);

  btColor4 = gtk_button_new_with_mnemonic ("");
  gtk_widget_show (btColor4);
  gtk_box_pack_start (GTK_BOX (hbox5), btColor4, FALSE, TRUE, 0);

  daColorTable = gtk_drawing_area_new ();
  gtk_widget_show (daColorTable);
  gtk_box_pack_start (GTK_BOX (vbox_color), daColorTable, FALSE, FALSE, 2);
  //gtk_widget_set_size_request (daColorTable, 150, 20);

  table2 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (vbox_color), table2, FALSE, FALSE, 0);

  label11 = gtk_label_new (_("contrast"));
  gtk_widget_show (label11);
  gtk_table_attach (GTK_TABLE (table2), label11, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label11), 0, 0.5);

  label12 = gtk_label_new (_("brightness"));
  gtk_widget_show (label12);
  gtk_table_attach (GTK_TABLE (table2), label12, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label12), 0, 0.5);

  sclContrast = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (50, 0, 100, 1, 10, 0)));
  gtk_widget_show (sclContrast);
  gtk_table_attach (GTK_TABLE (table2), sclContrast, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_sensitive (sclContrast, FALSE);

  sclBrightness = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (50, 0, 100, 1, 10, 0)));
  gtk_widget_show (sclBrightness);
  gtk_table_attach (GTK_TABLE (table2), sclBrightness, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_sensitive (sclBrightness, FALSE);

  hbox3 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox_color), hbox3, FALSE, TRUE, 0);

  cbtnLog = gtk_check_button_new_with_mnemonic (_("log"));
  gtk_widget_show (cbtnLog);
  gtk_box_pack_start (GTK_BOX (hbox3), cbtnLog, FALSE, FALSE, 0);

  label22 = gtk_label_new (_("gamma="));
  gtk_widget_show (label22);
  gtk_box_pack_start (GTK_BOX (hbox3), label22, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label22), GTK_JUSTIFY_RIGHT);

  etGamma = gtk_entry_new ();
  gtk_widget_show (etGamma);
  gtk_box_pack_end (GTK_BOX (hbox3), etGamma, FALSE, TRUE, 4);
  gtk_widget_set_sensitive (etGamma, FALSE);
  gtk_entry_set_width_chars (GTK_ENTRY (etGamma), 4);
  gtk_entry_set_text (GTK_ENTRY (etGamma), _("0.3"));

  label5 = gtk_label_new (_("color"));
  gtk_widget_show (label5);
  gtk_frame_set_label_widget (GTK_FRAME (frame_color), label5);

  hbox26 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox26);
  gtk_box_pack_end (GTK_BOX (vbox14), hbox26, TRUE, TRUE, 0);

  frame_zoom = gtk_frame_new (NULL);
  gtk_widget_show (frame_zoom);
  gtk_box_pack_start (GTK_BOX (hbox26), frame_zoom, TRUE, FALSE, 0);

  cbxZoom = gtk_combo_box_new_text ();
  gtk_widget_show (cbxZoom);
  gtk_container_add (GTK_CONTAINER (frame_zoom), cbxZoom);
  //gtk_box_pack_start (GTK_BOX (hbox18), cbxZoom, TRUE, FALSE, 0);
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxZoom), _("x 0.5"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxZoom), _("x 1"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxZoom), _("x 2"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxZoom), _("x 4"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxZoom), _("x 8"));

  label14 = gtk_label_new (_("zoom"));
  gtk_widget_show (label14);
  gtk_frame_set_label_widget (GTK_FRAME (frame_zoom), label14);

  frame32 = gtk_frame_new (NULL);
  gtk_widget_show (frame32);
  gtk_box_pack_end (GTK_BOX (hbox26), frame32, FALSE, FALSE, 0);
  //gtk_widget_set_size_request (frame32, -1, 41);

  //fixed2 = gtk_fixed_new ();
  //gtk_widget_show (fixed2);
  //gtk_container_add (GTK_CONTAINER (frame32), fixed2);
  //gtk_widget_set_size_request (fixed2, 200, -1);

  cbxRecent = gtk_combo_box_new_text ();
  gtk_widget_show (cbxRecent);
  gtk_widget_set_size_request (cbxRecent, 320, -1);
  //gtk_fixed_put (GTK_FIXED (fixed2), cbxRecent, 0, 0);
  gtk_container_add (GTK_CONTAINER (frame32), cbxRecent);
  //gtk_container_set_resize_mode (GTK_CONTAINER (cbxRecent), GTK_RESIZE_PARENT);
  //gtk_combo_box_set_wrap_width (GTK_COMBO_BOX(cbxRecent), 1);

  label89 = gtk_label_new (_("recent images"));
  gtk_widget_show (label89);
  gtk_frame_set_label_widget (GTK_FRAME (frame32), label89);

  label1 = gtk_label_new (_("basics"));
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label1);

  vbox_ext_tab = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox_ext_tab);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox_ext_tab);

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox9);
  gtk_box_pack_start (GTK_BOX (vbox_ext_tab), hbox9, FALSE, TRUE, 0);
  //gtk_widget_set_size_request (hbox9, -1, 50);

  frame14 = gtk_frame_new (NULL);
  gtk_widget_show (frame14);
  gtk_box_pack_start (GTK_BOX (hbox9), frame14, FALSE, FALSE, 2);

  hbox18 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox18);
  gtk_container_add (GTK_CONTAINER (frame14), hbox18);
  gtk_container_set_border_width (GTK_CONTAINER (hbox18), 5);

  cbxRef = gtk_combo_box_new_text ();
  gtk_widget_show (cbxRef);
  gtk_box_pack_start (GTK_BOX (hbox18), cbxRef, TRUE, TRUE, 0);
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxRef), _("none"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxRef), _("Sucrose"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxRef), _("AgBH"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxRef), _("user defined"));

  label35 = gtk_label_new (_("reference pattern"));
  gtk_widget_show (label35);
  gtk_frame_set_label_widget (GTK_FRAME (frame14), label35);

  frame15 = gtk_frame_new (NULL);
  gtk_widget_show (frame15);
  gtk_box_pack_start (GTK_BOX (hbox9), frame15, TRUE, TRUE, 0);

  btShowStat = gtk_button_new_with_mnemonic (_("show"));
  gtk_widget_show (btShowStat);
  gtk_container_add (GTK_CONTAINER (frame15), btShowStat);
  gtk_container_set_border_width (GTK_CONTAINER (btShowStat), 5);
  gtk_widget_set_sensitive (btShowStat, FALSE);

  label36 = gtk_label_new (_("image stats"));
  gtk_widget_show (label36);
  gtk_frame_set_label_widget (GTK_FRAME (frame15), label36);

  hbox16 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox16);
  gtk_box_pack_start (GTK_BOX (vbox_ext_tab), hbox16, FALSE, FALSE, 0);

  frame30 = gtk_frame_new (NULL);
  gtk_widget_show (frame30);
  gtk_box_pack_start (GTK_BOX (hbox16), frame30, TRUE, TRUE, 2);

  fixed1 = gtk_fixed_new ();
  gtk_widget_show (fixed1);
  gtk_container_add (GTK_CONTAINER (frame30), fixed1);
  //gtk_widget_set_size_request (fixed1, 145, 74);

  table6 = gtk_table_new (3, 2, FALSE);
  gtk_widget_show (table6);
  gtk_fixed_put (GTK_FIXED (fixed1), table6, 0, 0);
  //gtk_widget_set_size_request (table6, 175, 75);

  cbxFlatField = gtk_combo_box_new_text ();
  gtk_widget_show (cbxFlatField);
  gtk_table_attach (GTK_TABLE (table6), cbxFlatField, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 1, 1);
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxFlatField), _("none"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxFlatField), _("load"));
  gtk_widget_set_size_request (cbxFlatField, 175, -1);

  label80 = gtk_label_new (_("distortion"));
  gtk_widget_show (label80);
  gtk_table_attach (GTK_TABLE (table6), label80, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 2, 0);
  gtk_misc_set_alignment (GTK_MISC (label80), 0, 0.5);

  label79 = gtk_label_new (_("f. flield "));
  gtk_widget_show (label79);
  gtk_table_attach (GTK_TABLE (table6), label79, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 2, 0);
  gtk_label_set_justify (GTK_LABEL (label79), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label79), 0, 0.5);

  label78 = gtk_label_new (_("mask"));
  gtk_widget_show (label78);
  gtk_table_attach (GTK_TABLE (table6), label78, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label78), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label78), 0, 0.5);

  cbxDistortion = gtk_combo_box_new_text ();
  gtk_widget_show (cbxDistortion);
  gtk_table_attach (GTK_TABLE (table6), cbxDistortion, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 1, 1);
  gtk_widget_set_sensitive (cbxDistortion, FALSE);
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxDistortion), _("none"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxDistortion), _("load"));
  gtk_widget_set_size_request (cbxDistortion, 175, -1);

  cbxMask = gtk_combo_box_new_text ();
  gtk_widget_show (cbxMask);
  gtk_table_attach (GTK_TABLE (table6), cbxMask, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 1, 1);
  //gtk_widget_set_size_request (cbxMask, 120, -1);
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxMask), _("none"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cbxMask), _("load"));
  gtk_widget_set_size_request (cbxMask, 175, -1);

  label83 = gtk_label_new (_("files"));
  gtk_widget_show (label83);
  gtk_frame_set_label_widget (GTK_FRAME (frame30), label83);
  gtk_label_set_use_markup (GTK_LABEL (label83), TRUE);

  frame31 = gtk_frame_new (NULL);
  gtk_widget_show (frame31);
  gtk_box_pack_start (GTK_BOX (hbox16), frame31, TRUE, TRUE, 2);

  vbox8 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox8);
  gtk_container_add (GTK_CONTAINER (frame31), vbox8);

  hbox17 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox17);
  gtk_box_pack_start (GTK_BOX (vbox8), hbox17, TRUE, TRUE, 0);

  label67 = gtk_label_new (_("1D grid: "));
  gtk_widget_show (label67);
  gtk_box_pack_start (GTK_BOX (hbox17), label67, FALSE, FALSE, 0);

  et1DGrid = gtk_entry_new ();
  gtk_widget_show (et1DGrid);
  gtk_box_pack_start (GTK_BOX (hbox17), et1DGrid, FALSE, FALSE, 2);
  //gtk_widget_set_size_request (et1DGrid, 20, -1);
  gtk_entry_set_max_length (GTK_ENTRY (et1DGrid), 3);
  gtk_entry_set_text (GTK_ENTRY (et1DGrid), _("3"));
  gtk_entry_set_width_chars (GTK_ENTRY (et1DGrid), 3);

  label68 = gtk_label_new (_("pixels "));
  gtk_widget_show (label68);
  gtk_box_pack_start (GTK_BOX (hbox17), label68, FALSE, FALSE, 0);

  hbox19 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox19);
  gtk_box_pack_start (GTK_BOX (vbox8), hbox19, TRUE, TRUE, 0);

  cbUpdate = gtk_check_button_new_with_mnemonic (_("update image"));
  gtk_widget_show (cbUpdate);
  gtk_box_pack_start (GTK_BOX (vbox8), cbUpdate, FALSE, FALSE, 0);

  label84 = gtk_label_new (_("other pref."));
  gtk_widget_show (label84);
  gtk_frame_set_label_widget (GTK_FRAME (frame31), label84);
  gtk_label_set_use_markup (GTK_LABEL (label84), TRUE);

  frame25 = gtk_frame_new (NULL);
  gtk_widget_show (frame25);
  gtk_box_pack_start (GTK_BOX (vbox_ext_tab), frame25, FALSE, TRUE, 0);
  //gtk_widget_set_size_request (frame25, -1, 40);

  hbox20 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox20);
  gtk_container_add (GTK_CONTAINER (frame25), hbox20);

  cbtnDezinger = gtk_check_button_new_with_mnemonic (_("activate"));
  gtk_widget_show (cbtnDezinger);
  gtk_box_pack_start (GTK_BOX (hbox20), cbtnDezinger, TRUE, TRUE, 0);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (cbtnDezinger),1);

  label70 = gtk_label_new (_("  radius: "));
  gtk_widget_show (label70);
  gtk_box_pack_start (GTK_BOX (hbox20), label70, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label70), GTK_JUSTIFY_RIGHT);

  etZingerRadius = gtk_entry_new ();
  gtk_widget_show (etZingerRadius);
  gtk_box_pack_start (GTK_BOX (hbox20), etZingerRadius, TRUE, FALSE, 0);
  //gtk_widget_set_size_request (etZingerRadius, 15, -1);
  gtk_entry_set_width_chars (GTK_ENTRY (etZingerRadius), 3);
  gtk_entry_set_text (GTK_ENTRY (etZingerRadius), _("5"));

  label71 = gtk_label_new (_("pixels,"));
  gtk_widget_show (label71);
  gtk_box_pack_start (GTK_BOX (hbox20), label71, FALSE, FALSE, 0);

  label72 = gtk_label_new (_("   tolerance: "));
  gtk_widget_show (label72);
  gtk_box_pack_start (GTK_BOX (hbox20), label72, FALSE, FALSE, 0);

  etZingerTolerance = gtk_entry_new ();
  gtk_widget_show (etZingerTolerance);
  gtk_box_pack_start (GTK_BOX (hbox20), etZingerTolerance, FALSE, TRUE, 3);
  //gtk_widget_set_size_request (etZingerTolerance, 25, -1);
  gtk_entry_set_width_chars (GTK_ENTRY (etZingerTolerance), 5);
  gtk_entry_set_text (GTK_ENTRY (etZingerTolerance), _("3.0"));

  label69 = gtk_label_new (_("dezinger"));
  gtk_widget_show (label69);
  gtk_frame_set_label_widget (GTK_FRAME (frame25), label69);
  gtk_label_set_use_markup (GTK_LABEL (label69), TRUE);

  frame34 = gtk_frame_new (NULL);
  gtk_widget_show (frame34);
  gtk_box_pack_start (GTK_BOX (vbox_ext_tab), frame34, TRUE, TRUE, 0);

  hbox27 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox27);
  gtk_container_add (GTK_CONTAINER (frame34), hbox27);

  label93 = gtk_label_new (_("threshold="));
  gtk_widget_show (label93);
  gtk_box_pack_start (GTK_BOX (hbox27), label93, FALSE, FALSE, 0);

  etPeakThresh = gtk_entry_new ();
  gtk_widget_show (etPeakThresh);
  gtk_box_pack_start (GTK_BOX (hbox27), etPeakThresh, TRUE, FALSE, 0);
  //gtk_widget_set_size_request (etPeakThresh, 50, -1);
  gtk_entry_set_width_chars (GTK_ENTRY (etPeakThresh), 5);
  gtk_entry_set_text (GTK_ENTRY (etPeakThresh), _("200"));

  tbShowPeak = gtk_button_new_with_mnemonic (_("search"));
  gtk_widget_show (tbShowPeak);
  gtk_box_pack_start (GTK_BOX (hbox27), tbShowPeak, TRUE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (tbShowPeak), 5);

  label92 = gtk_label_new (_("peak search"));
  gtk_widget_show (label92);
  gtk_frame_set_label_widget (GTK_FRAME (frame34), label92);
  gtk_label_set_use_markup (GTK_LABEL (label92), TRUE);

  label2 = gtk_label_new (_("extended functions"));
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label2);

  vbox_multi_tab = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox_multi_tab);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox_multi_tab);

  frame7 = gtk_frame_new (NULL);
  gtk_widget_show (frame7);
  gtk_box_pack_start (GTK_BOX (vbox_multi_tab), frame7, TRUE, TRUE, 0);
  //gtk_widget_set_size_request (frame7, -1, 110);

  scrolledwindow8 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow8);
  gtk_container_add (GTK_CONTAINER (frame7), scrolledwindow8);
  //gtk_widget_set_size_request (scrolledwindow8, 150, 60);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow8), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow8), GTK_SHADOW_IN);

  tvFileList = gtk_text_view_new ();
  gtk_widget_show (tvFileList);
  gtk_container_add (GTK_CONTAINER (scrolledwindow8), tvFileList);
  //gtk_widget_set_size_request (tvFileList, 276, 96);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (tvFileList), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (tvFileList), FALSE);
  gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (tvFileList)), _("click to load a list of files to be processed."), -1);

  label28 = gtk_label_new (_("file list"));
  gtk_widget_show (label28);
  gtk_frame_set_label_widget (GTK_FRAME (frame7), label28);

  hbox21 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox21);
  gtk_box_pack_start (GTK_BOX (vbox_multi_tab), hbox21, TRUE, TRUE, 0);
  //gtk_widget_set_size_request (hbox21, -1, 70);

  frame28 = gtk_frame_new (NULL);
  gtk_widget_show (frame28);
  gtk_box_pack_start (GTK_BOX (hbox21), frame28, TRUE, TRUE, 0);

  scrolledwindow7 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow7);
  gtk_container_add (GTK_CONTAINER (frame28), scrolledwindow7);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow7), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow7), GTK_SHADOW_IN);

  tvComList = gtk_text_view_new ();
  gtk_widget_show (tvComList);
  gtk_container_add (GTK_CONTAINER (scrolledwindow7), tvComList);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (tvComList), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (tvComList), FALSE);
  gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (tvComList)), _("click to load a list of\ncommands to be \nexecuted for each file."), -1);

  label81 = gtk_label_new (_("command list"));
  gtk_widget_show (label81);
  gtk_frame_set_label_widget (GTK_FRAME (frame28), label81);
  gtk_label_set_use_markup (GTK_LABEL (label81), TRUE);

  frame33 = gtk_frame_new (NULL);
  gtk_widget_show (frame33);
  gtk_box_pack_start (GTK_BOX (hbox21), frame33, TRUE, TRUE, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame33), GTK_SHADOW_NONE);

  vbox13 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox13);
  gtk_container_add (GTK_CONTAINER (frame33), vbox13);

  hbox22 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox22);
  gtk_box_pack_start (GTK_BOX (vbox13), hbox22, TRUE, TRUE, 0);

  btnFirst = gtk_button_new ();
  gtk_widget_show (btnFirst);
  gtk_box_pack_start (GTK_BOX (hbox22), btnFirst, TRUE, TRUE, 0);

  image6 = gtk_image_new_from_stock ("gtk-goto-first", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image6);
  gtk_container_add (GTK_CONTAINER (btnFirst), image6);

  btnBack = gtk_button_new ();
  gtk_widget_show (btnBack);
  gtk_box_pack_start (GTK_BOX (hbox22), btnBack, TRUE, TRUE, 0);

  image7 = gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image7);
  gtk_container_add (GTK_CONTAINER (btnBack), image7);

  btnForward = gtk_button_new ();
  gtk_widget_show (btnForward);
  gtk_box_pack_start (GTK_BOX (hbox22), btnForward, TRUE, TRUE, 0);

  image8 = gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image8);
  gtk_container_add (GTK_CONTAINER (btnForward), image8);

  btnLast = gtk_button_new ();
  gtk_widget_show (btnLast);
  gtk_box_pack_start (GTK_BOX (hbox22), btnLast, TRUE, TRUE, 0);

  image9 = gtk_image_new_from_stock ("gtk-goto-last", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image9);
  gtk_container_add (GTK_CONTAINER (btnLast), image9);

  btnGo = gtk_button_new_with_mnemonic (_("go"));
  gtk_widget_show (btnGo);
  gtk_box_pack_start (GTK_BOX (vbox13), btnGo, TRUE, FALSE, 0);

  label90 = gtk_label_new (_("slide show"));
  gtk_widget_show (label90);
  gtk_frame_set_label_widget (GTK_FRAME (frame33), label90);
  gtk_label_set_use_markup (GTK_LABEL (label90), TRUE);

  label3 = gtk_label_new (_("multiple files"));
  gtk_widget_show (label3);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), label3);

  vbox_manip_tab = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox_manip_tab);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox_manip_tab);
  gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook1), vbox_manip_tab,
                                      FALSE, FALSE, GTK_PACK_START);

  frame19 = gtk_frame_new (NULL);
  gtk_widget_show (frame19);
  gtk_box_pack_start (GTK_BOX (vbox_manip_tab), frame19, FALSE, FALSE, 2);
  //gtk_widget_set_size_request (frame19, 296, 45);

  hbox13a = gtk_hbox_new (FALSE, 1);
  gtk_widget_show (hbox13a);
  gtk_container_add (GTK_CONTAINER (frame19), hbox13a);

  btnStore = gtk_button_new_with_mnemonic (_("store"));
  gtk_widget_show (btnStore);
  gtk_box_pack_start (GTK_BOX (hbox13a), btnStore, FALSE, FALSE, 0);
  //gtk_widget_set_size_request (btnStore, 35, 23);

  btnRecall = gtk_button_new_with_mnemonic (_("recall"));
  gtk_widget_show (btnRecall);
  gtk_box_pack_start (GTK_BOX (hbox13a), btnRecall, FALSE, FALSE, 0);
  //gtk_widget_set_size_request (btnRecall, 35, 23);

  sbCurSet_adj = gtk_adjustment_new (0, 0, 5, 1, 10, 0);
  sbCurSet = gtk_spin_button_new (GTK_ADJUSTMENT (sbCurSet_adj), 1, 0);
  gtk_widget_show (sbCurSet);
  gtk_box_pack_start (GTK_BOX (hbox13a), sbCurSet, FALSE, FALSE, 0);
  //gtk_widget_set_size_request (sbCurSet, 25, 22);
  gtk_entry_set_width_chars (GTK_ENTRY(sbCurSet), 1); 
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (sbCurSet), TRUE);

  etSetComments = gtk_entry_new ();
  gtk_widget_show (etSetComments);
  gtk_box_pack_end (GTK_BOX (hbox13a), etSetComments, TRUE, TRUE, 2);
  //gtk_widget_set_size_request (etSetComments, 190, 20);
  gtk_entry_set_text (GTK_ENTRY (etSetComments), _("(empty)"));

  label45 = gtk_label_new (_("memory"));
  gtk_widget_show (label45);
  gtk_frame_set_label_widget (GTK_FRAME (frame19), label45);

  frame20 = gtk_frame_new (NULL);
  gtk_widget_show (frame20);
  gtk_box_pack_start (GTK_BOX (vbox_manip_tab), frame20, FALSE, FALSE, 2);
  //gtk_widget_set_size_request (frame20, -1, 45);
  gtk_container_set_border_width (GTK_CONTAINER (frame20), 2);

  hbox14 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox14);
  gtk_container_add (GTK_CONTAINER (frame20), hbox14);

  sbSet1_adj = gtk_adjustment_new (2, 0, 5, 1, 10, 0);
  sbSet1 = gtk_spin_button_new (GTK_ADJUSTMENT (sbSet1_adj), 1, 0);
  gtk_widget_show (sbSet1);
  gtk_box_pack_start (GTK_BOX (hbox14), sbSet1, FALSE, FALSE, 2);
  //gtk_widget_set_size_request (sbSet1, 26, 18);
  gtk_entry_set_width_chars (GTK_ENTRY (sbSet1), 1);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (sbSet1), TRUE);

  label48 = gtk_label_new (_("+ ("));
  gtk_widget_show (label48);
  gtk_box_pack_start (GTK_BOX (hbox14), label48, FALSE, FALSE, 3);

  sbSet2_adj = gtk_adjustment_new (1, 0, 5, 1, 10, 0);
  sbSet2 = gtk_spin_button_new (GTK_ADJUSTMENT (sbSet2_adj), 1, 0);
  gtk_widget_show (sbSet2);
  gtk_box_pack_start (GTK_BOX (hbox14), sbSet2, FALSE, FALSE, 2);
  //gtk_widget_set_size_request (sbSet2, 26, 18);
  gtk_entry_set_width_chars (GTK_ENTRY (sbSet2), 1);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (sbSet2), TRUE);

  label49 = gtk_label_new (_("-"));
  gtk_widget_show (label49);
  gtk_box_pack_start (GTK_BOX (hbox14), label49, FALSE, FALSE, 3);

  etConst = gtk_entry_new ();
  gtk_widget_show (etConst);
  gtk_box_pack_start (GTK_BOX (hbox14), etConst, FALSE, FALSE, 2);
  gtk_entry_set_width_chars (GTK_ENTRY (etConst), 5);
  //gtk_widget_set_size_request (etConst, 30, 21);

  label86 = gtk_label_new (_(") x"));
  gtk_widget_show (label86);
  gtk_box_pack_start (GTK_BOX (hbox14), label86, FALSE, FALSE, 3);

  etScaleFactor = gtk_entry_new ();
  gtk_widget_show (etScaleFactor);
  gtk_box_pack_start (GTK_BOX (hbox14), etScaleFactor, FALSE, FALSE, 0);
  gtk_entry_set_width_chars (GTK_ENTRY (etScaleFactor), 5);
  //gtk_widget_set_size_request (etScaleFactor, 30, 21);

  label87 = gtk_label_new (_(" "));
  gtk_widget_show (label87);
  gtk_box_pack_start (GTK_BOX (hbox14), label87, FALSE, FALSE, 0);

  btnCalculate = gtk_button_new_with_mnemonic (_("calculate"));
  gtk_widget_show (btnCalculate);
  gtk_box_pack_start (GTK_BOX (hbox14), btnCalculate, TRUE, FALSE, 0);
  //gtk_widget_set_size_request (btnCalculate, 55, 20);
  gtk_container_set_border_width (GTK_CONTAINER (btnCalculate), 2);

  btnSaveData = gtk_button_new_with_mnemonic (_("save"));
  gtk_widget_show (btnSaveData);
  gtk_box_pack_start (GTK_BOX (hbox14), btnSaveData, TRUE, FALSE, 0);
  //gtk_widget_set_size_request (btnSaveData, 55, 20);
  gtk_container_set_border_width (GTK_CONTAINER (btnSaveData), 2);

  label46 = gtk_label_new (_("evaluation"));
  gtk_widget_show (label46);
  gtk_frame_set_label_widget (GTK_FRAME (frame20), label46);

  frame_GISAXS = gtk_frame_new (NULL);
  gtk_widget_show (frame_GISAXS);
  gtk_box_pack_start (GTK_BOX (vbox_manip_tab), frame_GISAXS, FALSE, FALSE, 2);
  //gtk_widget_set_size_request (frame_GISAXS, 293, 45);

  hbox15 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox15);
  gtk_container_add (GTK_CONTAINER (frame_GISAXS), hbox15);

  cbDetRec = gtk_check_button_new_with_mnemonic (_("recp. spc."));
  gtk_widget_show (cbDetRec);
  gtk_box_pack_start (GTK_BOX (hbox15), cbDetRec, TRUE, FALSE, 0);

  vseparator1 = gtk_vseparator_new ();
  gtk_widget_show (vseparator1);
  gtk_box_pack_start (GTK_BOX (hbox15), vseparator1, TRUE, FALSE, 0);

  btMarkQrz = gtk_button_new_with_mnemonic (_("mark"));
  gtk_widget_show (btMarkQrz);
  gtk_box_pack_start (GTK_BOX (hbox15), btMarkQrz, TRUE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (btMarkQrz), 4);
  gtk_widget_set_sensitive (btMarkQrz, FALSE);

  label53 = gtk_label_new (_("  (qr = "));
  gtk_widget_show (label53);
  gtk_box_pack_start (GTK_BOX (hbox15), label53, TRUE, FALSE, 0);

  etQr = gtk_entry_new ();
  gtk_widget_show (etQr);
  gtk_box_pack_start (GTK_BOX (hbox15), etQr, TRUE, FALSE, 0);
  gtk_entry_set_width_chars (GTK_ENTRY(etQr),8);
  //gtk_widget_set_size_request (etQr, 50, -1);

  label54 = gtk_label_new (_(" , qz = "));
  gtk_widget_show (label54);
  gtk_box_pack_start (GTK_BOX (hbox15), label54, TRUE, FALSE, 0);

  etQz = gtk_entry_new ();
  gtk_widget_show (etQz);
  gtk_box_pack_start (GTK_BOX (hbox15), etQz, TRUE, FALSE, 0);
  gtk_entry_set_width_chars (GTK_ENTRY(etQz),8);
  //gtk_widget_set_size_request (etQz, 50, -1);

  label55 = gtk_label_new (_(" )  "));
  gtk_widget_show (label55);
  gtk_box_pack_start (GTK_BOX (hbox15), label55, TRUE, FALSE, 0);

  label_GISAXS = gtk_label_new (_("GISAXS"));
  gtk_widget_show (label_GISAXS);
  gtk_frame_set_label_widget (GTK_FRAME (frame_GISAXS), label_GISAXS);

  frame21 = gtk_frame_new (NULL);
  gtk_widget_show (frame21);
  gtk_box_pack_start (GTK_BOX (vbox_manip_tab), frame21, TRUE, TRUE, 2);
  //gtk_widget_set_size_request (frame21, -1, 56);

  table4 = gtk_table_new (2, 6, FALSE);
  gtk_widget_show (table4);
  gtk_container_add (GTK_CONTAINER (frame21), table4);
  //gtk_widget_set_size_request (table4, -1, 35);

  label57 = gtk_label_new (_(" center = ( "));
  gtk_widget_show (label57);
  gtk_table_attach (GTK_TABLE (table4), label57, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label57), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label57), 0, 0.5);

  etLineProfileCx = gtk_entry_new ();
  gtk_widget_show (etLineProfileCx);
  gtk_table_attach (GTK_TABLE (table4), etLineProfileCx, 1, 2, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_width_chars (GTK_ENTRY (etLineProfileCx), 5);
  //gtk_widget_set_size_request (etLineProfileCx, 40, 21);

  label59 = gtk_label_new (_(" )  length = "));
  gtk_widget_show (label59);
  gtk_table_attach (GTK_TABLE (table4), label59, 4, 5, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label59), 0, 0.5);

  etLineProfileLength = gtk_entry_new ();
  gtk_widget_show (etLineProfileLength);
  gtk_table_attach (GTK_TABLE (table4), etLineProfileLength, 5, 6, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_width_chars (GTK_ENTRY (etLineProfileLength), 5);
  //gtk_widget_set_size_request (etLineProfileLength, 50, 21);

  label60 = gtk_label_new (_(" orientation = "));
  gtk_widget_show (label60);
  gtk_table_attach (GTK_TABLE (table4), label60, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label60), 0, 0.5);

  etLineProfileOrient = gtk_entry_new ();
  gtk_widget_show (etLineProfileOrient);
  gtk_table_attach (GTK_TABLE (table4), etLineProfileOrient, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_width_chars (GTK_ENTRY (etLineProfileOrient), 5);
  //gtk_widget_set_size_request (etLineProfileOrient, 30, 21);

  label61 = gtk_label_new (_("avg. rad. ="));
  gtk_widget_show (label61);
  gtk_table_attach (GTK_TABLE (table4), label61, 3, 4, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label61), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label61), 0, 0.5);

  etLineProfileRadius = gtk_entry_new ();
  gtk_widget_show (etLineProfileRadius);
  gtk_table_attach (GTK_TABLE (table4), etLineProfileRadius, 4, 5, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_width_chars (GTK_ENTRY (etLineProfileRadius), 5);
  //gtk_widget_set_size_request (etLineProfileRadius, 20, 21);

  btnExtract = gtk_button_new_with_mnemonic (_("extract"));
  gtk_widget_show (btnExtract);
  gtk_table_attach (GTK_TABLE (table4), btnExtract, 5, 6, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 3);

  etLineProfileCy = gtk_entry_new ();
  gtk_widget_show (etLineProfileCy);
  gtk_table_attach (GTK_TABLE (table4), etLineProfileCy, 3, 4, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_width_chars (GTK_ENTRY (etLineProfileCy), 5);  
  //gtk_widget_set_size_request (etLineProfileCy, 40, 21);

  label58 = gtk_label_new (_(" , "));
  gtk_widget_show (label58);
  gtk_table_attach (GTK_TABLE (table4), label58, 2, 3, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label58), 0, 0.5);

  label88 = gtk_label_new (_(" "));
  gtk_widget_show (label88);
  gtk_table_attach (GTK_TABLE (table4), label88, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label_ex = gtk_label_new (_("line profile"));
  gtk_widget_show (label_ex);
  gtk_frame_set_label_widget (GTK_FRAME (frame21), label_ex);

  label44 = gtk_label_new (_("manipulation"));
  gtk_widget_show (label44);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 3), label44);

  scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow6);
  gtk_box_pack_start (GTK_BOX (vbox_control_window), scrolledwindow6, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow6), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  tvInfo = gtk_text_view_new ();
  gtk_widget_show (tvInfo);
  gtk_container_add (GTK_CONTAINER (scrolledwindow6), tvInfo);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (tvInfo), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (tvInfo), FALSE);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox_control_window), hseparator1, FALSE, FALSE, 0);

  lbStatusMsg = gtk_label_new ("");
  gtk_widget_show (lbStatusMsg);
  gtk_box_pack_end (GTK_BOX (vbox_control_window), lbStatusMsg, FALSE, FALSE, 2);
  //gtk_widget_set_size_request (lbStatusMsg, 46, 17);
  gtk_misc_set_alignment (GTK_MISC (lbStatusMsg), 0, 0);

  g_signal_connect ((gpointer) wControl, "delete_event",
                    G_CALLBACK (gtk_true),
                    NULL);
  g_signal_connect ((gpointer) open1, "activate",
                    G_CALLBACK (on_open1_activate),
                    NULL);
  g_signal_connect ((gpointer) save_as_jpg1, "activate",
                    G_CALLBACK (on_save_as_jpg1_activate),
                    NULL);
  g_signal_connect ((gpointer) fontSel, "activate",
                    G_CALLBACK (on_change_font_activate),
                    NULL);
  g_signal_connect ((gpointer) quit1, "activate",
                    G_CALLBACK (on_quit1_activate),
                    NULL);
  g_signal_connect ((gpointer) format_mar, "activate",
                    G_CALLBACK (on_format_changed),
                    NULL);
  g_signal_connect ((gpointer) format_waxs, "activate",
                    G_CALLBACK (on_format_changed),
                    NULL);
  g_signal_connect ((gpointer) format_pilatus, "activate",
                    G_CALLBACK (on_format_changed),
                    NULL);
  g_signal_connect ((gpointer) format_binary_short, "activate",
                    G_CALLBACK (on_format_changed),
                    NULL);
  g_signal_connect ((gpointer) format_bruker, "activate",
                    G_CALLBACK (on_format_changed),
                    NULL);
  g_signal_connect ((gpointer) format_PI, "activate",
                    G_CALLBACK (on_format_changed),
                    NULL);
  g_signal_connect ((gpointer) about1, "activate",
                    G_CALLBACK (on_about1_activate),
                    NULL);
  g_signal_connect ((gpointer) ckbtnGeo, "toggled",
                    G_CALLBACK (change_sample_geometry),
                    NULL);
  g_signal_connect ((gpointer) btConfDetGeo, "clicked",
                    G_CALLBACK (config_DetGeo),
                    NULL);
  g_signal_connect ((gpointer) btnSavePara, "clicked",
                    G_CALLBACK (save_Parameter_set),
                    NULL);
  g_signal_connect ((gpointer) btnLoadPara, "clicked",
                    G_CALLBACK (load_Parameter_set),
                    NULL);
  g_signal_connect ((gpointer) tbAutoZScale, "toggled",
                    G_CALLBACK (on_autoscale_toggled),
                    NULL);
  g_signal_connect ((gpointer) btColor0, "clicked",
                    G_CALLBACK (change_color),
                    NULL);
  g_signal_connect ((gpointer) btColor1, "clicked",
                    G_CALLBACK (change_color),
                    NULL);
  g_signal_connect ((gpointer) btColor2, "clicked",
                    G_CALLBACK (change_color),
                    NULL);
  g_signal_connect ((gpointer) btColor3, "clicked",
                    G_CALLBACK (change_color),
                    NULL);
  g_signal_connect ((gpointer) btColor4, "clicked",
                    G_CALLBACK (change_color),
                    NULL);
  g_signal_connect ((gpointer) daColorTable, "expose_event",
                    G_CALLBACK (draw_color_table),
                    NULL);
  g_signal_connect ((gpointer) cbtnLog, "toggled",
                    G_CALLBACK (on_log_toggled),
                    NULL);
  g_signal_connect ((gpointer) cbxZoom, "changed",
                    G_CALLBACK (on_Zoom_changed),
                    NULL);
  g_signal_connect ((gpointer) cbxRecent, "changed",
                    G_CALLBACK (load_recent_image),
                    NULL);
  g_signal_connect ((gpointer) cbxRef, "changed",
                    G_CALLBACK (on_RefPattern_changed),
                    NULL);
  g_signal_connect ((gpointer) btShowStat, "clicked",
                    G_CALLBACK (show_image_statistics),
                    NULL);
  g_signal_connect ((gpointer) cbxFlatField, "changed",
                    G_CALLBACK (on_ffield_file_changed),
                    NULL);
  g_signal_connect ((gpointer) cbxMask, "changed",
                    G_CALLBACK (on_mask_file_changed),
                    NULL);
  g_signal_connect ((gpointer) cbUpdate, "toggled",
                    G_CALLBACK (on_cbUpdate_toggled),
                    NULL);
  g_signal_connect ((gpointer) cbtnDezinger, "toggled",
                    G_CALLBACK (on_dezinger_clicked),
                    NULL);
  g_signal_connect ((gpointer) tbShowPeak, "clicked",
                    G_CALLBACK (on_showpeak_clicked),
                    NULL);
  g_signal_connect ((gpointer) tvFileList, "button_press_event",
                    G_CALLBACK (load_file_list),
                    NULL);
  g_signal_connect ((gpointer) tvComList, "button_press_event",
                    G_CALLBACK (load_command_list),
                    NULL);
  g_signal_connect ((gpointer) btnGo, "clicked",
                    G_CALLBACK (start_processing),
                    NULL);
  g_signal_connect ((gpointer) btnStore, "clicked",
                    G_CALLBACK (store_current_set),
                    NULL);
  g_signal_connect ((gpointer) btnRecall, "clicked",
                    G_CALLBACK (recall_stored_set),
                    NULL);
  g_signal_connect ((gpointer) sbCurSet, "changed",
                    G_CALLBACK (load_set_comments),
                    NULL);
  g_signal_connect ((gpointer) etSetComments, "activate",
                    G_CALLBACK (update_set_comments),
                    NULL);
  g_signal_connect ((gpointer) btnCalculate, "clicked",
                    G_CALLBACK (evaluate_expression),
                    NULL);
  g_signal_connect ((gpointer) btnSaveData, "clicked",
                    G_CALLBACK (save_current_set),
                    NULL);
  g_signal_connect ((gpointer) cbDetRec, "toggled",
                    G_CALLBACK (toggle_DetRec),
                    NULL);
  g_signal_connect ((gpointer) btMarkQrz, "clicked",
                    G_CALLBACK (MarkQrz),
                    NULL);
  g_signal_connect ((gpointer) btnExtract, "clicked",
                    G_CALLBACK (extract_line_profile),
                    NULL);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (wControl, wControl, "wControl");
  GLADE_HOOKUP_OBJECT (wControl, vbox_control_window, "vbox_control_window");
  GLADE_HOOKUP_OBJECT (wControl, menubar1, "menubar1");
  GLADE_HOOKUP_OBJECT (wControl, menuitem1, "menuitem1");
  GLADE_HOOKUP_OBJECT (wControl, menuitem1_menu, "menuitem1_menu");
  GLADE_HOOKUP_OBJECT (wControl, open1, "open1");
  GLADE_HOOKUP_OBJECT (wControl, save_as_jpg1, "save_as_jpg1");
  GLADE_HOOKUP_OBJECT (wControl, separatormenuitem1, "separatormenuitem1");
  GLADE_HOOKUP_OBJECT (wControl, quit1, "quit1");
  GLADE_HOOKUP_OBJECT (wControl, menuitem3, "menuitem3");
  GLADE_HOOKUP_OBJECT (wControl, menuitem3_menu, "menuitem3_menu");
  GLADE_HOOKUP_OBJECT (wControl, format_mar, "format_mar");
  GLADE_HOOKUP_OBJECT (wControl, format_waxs, "format_waxs");
  GLADE_HOOKUP_OBJECT (wControl, format_pilatus, "format_pilatus");
  GLADE_HOOKUP_OBJECT (wControl, format_binary_short, "format_binary_short");
  GLADE_HOOKUP_OBJECT (wControl, format_bruker, "format_bruker");
  GLADE_HOOKUP_OBJECT (wControl, format_PI, "format_PI");
  GLADE_HOOKUP_OBJECT (wControl, menuitem4, "menuitem4");
  GLADE_HOOKUP_OBJECT (wControl, menuitem4_menu, "menuitem4_menu");
  GLADE_HOOKUP_OBJECT (wControl, about1, "about1");
  GLADE_HOOKUP_OBJECT (wControl, notebook1, "notebook1");
  GLADE_HOOKUP_OBJECT (wControl, vbox14, "vbox14");
  GLADE_HOOKUP_OBJECT (wControl, hbox25, "hbox25");
  GLADE_HOOKUP_OBJECT (wControl, frame_experimental, "frame_experimental");
  GLADE_HOOKUP_OBJECT (wControl, vbox12, "vbox12");
  GLADE_HOOKUP_OBJECT (wControl, table3, "table3");
  GLADE_HOOKUP_OBJECT (wControl, label15, "label15");
  GLADE_HOOKUP_OBJECT (wControl, label16, "label16");
  GLADE_HOOKUP_OBJECT (wControl, label17, "label17");
  GLADE_HOOKUP_OBJECT (wControl, label18, "label18");
  GLADE_HOOKUP_OBJECT (wControl, label19, "label19");
  GLADE_HOOKUP_OBJECT (wControl, etCenY, "etCenY");
  GLADE_HOOKUP_OBJECT (wControl, etCenX, "etCenX");
  //GLADE_HOOKUP_OBJECT (wControl, etDetSwing, "etDetSwing");
  GLADE_HOOKUP_OBJECT (wControl, etDd, "etDd");
  GLADE_HOOKUP_OBJECT (wControl, btConfDetGeo, "btConfDetGeo");
  GLADE_HOOKUP_OBJECT (wControl, etWavelength, "etWavelength");
  GLADE_HOOKUP_OBJECT (wControl, ckbtnGeo, "ckbtnGeo");
  GLADE_HOOKUP_OBJECT (wControl, hbox23, "hbox23");
  GLADE_HOOKUP_OBJECT (wControl, label76, "label76");
  GLADE_HOOKUP_OBJECT (wControl, etSamNormal, "etSamNormal");
  GLADE_HOOKUP_OBJECT (wControl, label77, "label77");
  GLADE_HOOKUP_OBJECT (wControl, etIncidentAng, "etIncidentAng");
  GLADE_HOOKUP_OBJECT (wControl, hbox24, "hbox24");
  GLADE_HOOKUP_OBJECT (wControl, label91, "label91");
  GLADE_HOOKUP_OBJECT (wControl, sbParaSet, "sbParaSet");
  GLADE_HOOKUP_OBJECT (wControl, btnSavePara, "btnSavePara");
  GLADE_HOOKUP_OBJECT (wControl, btnLoadPara, "btnLoadPara");
  GLADE_HOOKUP_OBJECT (wControl, label13, "label13");
  GLADE_HOOKUP_OBJECT (wControl, frame_color, "frame_color");
  GLADE_HOOKUP_OBJECT (wControl, vbox_color, "vbox_color");
  GLADE_HOOKUP_OBJECT (wControl, hbox4, "hbox4");
  GLADE_HOOKUP_OBJECT (wControl, label24, "label24");
  GLADE_HOOKUP_OBJECT (wControl, etZmin, "etZmin");
  GLADE_HOOKUP_OBJECT (wControl, label25, "label25");
  GLADE_HOOKUP_OBJECT (wControl, etZmax, "etZmax");
  GLADE_HOOKUP_OBJECT (wControl, tbAutoZScale, "tbAutoZScale");
  GLADE_HOOKUP_OBJECT (wControl, hbox5, "hbox5");
  GLADE_HOOKUP_OBJECT (wControl, btColor0, "btColor0");
  GLADE_HOOKUP_OBJECT (wControl, btColor1, "btColor1");
  GLADE_HOOKUP_OBJECT (wControl, btColor2, "btColor2");
  GLADE_HOOKUP_OBJECT (wControl, btColor3, "btColor3");
  GLADE_HOOKUP_OBJECT (wControl, btColor4, "btColor4");
  GLADE_HOOKUP_OBJECT (wControl, daColorTable, "daColorTable");
  GLADE_HOOKUP_OBJECT (wControl, table2, "table2");
  GLADE_HOOKUP_OBJECT (wControl, label11, "label11");
  GLADE_HOOKUP_OBJECT (wControl, label12, "label12");
  GLADE_HOOKUP_OBJECT (wControl, sclContrast, "sclContrast");
  GLADE_HOOKUP_OBJECT (wControl, sclBrightness, "sclBrightness");
  GLADE_HOOKUP_OBJECT (wControl, hbox3, "hbox3");
  GLADE_HOOKUP_OBJECT (wControl, cbtnLog, "cbtnLog");
  GLADE_HOOKUP_OBJECT (wControl, label22, "label22");
  GLADE_HOOKUP_OBJECT (wControl, etGamma, "etGamma");
  GLADE_HOOKUP_OBJECT (wControl, label5, "label5");
  GLADE_HOOKUP_OBJECT (wControl, hbox26, "hbox26");
  GLADE_HOOKUP_OBJECT (wControl, frame_zoom, "frame_zoom");
  GLADE_HOOKUP_OBJECT (wControl, cbxZoom, "cbxZoom");
  GLADE_HOOKUP_OBJECT (wControl, label14, "label14");
  GLADE_HOOKUP_OBJECT (wControl, frame32, "frame32");
  //GLADE_HOOKUP_OBJECT (wControl, fixed2, "fixed2");
  GLADE_HOOKUP_OBJECT (wControl, cbxRecent, "cbxRecent");
  GLADE_HOOKUP_OBJECT (wControl, label89, "label89");
  GLADE_HOOKUP_OBJECT (wControl, label1, "label1");
  GLADE_HOOKUP_OBJECT (wControl, vbox_ext_tab, "vbox_ext_tab");
  GLADE_HOOKUP_OBJECT (wControl, hbox9, "hbox9");
  GLADE_HOOKUP_OBJECT (wControl, frame14, "frame14");
  GLADE_HOOKUP_OBJECT (wControl, hbox18, "hbox18");
  GLADE_HOOKUP_OBJECT (wControl, cbxRef, "cbxRef");
  GLADE_HOOKUP_OBJECT (wControl, label35, "label35");
  GLADE_HOOKUP_OBJECT (wControl, frame15, "frame15");
  GLADE_HOOKUP_OBJECT (wControl, btShowStat, "btShowStat");
  GLADE_HOOKUP_OBJECT (wControl, label36, "label36");
  GLADE_HOOKUP_OBJECT (wControl, hbox16, "hbox16");
  GLADE_HOOKUP_OBJECT (wControl, frame30, "frame30");
  GLADE_HOOKUP_OBJECT (wControl, fixed1, "fixed1");
  GLADE_HOOKUP_OBJECT (wControl, table6, "table6");
  GLADE_HOOKUP_OBJECT (wControl, cbxFlatField, "cbxFlatField");
  GLADE_HOOKUP_OBJECT (wControl, label80, "label80");
  GLADE_HOOKUP_OBJECT (wControl, label79, "label79");
  GLADE_HOOKUP_OBJECT (wControl, label78, "label78");
  GLADE_HOOKUP_OBJECT (wControl, cbxDistortion, "cbxDistortion");
  GLADE_HOOKUP_OBJECT (wControl, cbxMask, "cbxMask");
  GLADE_HOOKUP_OBJECT (wControl, label83, "label83");
  GLADE_HOOKUP_OBJECT (wControl, frame31, "frame31");
  GLADE_HOOKUP_OBJECT (wControl, vbox8, "vbox8");
  GLADE_HOOKUP_OBJECT (wControl, hbox17, "hbox17");
  GLADE_HOOKUP_OBJECT (wControl, label67, "label67");
  GLADE_HOOKUP_OBJECT (wControl, et1DGrid, "et1DGrid");
  GLADE_HOOKUP_OBJECT (wControl, label68, "label68");
  GLADE_HOOKUP_OBJECT (wControl, hbox19, "hbox19");
  GLADE_HOOKUP_OBJECT (wControl, label84, "label84");
  GLADE_HOOKUP_OBJECT (wControl, cbUpdate, "cbUpdate");
  GLADE_HOOKUP_OBJECT (wControl, frame25, "frame25");
  GLADE_HOOKUP_OBJECT (wControl, hbox20, "hbox20");
  GLADE_HOOKUP_OBJECT (wControl, cbtnDezinger, "cbtnDezinger");
  GLADE_HOOKUP_OBJECT (wControl, label70, "label70");
  GLADE_HOOKUP_OBJECT (wControl, etZingerRadius, "etZingerRadius");
  GLADE_HOOKUP_OBJECT (wControl, label71, "label71");
  GLADE_HOOKUP_OBJECT (wControl, label72, "label72");
  GLADE_HOOKUP_OBJECT (wControl, etZingerTolerance, "etZingerTolerance");
  GLADE_HOOKUP_OBJECT (wControl, label69, "label69");
  GLADE_HOOKUP_OBJECT (wControl, label2, "label2");
  GLADE_HOOKUP_OBJECT (wControl, vbox_multi_tab, "vbox_multi_tab");
  GLADE_HOOKUP_OBJECT (wControl, frame7, "frame7");
  GLADE_HOOKUP_OBJECT (wControl, scrolledwindow8, "scrolledwindow8");
  GLADE_HOOKUP_OBJECT (wControl, tvFileList, "tvFileList");
  GLADE_HOOKUP_OBJECT (wControl, label28, "label28");
  GLADE_HOOKUP_OBJECT (wControl, hbox21, "hbox21");
  GLADE_HOOKUP_OBJECT (wControl, frame28, "frame28");
  GLADE_HOOKUP_OBJECT (wControl, scrolledwindow7, "scrolledwindow7");
  GLADE_HOOKUP_OBJECT (wControl, tvComList, "tvComList");
  GLADE_HOOKUP_OBJECT (wControl, label81, "label81");
  GLADE_HOOKUP_OBJECT (wControl, frame33, "frame33");
  GLADE_HOOKUP_OBJECT (wControl, vbox13, "vbox13");
  GLADE_HOOKUP_OBJECT (wControl, hbox22, "hbox22");
  GLADE_HOOKUP_OBJECT (wControl, btnFirst, "btnFirst");
  GLADE_HOOKUP_OBJECT (wControl, image6, "image6");
  GLADE_HOOKUP_OBJECT (wControl, btnBack, "btnBack");
  GLADE_HOOKUP_OBJECT (wControl, image7, "image7");
  GLADE_HOOKUP_OBJECT (wControl, btnForward, "btnForward");
  GLADE_HOOKUP_OBJECT (wControl, image8, "image8");
  GLADE_HOOKUP_OBJECT (wControl, btnLast, "btnLast");
  GLADE_HOOKUP_OBJECT (wControl, image9, "image9");
  GLADE_HOOKUP_OBJECT (wControl, btnGo, "btnGo");
  GLADE_HOOKUP_OBJECT (wControl, label90, "label90");
  GLADE_HOOKUP_OBJECT (wControl, label3, "label3");
  GLADE_HOOKUP_OBJECT (wControl, vbox_manip_tab, "vbox_manip_tab");
  GLADE_HOOKUP_OBJECT (wControl, frame19, "frame19");
  GLADE_HOOKUP_OBJECT (wControl, hbox13a, "hbox13a");
  GLADE_HOOKUP_OBJECT (wControl, btnStore, "btnStore");
  GLADE_HOOKUP_OBJECT (wControl, btnRecall, "btnRecall");
  GLADE_HOOKUP_OBJECT (wControl, sbCurSet, "sbCurSet");
  GLADE_HOOKUP_OBJECT (wControl, etSetComments, "etSetComments");
  GLADE_HOOKUP_OBJECT (wControl, label45, "label45");
  GLADE_HOOKUP_OBJECT (wControl, frame20, "frame20");
  GLADE_HOOKUP_OBJECT (wControl, hbox14, "hbox14");
  GLADE_HOOKUP_OBJECT (wControl, sbSet1, "sbSet1");
  GLADE_HOOKUP_OBJECT (wControl, label48, "label48");
  GLADE_HOOKUP_OBJECT (wControl, sbSet2, "sbSet2");
  GLADE_HOOKUP_OBJECT (wControl, label49, "label49");
  GLADE_HOOKUP_OBJECT (wControl, etConst, "etConst");
  GLADE_HOOKUP_OBJECT (wControl, label86, "label86");
  GLADE_HOOKUP_OBJECT (wControl, etScaleFactor, "etScaleFactor");
  GLADE_HOOKUP_OBJECT (wControl, label87, "label87");
  GLADE_HOOKUP_OBJECT (wControl, btnCalculate, "btnCalculate");
  GLADE_HOOKUP_OBJECT (wControl, btnSaveData, "btnSaveData");
  GLADE_HOOKUP_OBJECT (wControl, label46, "label46");
  GLADE_HOOKUP_OBJECT (wControl, frame_GISAXS, "frame_GISAXS");
  GLADE_HOOKUP_OBJECT (wControl, hbox15, "hbox15");
  GLADE_HOOKUP_OBJECT (wControl, cbDetRec, "cbDetRec");
  GLADE_HOOKUP_OBJECT (wControl, vseparator1, "vseparator1");
  GLADE_HOOKUP_OBJECT (wControl, btMarkQrz, "btMarkQrz");
  GLADE_HOOKUP_OBJECT (wControl, label53, "label53");
  GLADE_HOOKUP_OBJECT (wControl, etQr, "etQr");
  GLADE_HOOKUP_OBJECT (wControl, label54, "label54");
  GLADE_HOOKUP_OBJECT (wControl, etQz, "etQz");
  GLADE_HOOKUP_OBJECT (wControl, label55, "label55");
  GLADE_HOOKUP_OBJECT (wControl, label_GISAXS, "label_GISAXS");
  GLADE_HOOKUP_OBJECT (wControl, frame21, "frame21");
  GLADE_HOOKUP_OBJECT (wControl, table4, "table4");
  GLADE_HOOKUP_OBJECT (wControl, label57, "label57");
  GLADE_HOOKUP_OBJECT (wControl, etLineProfileCx, "etLineProfileCx");
  GLADE_HOOKUP_OBJECT (wControl, label59, "label59");
  GLADE_HOOKUP_OBJECT (wControl, etLineProfileLength, "etLineProfileLength");
  GLADE_HOOKUP_OBJECT (wControl, label60, "label60");
  GLADE_HOOKUP_OBJECT (wControl, etLineProfileOrient, "etLineProfileOrient");
  GLADE_HOOKUP_OBJECT (wControl, label61, "label61");
  GLADE_HOOKUP_OBJECT (wControl, etLineProfileRadius, "etLineProfileRadius");
  GLADE_HOOKUP_OBJECT (wControl, btnExtract, "btnExtract");
  GLADE_HOOKUP_OBJECT (wControl, etLineProfileCy, "etLineProfileCy");
  GLADE_HOOKUP_OBJECT (wControl, label58, "label58");
  GLADE_HOOKUP_OBJECT (wControl, label88, "label88");
  GLADE_HOOKUP_OBJECT (wControl, label_ex, "label_ex");
  GLADE_HOOKUP_OBJECT (wControl, label44, "label44");
  GLADE_HOOKUP_OBJECT (wControl, scrolledwindow6, "scrolledwindow6");
  GLADE_HOOKUP_OBJECT (wControl, tvInfo, "tvInfo");
  GLADE_HOOKUP_OBJECT (wControl, hseparator1, "hseparator1");
  GLADE_HOOKUP_OBJECT (wControl, lbStatusMsg, "lbStatusMsg");
  GLADE_HOOKUP_OBJECT (wControl, etPeakThresh, "etPeakThresh");
  GLADE_HOOKUP_OBJECT (wControl, tbShowPeak, "tbShowPeak");

  gtk_window_add_accel_group (GTK_WINDOW (wControl), accel_group);

  // these lines must be manually added to the code automatically 
  // generated by glade-2
  g_signal_connect ((gpointer) etZmin, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (0) );
  g_signal_connect ((gpointer) etZmax, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (1) );
  g_signal_connect ((gpointer) etGamma, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (2) );
  g_signal_connect ((gpointer) etWavelength, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (3) );
  g_signal_connect ((gpointer) etDd, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (4) );
  /*
  g_signal_connect ((gpointer) etDetSwing, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (5) );
  */
  g_signal_connect ((gpointer) etCenX, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (6) );
  g_signal_connect ((gpointer) etCenY, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (7) );
  g_signal_connect ((gpointer) etSamNormal, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (8) );
  g_signal_connect ((gpointer) etIncidentAng, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (9) );
  g_signal_connect ((gpointer) etZingerRadius, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (10) );
  g_signal_connect ((gpointer) etZingerTolerance, "activate",
                    G_CALLBACK (on_editing_done),
                    GINT_TO_POINTER (11) );

  g_signal_connect ((gpointer) btnFirst, "clicked",
                    G_CALLBACK (slideshow_event),
                    GINT_TO_POINTER (1) );
  g_signal_connect ((gpointer) btnBack, "clicked",
                    G_CALLBACK (slideshow_event),
                    GINT_TO_POINTER (2) );
  g_signal_connect ((gpointer) btnForward, "clicked",
                    G_CALLBACK (slideshow_event),
                    GINT_TO_POINTER (3) );
  g_signal_connect ((gpointer) btnLast, "clicked",
                    G_CALLBACK (slideshow_event),
                    GINT_TO_POINTER (4) );

  // remove corresponding lines from automaticlly generated code
  g_signal_connect ((gpointer) format_mar, "activate",
                    G_CALLBACK (on_format_changed),
                    GINT_TO_POINTER (0));
  g_signal_connect ((gpointer) format_waxs, "activate",
                    G_CALLBACK (on_format_changed),
                    GINT_TO_POINTER (1));
  g_signal_connect ((gpointer) format_pilatus, "activate",
                    G_CALLBACK (on_format_changed),
                    GINT_TO_POINTER (2));
  g_signal_connect ((gpointer) format_binary_short, "activate",
                    G_CALLBACK (on_format_changed),
                    GINT_TO_POINTER (3));
  g_signal_connect ((gpointer) format_bruker, "activate",
                    G_CALLBACK (on_format_changed),
                    GINT_TO_POINTER (4));
  g_signal_connect ((gpointer) format_PI, "activate",
                    G_CALLBACK (on_format_changed),
                    GINT_TO_POINTER (5));

  return wControl;
}


GtkWidget*
create_dlgDetGeo (void)
{
  GtkWidget *dlgDetGeo;
  GtkWidget *dialog_vbox1;
  GtkWidget *table7;
  GtkWidget *label92;
  GtkWidget *label93;
  GtkWidget *label94;
  GtkWidget *label95;
  GtkWidget *label96;
  GtkWidget *etDetOrient;
  GtkWidget *etDetPhi;
  GtkWidget *etDetTilt;
  GtkWidget *etBeamTiltX,*etBeamTiltY;
  GtkWidget *dialog_action_area1;
  GtkWidget *okbutton1;

  dlgDetGeo = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dlgDetGeo), _("Detector Geometry"));
  gtk_window_set_resizable (GTK_WINDOW (dlgDetGeo), FALSE);
  gtk_window_set_decorated (GTK_WINDOW (dlgDetGeo), FALSE);
  gtk_window_set_type_hint (GTK_WINDOW (dlgDetGeo), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (dlgDetGeo)->vbox;
  gtk_widget_show (dialog_vbox1);

  table7 = gtk_table_new (5, 2, FALSE);
  gtk_widget_show (table7);
  gtk_box_pack_end (GTK_BOX (dialog_vbox1), table7, FALSE, FALSE, 2);
  gtk_table_set_row_spacings (GTK_TABLE (table7), 6);
  gtk_table_set_col_spacings (GTK_TABLE (table7), 3);

  label92 = gtk_label_new (_("CCD orientation (deg) ="));
  gtk_widget_show (label92);
  gtk_table_attach (GTK_TABLE (table7), label92, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label92), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label92), 0, 0.5);
  gtk_label_set_single_line_mode (GTK_LABEL (label92), TRUE);

  label93 = gtk_label_new (_("detector tilt (deg) ="));
  gtk_widget_show (label93);
  gtk_table_attach (GTK_TABLE (table7), label93, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label93), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label93), 0, 0.5);
  gtk_label_set_single_line_mode (GTK_LABEL (label93), TRUE);

  label94 = gtk_label_new (_("detector roll (deg) ="));
  gtk_widget_show (label94);
  gtk_table_attach (GTK_TABLE (table7), label94, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label94), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label94), 0, 0.5);
  gtk_label_set_single_line_mode (GTK_LABEL (label94), TRUE);

  label95 = gtk_label_new (_("beam tilt X (deg) ="));
  gtk_widget_show (label95);
  gtk_table_attach (GTK_TABLE (table7), label95, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label95), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label95), 0, 0.5);
  gtk_label_set_single_line_mode (GTK_LABEL (label95), TRUE);

  label96 = gtk_label_new (_("beam tilt Y (deg) ="));
  gtk_widget_show (label96);
  gtk_table_attach (GTK_TABLE (table7), label96, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label96), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label96), 0, 0.5);
  gtk_label_set_single_line_mode (GTK_LABEL (label96), TRUE);

  etDetOrient = gtk_entry_new ();
  gtk_widget_show (etDetOrient);
  gtk_table_attach (GTK_TABLE (table7), etDetOrient, 1, 2, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_text (GTK_ENTRY (etDetOrient), _("0"));
  gtk_entry_set_width_chars (GTK_ENTRY (etDetOrient), 10);

  etDetPhi = gtk_entry_new ();
  gtk_widget_show (etDetPhi);
  gtk_table_attach (GTK_TABLE (table7), etDetPhi, 1, 2, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_text (GTK_ENTRY (etDetPhi), _("0"));
  gtk_entry_set_width_chars (GTK_ENTRY (etDetPhi), 10);

  etDetTilt = gtk_entry_new ();
  gtk_widget_show (etDetTilt);
  gtk_table_attach (GTK_TABLE (table7), etDetTilt, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_text (GTK_ENTRY (etDetTilt), _("0"));
  gtk_entry_set_width_chars (GTK_ENTRY (etDetTilt), 10);

  /**/
  etBeamTiltX = gtk_entry_new ();
  gtk_widget_show (etBeamTiltX);
  gtk_table_attach (GTK_TABLE (table7), etBeamTiltX, 1, 2, 3, 4,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_text (GTK_ENTRY (etBeamTiltX), _("0"));
  gtk_entry_set_width_chars (GTK_ENTRY (etBeamTiltX), 10);

  etBeamTiltY = gtk_entry_new ();
  gtk_widget_show (etBeamTiltY);
  gtk_table_attach (GTK_TABLE (table7), etBeamTiltY, 1, 2, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_text (GTK_ENTRY (etBeamTiltY), _("0"));
  gtk_entry_set_width_chars (GTK_ENTRY (etBeamTiltY), 10);
  /**/

  dialog_action_area1 = GTK_DIALOG (dlgDetGeo)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dlgDetGeo), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) okbutton1, "clicked",
                    G_CALLBACK (DetGeo_config_done),
                    NULL);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (dlgDetGeo, dlgDetGeo, "dlgDetGeo");
  GLADE_HOOKUP_OBJECT_NO_REF (dlgDetGeo, dialog_vbox1, "dialog_vbox1");
  GLADE_HOOKUP_OBJECT (dlgDetGeo, table7, "table7");
  GLADE_HOOKUP_OBJECT (dlgDetGeo, label92, "label92");
  GLADE_HOOKUP_OBJECT (dlgDetGeo, label93, "label93");
  GLADE_HOOKUP_OBJECT (dlgDetGeo, label94, "label94");
  GLADE_HOOKUP_OBJECT (dlgDetGeo, etDetOrient, "etDetOrient");
  GLADE_HOOKUP_OBJECT (dlgDetGeo, etDetPhi, "etDetPhi");
  GLADE_HOOKUP_OBJECT (dlgDetGeo, etDetTilt, "etDetTilt");
  GLADE_HOOKUP_OBJECT (dlgDetGeo, etBeamTiltX, "etBeamTiltX");
  GLADE_HOOKUP_OBJECT (dlgDetGeo, etBeamTiltY, "etBeamTiltY");
  GLADE_HOOKUP_OBJECT_NO_REF (dlgDetGeo, dialog_action_area1, "dialog_action_area1");
  GLADE_HOOKUP_OBJECT (dlgDetGeo, okbutton1, "okbutton1");

  return dlgDetGeo;
}
